/******************************************************************************

                        D S _ U T I L . H

******************************************************************************/

/******************************************************************************

  @file    ds_util.h
  @brief   Data Services Utility Functions Header File

  DESCRIPTION
  Header file for DS utility functions.

  ---------------------------------------------------------------------------
  Copyright (c) 2008 QUALCOMM Incorporated.
  All Rights Reserved. QUALCOMM Proprietary and Confidential.
  ---------------------------------------------------------------------------

******************************************************************************/

/******************************************************************************

                      EDIT HISTORY FOR FILE

  $Id: //release/linux-enablement/02.00.08.010.041/pkgs/proprietary/data/main/source/dss/inc/ds_util.h#2 $

when       who        what, where, why
--------   ---        -------------------------------------------------------
04/04/08   vk         Added support for client control of logging in dss
04/01/08   vk         Initial version

******************************************************************************/

#ifndef __DS_UTIL_H__
#define __DS_UTIL_H__

#include <stdio.h>
#include <stdlib.h>

/*===========================================================================
                     GLOBAL DEFINITIONS AND DECLARATIONS
===========================================================================*/

/*--------------------------------------------------------------------------- 
   Some general utility macros.
---------------------------------------------------------------------------*/
#define MIN(a,b)    ((a) < (b) ? (a) : (b))

/*--------------------------------------------------------------------------- 
   Log file pointer. All log messages are written to this file.
   This variable is not meant to be used directly by clients. 
---------------------------------------------------------------------------*/
extern FILE * ds_logFp;

/*--------------------------------------------------------------------------- 
   Definition of 'assert' macro. This is needed as ONCRPC/DSM hijacks path
   to include another file with name assert.h, so that the standard library
   assert is not available to ONCRPC clients.
---------------------------------------------------------------------------*/
/*
#ifndef FEATURE_DS_LINUX_NO_RPC
#ifndef NDEBUG
#define assert(a)                                               \
        if (!(a)) {                                             \
            fprintf(stderr, "%s, %d: assertion (a) failed!",    \
                    __FILE__,                                   \
                    __LINE__);\
            abort();                                            \
        }
#else
#define assert(a) if (!(a));
#endif
#endif
*/
#define ds_assert(a)                                            \
        if (!(a)) {                                             \
            fprintf(stderr, "%s, %d: assertion (a) failed!",    \
                    __FILE__,                                   \
                    __LINE__);\
            abort();                                            \
        }

/*--------------------------------------------------------------------------- 
   Definition of logging macro
---------------------------------------------------------------------------*/
#ifndef NDEBUG
#define ds_log(...) fprintf(ds_logFp, __VA_ARGS__)
#else
#define ds_log(...) 
#endif

/*--------------------------------------------------------------------------- 
   Definition of function entry/exit logging macros
---------------------------------------------------------------------------*/
#define ds_log_func_entry() ds_log("Entering function %s\n", __FUNCTION__);
#define ds_log_func_exit()  ds_log("Exiting function %s\n", __FUNCTION__);

/*--------------------------------------------------------------------------- 
   Macro to get log file ptr 
---------------------------------------------------------------------------*/
#define ds_get_logfp() (ds_logFp)

/*===========================================================================
                            GLOBAL FUNCTION DEFINITIONS
===========================================================================*/

/*===========================================================================
  FUNCTION  ds_log_init
===========================================================================*/
/*!
@brief
  Initializes logging to use the specified log file.

@return
  void

@note

  - Dependencies
    - None

  - Side Effects
    - None
*/
/*=========================================================================*/
void ds_log_init(FILE * logfp);

#endif /* __DS_UTIL_H__ */
