/*===========================================================================
Edit History of Huawei: 
time          author      Problem form No.     desc
===========================================================================*/
#ifndef __ATP_MSG_API_H__
#define __ATP_MSG_API_H__

#include "atptypes.h"

#define ATP_MSG_CATEGORY_DEF(x) ((x) << 16)

#define ATP_MSG_RECV_TIMEOUT_D (1000 * 5) // 5 seconds

#define ATP_MONITOR_SERVER_PORT_D (8011)

/* Detailed msg types */
typedef enum tagATP_MSG_TYPE_EN
{
    ATP_MSG_UNKNOWN,
    /*
     *    Universal config msg
     */
    ATP_MSG_CONFIG_START = ATP_MSG_CATEGORY_DEF(ATP_MODULE_CFM),
    ATP_MSG_CONFIG_SET = ATP_MSG_CONFIG_START,
    ATP_MSG_CONFIG_GET,
    ATP_MSG_CONFIG_ADD,
    ATP_MSG_CONFIG_DEL,
    ATP_MSG_CONFIG_GET_INST,
    ATP_MSG_VALUE_CHG,
    /*
     *    System wide msg
     */
    ATP_MSG_SYS_START = ATP_MSG_CATEGORY_DEF(ATP_MODULE_SYSTEM),
    ATP_MSG_REGISTER = ATP_MSG_SYS_START,
    ATP_MSG_REBOOT,
    ATP_MSG_RESTORE_DFT,
    ATP_MSG_WAN_PDP_UP,
    ATP_MSG_WAN_PDP_DOWN,
    ATP_MSG_GPIO_WAKEUP,
    ATP_MSG_PID_EXIT,
    ATP_USB_PWR_RESET,
    ATP_SDCARD_ATTACH,
    ATP_SDCARD_REMOVE,
    ATP_MSG_PRO_RESTART,
    ATP_MSG_LAN_WLAN_UP,
    ATP_MSG_LAN_WLAN_DOWN,
    ATP_MSG_LAN_USB_UP,
    ATP_MSG_LAN_USB_DOWN,
    ATP_MSG_NDIS_UP,
    ATP_MSG_NDIS_DOWN,
    /*QMI Server start EVENT up*/
    ATP_MSG_QMI_SERVER_START,	//wenp added 17
    ATP_MSG_QMI_SERVER_STOP,
    ATP_EXTERNAL_CHARGER_ATTACH,
    ATP_EXTERNAL_CHARGER_REMOVE,
    ATP_MSG_DHCP_CLIENT_UP,
    ATP_MSG_PC_DIAL_UP,
    ATP_MSG_PC_DIAL_DOWN,
    ATP_MSG_DIALING_ACTIVE,
    ATP_MSG_DS_CONNECTED,
    ATP_MSG_DISCONNECTING,
    ATP_MSG_DS_DISCONNECTED,
    ATP_MSG_DS_FAILED,
    ATP_MSG_WAN_CHG,
    ATP_MSG_ROAMING_ON,
    ATP_MSG_ROAMING_OFF,
    ATP_MSG_WIFI_USER_DISCONN,/* BB5D02667,xKF17544 2009/11/6 */
    ATP_INS_CPIN_READY,
    ATP_INS_CPIN_PUK,
    ATP_INS_SIM_LOCK,
    ATP_MSG_ROAM_MANUAL,
    /*
     *    CWMP msg
     */
    ATP_MSG_CWMP_START = ATP_MSG_CATEGORY_DEF(ATP_MODULE_CWMP),
    ATP_MSG_CWMP_DIAG_COMPLETE,
    ATP_MSG_CWMP_TRANS_COMPLETE,
    ATP_MSG_CWMP_OPTION43,
#ifdef FEATURE_FLUX_COLLECT
    ATP_MSG_NET_TIME, //ȡʱ
    ATP_MSG_CLEAR_FLUX,//webҳ
    ATP_MSG_CLEAR_FLUX_WEB,//Զʱ֪ͨweb
#endif
    ATP_MSG_REMOUNT_SD ,
    // please keep this one the last
    ATP_MSG_TYPE_NUM,
} ATP_MSG_TYPE_EN;

// This array global variable is used to record whether the events list in
// ATP_MSG_TYPE_EN happened or not. Some items need this information,
// and some others don't. Just put all the information together to make the
// code clear
extern char event_has_happened[ATP_MSG_TYPE_NUM];

typedef enum tagATP_ReStart_TYPE_EN
{
    ATP_RESTART_FIREWALL = 0,
    ATP_RESTART_WANPING,
    ATP_RESTART_IPFLT,
    ATP_RESTART_DMZ,
    ATP_RESTART_PORTMAPPING,
    ATP_RESTART_PT,
    ATP_RESTART_TELNET,
    ATP_RESTART_DHCPS,
    ATP_RESTART_WIFI,
    ATP_RESTART_DIAL,
    ATP_RESTART_UPNP,
    ATP_WIFI_PIN_RESET,
    ATP_MSG_WPS_TIMEOUT,
    ATP_MSG_WPS_FAIL,
    ATP_WIFI_MAC_FTL_CTL,
    ATP_WIFI_MAC_FTL_S,
    /*Other*/
    ATP_RESTART_SIP,
    ATP_RESTART_OTHER
} ATP_MSG_DATA_RESTART_EN;

#define ATP_MSG_MAX_NAME_LEN (15)

/*Message Boby*/
typedef     struct tagATP_MSG_DATA_ST
{
    VOS_UINT32 ulReType;                              /* ҪʱЧǸģ:ATP_MSG_DATA_RESTART_EN*/
    VOS_CHAR   acName[ATP_MSG_MAX_NAME_LEN + 1];     /* 쳣˳*/
} ATP_MSG_DATA_ST;

/* Message Header */
typedef struct tagATP_MSG_HEADER_ST
{
    VOS_UINT32 ulMsgType;
    VOS_UINT32 usSeqNum;

    VOS_CHAR aucSendApp[ATP_MSG_MAX_NAME_LEN + 1];    /* ͽ */
    VOS_CHAR aucRecvApp[ATP_MSG_MAX_NAME_LEN + 1];    /* ս */

    //VOS_UINT32  ulMsgData;

    ATP_MSG_DATA_ST stMsgData;

    VOS_UINT32 ulDataLen;
} ATP_MSG_HEADER_ST;

typedef enum tagATP_MSG_RET_CODE_EN
{
    ATP_MSG_RET_SOCKET_ERR = ATP_MNT_ERR_CODE_DEF(ATP_TEAM_MNT, ATP_MODULE_MSG),
    ATP_MSG_RET_PARA_ERR,
    ATP_MSG_RET_APPNAME_ERR,
    ATP_MSG_RET_NOT_INITED,
    ATP_MSG_RET_PEER_DEAD,
    ATP_MSG_RET_TIMEOUT,
    ATP_MSG_RET_MEM_ERR,
    ATP_MSG_RET_INTERNAL_ERR
} ATP_MSG_RET_CODE_EN;

typedef VOS_VOID (*AtpMsgProc)(const ATP_MSG_HEADER_ST *pstMsg);

typedef struct tagATP_MSG_QUEUE_ST
{
    ATP_MSG_HEADER_ST           *pstMsg;
    struct tagATP_MSG_QUEUE_ST   *pstNext;
} ATP_MSG_QUEUE_ST;

typedef struct tagATP_MSG_CONTEXT_ST
{
    VOS_CHAR             aucAppName[ATP_MSG_MAX_NAME_LEN + 1];
    VOS_INT32            lSock;
    VOS_BOOL             bExitLoop;
    AtpMsgProc           pfMsgProc;
    ATP_MSG_QUEUE_ST    *pstMsgQueue;   // Internal usage
} ATP_MSG_CONTEXT_ST;

extern ATP_MSG_CONTEXT_ST g_stMsgCtx;

VOS_UINT32                ATP_MSG_QueueAppend( ATP_MSG_QUEUE_ST        **ppstMsgQueue,
                                               ATP_MSG_HEADER_ST       *                pstMsg);

VOS_VOID                  ATP_MSG_QueueClear(ATP_MSG_QUEUE_ST    *pstMsgQueue);

VOS_UINT32                ATP_MSG_Init(const VOS_CHAR *pszAppName);

VOS_VOID                  ATP_MSG_Destroy();

//VOS_UINT32 ATP_MSG_GetAppName(VOS_CHAR *pszName);

/*
 *    Message Dispatch Interfaces
 */

VOS_VOID                  ATP_MSG_RegisterMsgProc(AtpMsgProc pFunc);

VOS_UINT32                ATP_MSG_GetAndDispatch();

//VOS_UINT32 VTOP_MSG_RegisterTrace(PFUNC_MSGHandler pFunc);

//VOS_UINT32 VTOP_MSG_UnRegTrace();

/*
 *    Message Sending & Recving Interfaces
 */

VOS_UINT32                ATP_MSG_Send(const ATP_MSG_HEADER_ST *pstMsg);

VOS_UINT32                ATP_MSG_SendAndRecv(const ATP_MSG_HEADER_ST  *pstMsg,
                                              ATP_MSG_HEADER_ST        **              ppstResp,
                                              VOS_UINT32                               ulTimeout);

VOS_UINT32                ATP_MSG_Recv(VOS_INT32 sSock,
                                       ATP_MSG_HEADER_ST   **   ppstMsg,
                                       VOS_UINT32               ulTimeout);

/*
 *    Message Utilities
 */

VOS_UINT32                ATP_MSG_RegEvent(VOS_UINT32 ulEvtType);

extern VOS_VOID*          ATP_MSG_GetMsgBlock(VOS_UINT32 ulMsgSize);
extern VOS_VOID*          ATP_MSG_GetMsgByHead(ATP_MSG_HEADER_ST* pstMsgHead);
extern ATP_MSG_HEADER_ST* ATP_MSG_GetMsgHead(VOS_VOID* pvMsg);

/*
 *    Message Structure Utilities
 */
VOS_UINT32                ATP_MSG_QueueMsg(const ATP_MSG_HEADER_ST *pstRecvMsg);
VOS_UINT32                ATP_MSG_RequeueMsgs();
VOS_UINT32                ATP_MSG_DupMsg(const ATP_MSG_HEADER_ST *pstRecvMsg);

#ifdef ATP_DEBUG
#undef ATP_DEBUG
#endif

#ifdef ATP_DEBUG
 #define ATP_MSG_DEBUG(format, args...) {printf("FILE: %s LINE: %d: ", __FILE__, __LINE__); printf(format, ## args);}
#else
 #define ATP_MSG_DEBUG(x...)
#endif

#endif
