/* linux/arch/arm/mach-msm/board-halibut.c
 *
 * Copyright (C) 2007 Google, Inc.
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * Author: Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/usb/mass_storage_function.h>
#include <linux/input.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>
#include <asm/mach/mmc.h>

#include <asm/arch/board.h>
#include <asm/arch/msm_iomap.h>
#include <asm/arch/msm_fb.h>
#include <linux/device.h> /* for vreg.h */
#include <asm/arch/vreg.h>
#include <asm/arch/mpp.h>
#include <asm/arch/gpio.h>
#include <asm/arch/system.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <linux/delay.h>
#include <linux/bootmem.h>

#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#include <asm/arch/msm_rpcrouter.h>

#include "clock.h"
#include <linux/i2c.h>
#include <asm/arch/gpio.h>
#include "proc_comm.h"
#include "socinfo.h"
#include "../../../drivers/usb/function/msm_hsusb.h"

#ifdef CONFIG_STACKED_MEMORY
#define MSM_SMI_BASE		0x100000
#define MSM_SMI_SIZE		0x800000

#define MSM_PMEM_GPU0_BASE	MSM_SMI_BASE
#define MSM_PMEM_GPU0_SIZE	0x800000
#endif

/* XXX These sizes might not be hardcoded in the future, but
 * instead be obtained via tags or the kernel command line.
 */
#define MSM_PMEM_MDP_SIZE	0x800000
#define MSM_PMEM_ADSP_SIZE	0x800000
#define MSM_PMEM_GPU1_SIZE	0x800000
#define MSM_FB_SIZE		    0x800000

static int halibut_ffa;
module_param_named(ffa, halibut_ffa, int, S_IRUGO | S_IWUSR | S_IWGRP);

void halibut_init_keypad(int ffa);

static struct resource smc91x_resources[] = {
	[0] = {
		.start	= 0x9C004300,
		.end	= 0x9C004400,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= MSM_GPIO_TO_INT(49),
		.end	= MSM_GPIO_TO_INT(49),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device smc91x_device = {
	.name		= "smc91x",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(smc91x_resources),
	.resource	= smc91x_resources,
};

static struct usb_function_map usb_functions_map[] = {
	{"diag", 0},
	{"adb", 1},
	{"modem", 2},
	{"nmea", 3},
	{"mass_storage", 4},
	{"ethernet", 5},
};

/* dynamic composition */
static struct usb_composition usb_func_composition[] = {
	{
		.product_id         = 0x9012,
		.functions	    = 0x5, /* 0101 */
	},

	{
		.product_id         = 0x9013,
		.functions	    = 0x15, /* 10101 */
	},

	{
		.product_id         = 0x9014,
		.functions	    = 0x30, /* 110000 */
	},

	{
		.product_id         = 0x9016,
		.functions	    = 0xD, /* 01101 */
	},

	{
		.product_id         = 0x9017,
		.functions	    = 0x1D, /* 11101 */
	},

	{
		.product_id         = 0xF000,
		.functions	    = 0x10, /* 10000 */
	},

	{
		.product_id         = 0xF009,
		.functions	    = 0x20, /* 100000 */
	},

	{
		.product_id         = 0x9018,
		.functions	    = 0x1F, /* 011111 */
	},

};

static unsigned usb_gpio_lpm_config[] = {
	GPIO_CFG(111, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 0 */
	GPIO_CFG(112, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 1 */
	GPIO_CFG(113, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 2 */
	GPIO_CFG(114, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 3 */
	GPIO_CFG(115, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 4 */
	GPIO_CFG(116, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 5 */
	GPIO_CFG(117, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 6 */
	GPIO_CFG(118, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DATA 7 */
	GPIO_CFG(119, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* DIR */
	GPIO_CFG(120, 1, GPIO_INPUT, GPIO_NO_PULL, GPIO_2MA),	/* NEXT */
	GPIO_CFG(121, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* STOP */
};

static unsigned usb_gpio_lpm_unconfig[] = {
	GPIO_CFG(111, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 0 */
	GPIO_CFG(112, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 1 */
	GPIO_CFG(113, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 2 */
	GPIO_CFG(114, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 3 */
	GPIO_CFG(115, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 4 */
	GPIO_CFG(116, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 5 */
	GPIO_CFG(117, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 6 */
	GPIO_CFG(118, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DATA 7 */
	GPIO_CFG(119, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DIR */
	GPIO_CFG(120, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* NEXT */
	GPIO_CFG(121, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_2MA), /* STOP */
};

static int usb_config_gpio(int config)
{
	int pin, rc;

	if (config) {
		for (pin = 0; pin < ARRAY_SIZE(usb_gpio_lpm_config); pin++) {
			rc = gpio_tlmm_config(usb_gpio_lpm_config[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, usb_gpio_lpm_config[pin], rc);
				return -EIO;
			}
		}
	} else {
		for (pin = 0; pin < ARRAY_SIZE(usb_gpio_lpm_unconfig); pin++) {
			rc = gpio_tlmm_config(usb_gpio_lpm_unconfig[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, usb_gpio_lpm_config[pin], rc);
				return -EIO;
			}
		}
	}

	return 0;
}

static struct msm_hsusb_platform_data msm_hsusb_pdata = {
	.version	= 0x0100,
	.phy_info	= USB_PHY_EXTERNAL,
	.vendor_id          = 0x5c6,
	.product_name       = "Qualcomm HSUSB Device",
	.serial_number      = "1234567890ABCDEF",
	.manufacturer_name  = "Qualcomm Incorporated",
	.compositions	= usb_func_composition,
	.num_compositions = ARRAY_SIZE(usb_func_composition),
	.function_map   = usb_functions_map,
	.num_functions	= ARRAY_SIZE(usb_functions_map),
	.ulpi_data_1_pin = 112,
	.ulpi_data_3_pin = 114,
	.config_gpio 	= usb_config_gpio,
};

static char *msm_fb_vreg[] = {
	"gp5"
};

#define MSM_FB_VREG_OP(name, op)					\
	do {vreg = vreg_get(0, name);					\
	if (vreg_##op(vreg))						\
		printk(KERN_ERR "%s: %s vreg operation failed \n",	\
		(vreg_##op == vreg_enable)?"vreg_enable":"vreg_disable",\
		name); } while (0)

static void msm_fb_mddi_power_save(int on)
{
	struct vreg *vreg;
	int i;

	for (i = 0; i < ARRAY_SIZE(msm_fb_vreg); i++) {
		if (on)
			MSM_FB_VREG_OP(msm_fb_vreg[i], enable);
		else
			MSM_FB_VREG_OP(msm_fb_vreg[i], disable);
	}
}

#define PM_VID_EN_CONFIG_PROC          24
#define PM_VID_EN_API_PROG             0x30000061
#define PM_VID_EN_API_VERS             0x00010001

static struct msm_rpc_endpoint *pm_vid_en_ep;

int msm_fb_pm_vid_en(int on)
{
	int rc = 0;
	struct msm_fb_pm_vid_en_req {
		struct rpc_request_hdr hdr;
		uint32_t on;
	} req;

	pm_vid_en_ep = msm_rpc_connect(PM_VID_EN_API_PROG,
					PM_VID_EN_API_VERS, 0);
	if (IS_ERR(pm_vid_en_ep)) {
		printk(KERN_ERR "%s: msm_rpc_connect failed! rc = %ld\n",
			__func__, PTR_ERR(pm_vid_en_ep));
		return -EINVAL;
	}

	req.on = cpu_to_be32(on);
	rc = msm_rpc_call(pm_vid_en_ep,
			PM_VID_EN_CONFIG_PROC,
			&req, sizeof(req),
			5 * HZ);
	if (rc)
		printk(KERN_ERR
			"%s: msm_rpc_call failed! rc = %d\n", __func__, rc);

	msm_rpc_close(pm_vid_en_ep);
	return rc;
}

static int mddi_toshiba_backlight_level(int level)
{
	int out_val;

	if (machine_is_msm7201a_ffa() || machine_is_qst1105_ffa()) {
		switch (level) {
		case 0:
			out_val = 0x00001387;
			break;
		case 1:
			out_val = 3700;
			break;
		case 2:
			out_val = 3200;
			break;
		case 3:
			out_val = 2700;
			break;
		case 4:
			out_val = 2200;
			break;
		default:
			out_val = 0x0000;
			break;
		}
	} else {
		switch (level) {
		case 0:
			out_val = 0x0000;
			break;
		case 1:
			out_val = 1250;
			break;
		case 2:
			out_val = 2500;
			break;
		case 3:
			out_val = 3750;
			break;
		case 4:
			out_val = 4999;
			break;
		default:
			out_val = 0x00001387;
			break;
		}
	}

	return out_val;
}

static int mddi_sharp_backlight_level(int level)
{
	if (machine_is_msm7201a_ffa() || machine_is_qst1105_ffa())
		return level;
	else
		return -1;
}

static struct tvenc_platform_data tvenc_pdata = {
	.pm_vid_en = msm_fb_pm_vid_en,
};

static struct mddi_platform_data mddi_pdata = {
	.mddi_power_save = msm_fb_mddi_power_save,
};

static struct msm_panel_common_pdata mddi_toshiba_pdata = {
	.backlight_level = mddi_toshiba_backlight_level,
};

static struct msm_panel_common_pdata mddi_sharp_pdata = {
	.backlight_level = mddi_sharp_backlight_level,
};

static struct resource msm_fb_resources[] = {
	{
		.flags  = IORESOURCE_DMA,
	}
};

static struct platform_device msm_fb_device = {
	.name   = "msm_fb",
	.id     = 0,
	.num_resources  = ARRAY_SIZE(msm_fb_resources),
	.resource       = msm_fb_resources,
};

static struct platform_device mddi_toshiba_device = {
	.name   = "mddi_toshiba_vga",
	.id     = 0,
	.dev    = {
		.platform_data = &mddi_toshiba_pdata,
	}
};

static struct platform_device mddi_sharp_device = {
	.name   = "mddi_sharp_qvga",
	.id     = 0,
	.dev    = {
		.platform_data = &mddi_sharp_pdata,
	}
};

static struct resource msm_serial0_resources[] = {
	{
		.start	= INT_UART3,
		.end	= INT_UART3,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= MSM_UART3_PHYS,
		.end	= MSM_UART3_PHYS + MSM_UART3_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct platform_device msm_serial0_device = {
	.name	= "msm_serial",
	.id	= 2,
	.num_resources	= ARRAY_SIZE(msm_serial0_resources),
	.resource	= msm_serial0_resources,
};

static struct resource usb_resources[] = {
	{
		.start	= MSM_HSUSB_PHYS,
		.end	= MSM_HSUSB_PHYS + MSM_HSUSB_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= INT_USB_HS,
		.end	= INT_USB_HS,
		.flags	= IORESOURCE_IRQ,
	},
};

#ifndef FEATURE_E5832
/* dynamic composition */
static struct usb_composition usb_func_composition[] = {
	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9012,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 2,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9013,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 3,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"mass_storage",  2,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9014,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 2,
		.funcs[0]           = {"mass_storage",  0,       1},
		.funcs[1]           = {"ethernet",         1, 2},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9016,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 3,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"nmea",          2,  1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9017,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 4,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"nmea",          2,  1},
		.funcs[3]           = {"mass_storage",  3,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0xF000,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 1,
		.funcs[0]           = {"mass_storage",  0,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0xF009,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 1,
		.funcs[0]           = {"ethernet",         0, 2},
	},

	{
		.vendor_id          = 0x18d1,
		.product_id         = 0xD00D,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 5,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"adb",           1,  1},
		.funcs[2]           = {"modem",         2,  1},
		.funcs[3]           = {"nmea",          3,  1},
		.funcs[4]           = {"mass_storage",  4,  1},
	},

};
#endif

static struct usb_mass_storage_platform_data usb_mass_storage_pdata = {
	.nluns          = 0x02,
	.buf_size       = 16384,
	.vendor         = "GOOGLE",
	.product        = "Mass storage",
	.release        = 0xffff,
};

static struct platform_device mass_storage_device = {
	.name           = "mass_storage",
	.id             = -1,
	.dev            = {
	.platform_data = &usb_mass_storage_pdata,
	},
};

#ifndef FEATURE_E5832
static struct msm_hsusb_platform_data msm_hsusb_pdata = {
	.version	= 0x0100,
	.phy_info	= USB_PHY_EXTERNAL,
	.compositions	= usb_func_composition,
	.num_compositions = ARRAY_SIZE(usb_func_composition),
};
#endif

static u64 dma_mask = 0xffffffffULL;
static struct platform_device msm_hsusb_device = {
	.name		= "msm_hsusb",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(usb_resources),
	.resource	= usb_resources,
	.dev		= {
		.dma_mask 		= &dma_mask,
		.coherent_dma_mask	= 0xffffffffULL,
		.platform_data		= &msm_hsusb_pdata,
	},
};

static struct android_pmem_platform_data android_pmem_pdata = {
	.name = "pmem",
	.no_allocator = 1,
	.cached = 1,
};

static struct android_pmem_platform_data android_pmem_adsp_pdata = {
	.name = "pmem_adsp",
	.no_allocator = 0,
	.cached = 0,
};

#ifdef CONFIG_STACKED_MEMORY
static struct android_pmem_platform_data android_pmem_gpu0_pdata = {
	.name = "pmem_gpu0",
	.start = MSM_PMEM_GPU0_BASE,
	.size = MSM_PMEM_GPU0_SIZE,
	.no_allocator = 1,
	.cached = 0,
};
#endif

static struct android_pmem_platform_data android_pmem_gpu1_pdata = {
	.name = "pmem_gpu1",
	.no_allocator = 1,
	.cached = 0,
};

static struct platform_device android_pmem_device = {
	.name = "android_pmem",
	.id = 0,
	.dev = { .platform_data = &android_pmem_pdata },
};

static struct platform_device android_pmem_adsp_device = {
	.name = "android_pmem",
	.id = 1,
	.dev = { .platform_data = &android_pmem_adsp_pdata },
};

#ifdef CONFIG_STACKED_MEMORY
static struct platform_device android_pmem_gpu0_device = {
	.name = "android_pmem",
	.id = 2,
	.dev = { .platform_data = &android_pmem_gpu0_pdata },
};
#endif

static struct platform_device android_pmem_gpu1_device = {
	.name = "android_pmem",
	.id = 3,
	.dev = { .platform_data = &android_pmem_gpu1_pdata },
};

#ifdef CONFIG_BT
static struct platform_device msm_bt_power_device = {
	.name = "bt_power",
};

enum {
	BT_WAKE,
	BT_RFR,
	BT_CTS,
	BT_RX,
	BT_TX,
	BT_PCM_DOUT,
	BT_PCM_DIN,
	BT_PCM_SYNC,
	BT_PCM_CLK,
	BT_HOST_WAKE,
};

static unsigned bt_config_power_on[] = {
	GPIO_CFG(42, 0, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* WAKE */
	GPIO_CFG(43, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* RFR */
	GPIO_CFG(44, 2, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* CTS */
	GPIO_CFG(45, 2, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* Rx */
	GPIO_CFG(46, 3, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* Tx */
	GPIO_CFG(68, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_DOUT */
	GPIO_CFG(69, 1, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* PCM_DIN */
	GPIO_CFG(70, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_SYNC */
	GPIO_CFG(71, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_CLK */
	GPIO_CFG(83, 0, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* HOST_WAKE */
};
static unsigned bt_config_power_off[] = {
	GPIO_CFG(42, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* WAKE */
	GPIO_CFG(43, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* RFR */
	GPIO_CFG(44, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* CTS */
	GPIO_CFG(45, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* Rx */
	GPIO_CFG(46, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* Tx */
	GPIO_CFG(68, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_DOUT */
	GPIO_CFG(69, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_DIN */
	GPIO_CFG(70, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_SYNC */
	GPIO_CFG(71, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_CLK */
	GPIO_CFG(83, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* HOST_WAKE */
};

static int bluetooth_power(int on)
{
	struct vreg *vreg_bt;
	int pin, rc;

	printk(KERN_DEBUG "%s\n", __func__);

	/* do not have vreg bt defined, gp6 is the same */
	/* vreg_get parameter 1 (struct device *) is ignored */
	vreg_bt = vreg_get(NULL, "gp6");
	if (IS_ERR(vreg_bt)) {
		printk(KERN_ERR "%s: vreg get failed (%ld)\n",
		       __func__, PTR_ERR(vreg_bt));
		return -EIO;
	}

	if (on) {
		for (pin = 0; pin < ARRAY_SIZE(bt_config_power_on); pin++) {
			rc = gpio_tlmm_config(bt_config_power_on[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, bt_config_power_on[pin], rc);
				return -EIO;
			}
		}

		/* units of mV, steps of 50 mV */
		rc = vreg_set_level(vreg_bt, 2600);
		if (rc) {
			printk(KERN_ERR "%s: vreg set level failed (%d)\n",
			       __func__, rc);
			return -EIO;
		}
		rc = vreg_enable(vreg_bt);
		if (rc) {
			printk(KERN_ERR "%s: vreg enable failed (%d)\n",
			       __func__, rc);
			return -EIO;
		}
	} else {
		rc = vreg_disable(vreg_bt);
		if (rc) {
			printk(KERN_ERR "%s: vreg disable failed (%d)\n",
			       __func__, rc);
			return -EIO;
		}
		for (pin = 0; pin < ARRAY_SIZE(bt_config_power_off); pin++) {
			rc = gpio_tlmm_config(bt_config_power_off[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, bt_config_power_off[pin], rc);
				return -EIO;
			}
		}
	}
	return 0;
}

static void __init bt_power_init(void)
{
	msm_bt_power_device.dev.platform_data = &bluetooth_power;
}
#else
#define bt_power_init(x) do {} while (0)
#endif

static struct resource bluesleep_resources[] = {
	{
		.name	= "gpio_host_wake",
		.start	= 83,
		.end	= 83,
		.flags	= IORESOURCE_IO,
	},
	{
		.name	= "gpio_ext_wake",
		.start	= 42,
		.end	= 42,
		.flags	= IORESOURCE_IO,
	},
	{
		.name	= "host_wake",
		.start	= MSM_GPIO_TO_INT(83),
		.end	= MSM_GPIO_TO_INT(83),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device msm_bluesleep_device = {
	.name = "bluesleep",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(bluesleep_resources),
	.resource	= bluesleep_resources,
};

static struct msm_camera_device_platform_data msm_camera_device_data[2] = {
	{
	.sensor_reset	= 89,
	.sensor_pwd	  = 85,
	.vcm_pwd      = 0,
	.sensor_name  = "mt9d112",
	},
	{
	.sensor_reset	= 89,
	.sensor_pwd	  = 85,
	.vcm_pwd      = 0,
	.sensor_name  = "mt9t013",
	},
};

static struct platform_device msm_camera_device = {
	.name	= "msm_camera",
	.id		= 0,
	.dev	= {
		.platform_data = &msm_camera_device_data[0],
	},
};


static struct platform_device *devices[] __initdata = {
#if !defined(CONFIG_MSM_SERIAL_DEBUGGER)
	&msm_serial0_device,
#endif
	&msm_fb_device,
	&mddi_toshiba_device,
	&mddi_sharp_device,
	&msm_hsusb_device,
	&mass_storage_device,
	&smc91x_device,
	&android_pmem_device,
	&android_pmem_adsp_device,
#ifdef CONFIG_STACKED_MEMORY
	&android_pmem_gpu0_device,
#endif
	&android_pmem_gpu1_device,
#ifdef CONFIG_BT
	&msm_bt_power_device,
#endif
	&msm_bluesleep_device,
	&msm_camera_device,
};

/*
 * All halibut slots are the same(ish)
 */

#ifdef HAVE_SDCC_SLOT_INSERT_DETECTION
static unsigned int halibut_sdcc_slot_status(struct device *dev)
{
	/*
	 * TODO: Hook this up to the FPGA for reading slot status
	 * For now return 0.
	 */
	return 0;
}
#endif

static unsigned int vreg_enabled;
static unsigned sdcc_cfg_data[][6] = {
	/* SDC1 configs */
	{
	GPIO_CFG(51, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(52, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(53, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(54, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(55, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(56, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	},
	/* SDC2 configs */
	{
	GPIO_CFG(62, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	GPIO_CFG(63, 2, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(64, 2, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(65, 2, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(66, 2, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(67, 2, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	},
	/* SDC3 configs */
	{
	GPIO_CFG(88, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	GPIO_CFG(89, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(90, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(91, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(92, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(93, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	},
	/* SDC4 configs */
	{
	GPIO_CFG(19, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(20, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(21, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(107, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(108, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(109, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	}
};

static int msm_sdcc_setup_gpio(int dev_id, unsigned enable)
{
	int i, rc;
	for (i = 0; i < ARRAY_SIZE(sdcc_cfg_data[dev_id - 1]); i++) {
		rc = gpio_tlmm_config(sdcc_cfg_data[dev_id - 1][i],
			enable ? GPIO_ENABLE : GPIO_DISABLE);
		if (rc) {
			printk(KERN_ERR "%s: gpio_tlmm_config(%#x)=%d\n",
				__func__, sdcc_cfg_data[dev_id - 1][i], rc);
		}
	}
	return rc;
}

static int msm_sdcc_setup_power(int dev_id, int on)
{
	int rc = 0;
	struct mpp *mpp_mmc;
	struct vreg *vreg_mmc;

	rc = msm_sdcc_setup_gpio(dev_id, on);
	if (rc)
		return -EIO;

	if ((on && vreg_enabled) || (!on && !vreg_enabled))
		return 0;

	if (machine_is_msm7201a_ffa() || machine_is_qst1105_ffa()) {
		mpp_mmc = mpp_get(NULL, "mpp3");
		if (!mpp_mmc) {
			printk(KERN_ERR "%s: mpp get failed\n", __func__);
			return -EIO;
		}

		rc = mpp_config_digital_out(mpp_mmc,
		     MPP_CFG(MPP_DLOGIC_LVL_MSMP,
		     on ? MPP_DLOGIC_OUT_CTRL_HIGH : MPP_DLOGIC_OUT_CTRL_LOW));
		if (rc) {
			printk(KERN_ERR
				"%s: Failed to configure mpp (%d)\n",
				__func__, rc);
			return rc;
		}
	} else {
		vreg_mmc = vreg_get(NULL, "mmc");
		if (IS_ERR(vreg_mmc)) {
			printk(KERN_ERR "%s: vreg get failed (%ld)\n",
			       __func__, PTR_ERR(vreg_mmc));
			return PTR_ERR(vreg_mmc);
		}

		rc = on ? vreg_enable(vreg_mmc) : vreg_disable(vreg_mmc);
		if (rc) {
			printk(KERN_ERR "%s: Failed to configure vreg (%d)\n",
				__func__, rc);
			return rc;
		}
	}
	vreg_enabled = on;
	return 0;
}

static struct mmc_platform_data halibut_sdcc_data = {
	.ocr_mask	= MMC_VDD_28_29,
#ifdef HAVE_SDCC_SLOT_INSERT_DETECTION
	.status		= halibut_sdcc_slot_status,
#endif
	.setup_power	= msm_sdcc_setup_power,
};

extern struct sys_timer msm_timer;

static void __init halibut_init_irq(void)
{
	msm_init_irq();
}

static struct clk halibut_clocks[] = {
	CLOCK("acpu_clk",	ACPU_CLK,	0),
	CLOCK("adm_clk",	ADM_CLK,	0),
	CLOCK("adsp_clk",	ADSP_CLK,	0),
	CLOCK("ebi1_clk",	EBI1_CLK,	OFF | MIN_SET | NO_SET),
	CLOCK("ebi2_clk",	EBI2_CLK,	0),
	CLOCK("ecodec_clk",	ECODEC_CLK,	0),
	CLOCK("emdh_clk",	EMDH_CLK,	OFF | MIN_MAX | NO_SET),
	CLOCK("gp_clk",		GP_CLK,		0),
	CLOCK("grp_clk",	GRP_CLK,	OFF),
	CLOCK("i2c_clk",	I2C_CLK,	0),
	CLOCK("icodec_rx_clk",	ICODEC_RX_CLK,	0),
	CLOCK("icodec_tx_clk",	ICODEC_TX_CLK,	0),
	CLOCK("imem_clk",	IMEM_CLK,	OFF),
	CLOCK("mdc_clk",	MDC_CLK,	0),
	CLOCK("mdp_clk",	MDP_CLK,	OFF),
	CLOCK("pbus_clk",	PBUS_CLK,	0),
	CLOCK("pcm_clk",	PCM_CLK,	0),
	CLOCK("pmdh_clk",	PMDH_CLK,	OFF | MIN_MAX | NO_SET),
	CLOCK("sdac_clk",	SDAC_CLK,	OFF),
	CLOCK("sdc1_clk",	SDC1_CLK,	OFF),
	CLOCK("sdc1_pclk",	SDC1_PCLK,	OFF),
	CLOCK("sdc2_clk",	SDC2_CLK,	OFF),
	CLOCK("sdc2_pclk",	SDC2_PCLK,	OFF),
	CLOCK("sdc3_clk",	SDC3_CLK,	OFF),
	CLOCK("sdc3_pclk",	SDC3_PCLK,	OFF),
	CLOCK("sdc4_clk",	SDC4_CLK,	OFF),
	CLOCK("sdc4_pclk",	SDC4_PCLK,	OFF),
	CLOCK("tsif_clk",	TSIF_CLK,	0),
	CLOCK("tsif_ref_clk",	TSIF_REF_CLK,	0),
	CLOCK("tv_dac_clk",	TV_DAC_CLK,	0),
	CLOCK("tv_enc_clk",	TV_ENC_CLK,	0),
	CLOCK("uart1_clk",	UART1_CLK,	OFF),
	CLOCK("uart2_clk",	UART2_CLK,	0),
	CLOCK("uart3_clk",	UART3_CLK,	OFF),
	CLOCK("uart1dm_clk",	UART1DM_CLK,	OFF),
	CLOCK("uart2dm_clk",	UART2DM_CLK,	0),
	CLOCK("usb_hs_clk",	USB_HS_CLK,	OFF),
	CLOCK("usb_hs_pclk",	USB_HS_PCLK,	OFF),
	CLOCK("usb_otg_clk",	USB_OTG_CLK,	0),
	CLOCK("vdc_clk",	VDC_CLK,	OFF | MIN_SET | NO_SET),
	CLOCK("vfe_clk",	VFE_CLK,	OFF),
	CLOCK("vfe_mdc_clk",	VFE_MDC_CLK,	OFF),
};

static struct msm_clock_platform_data halibut_clock_data = {
	.acpu_switch_time_us = 50,
	.max_speed_delta_khz = 256000,
	.vdd_switch_time_us = 62,
	.active_clocks = halibut_clocks,
	.active_clock_count = ARRAY_SIZE(halibut_clocks),
};

void msm_serial_debug_init(unsigned int base, int irq,
			   const char *clkname, int signal_irq);

static void __init halibut_init_mmc(void)
{
	msm_add_sdcc(1, &halibut_sdcc_data);

	if (machine_is_msm7201a_surf() || machine_is_qst1105_surf()) {
/*<BB5D01580 y68191 20090624 begin */
		//msm_add_sdcc(2, &halibut_sdcc_data);
/*BB5D01580 y68191 20090624 end> */
		msm_add_sdcc(4, &halibut_sdcc_data);
	}
}

static struct i2c_board_info i2c_devices[] = {
	{
		I2C_BOARD_INFO("mt9d112", 0x78 >> 1),
	},
	{
		I2C_BOARD_INFO("mt9t013", 0x6C >> 1),
	},
};

static uint32_t camera_off_gpio_table[] = {
	/* parallel CAMERA interfaces */
	GPIO_CFG(0,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT0 */
	GPIO_CFG(1,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT1 */
	GPIO_CFG(2,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT2 */
	GPIO_CFG(3,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT3 */
	GPIO_CFG(4,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT4 */
	GPIO_CFG(5,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT5 */
	GPIO_CFG(6,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT6 */
	GPIO_CFG(7,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT7 */
	GPIO_CFG(8,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT8 */
	GPIO_CFG(9,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT9 */
	GPIO_CFG(10, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT10 */
	GPIO_CFG(11, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT11 */
	GPIO_CFG(12, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* PCLK */
	GPIO_CFG(13, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* HSYNC_IN */
	GPIO_CFG(14, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* VSYNC_IN */
	GPIO_CFG(15, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* MCLK */
};

static uint32_t camera_on_gpio_table[] = {
	/* parallel CAMERA interfaces */
	GPIO_CFG(0,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT0 */
	GPIO_CFG(1,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT1 */
	GPIO_CFG(2,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT2 */
	GPIO_CFG(3,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT3 */
	GPIO_CFG(4,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT4 */
	GPIO_CFG(5,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT5 */
	GPIO_CFG(6,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT6 */
	GPIO_CFG(7,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT7 */
	GPIO_CFG(8,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT8 */
	GPIO_CFG(9,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT9 */
	GPIO_CFG(10, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT10 */
	GPIO_CFG(11, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT11 */
	GPIO_CFG(12, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_16MA), /* PCLK */
	GPIO_CFG(13, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* HSYNC_IN */
	GPIO_CFG(14, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* VSYNC_IN */
	GPIO_CFG(15, 1, GPIO_OUTPUT, GPIO_PULL_DOWN, GPIO_16MA), /* MCLK */
};

static void config_gpio_table(uint32_t *table, int len)
{
	int n;
	unsigned id;
	for (n = 0; n < len; n++) {
		id = table[n];
		msm_proc_comm(PCOM_RPC_GPIO_TLMM_CONFIG_EX, &id, 0);
	}
}

void config_camera_on_gpios(void)
{
	config_gpio_table(camera_on_gpio_table,
		ARRAY_SIZE(camera_on_gpio_table));
}

void config_camera_off_gpios(void)
{
	config_gpio_table(camera_off_gpio_table,
		ARRAY_SIZE(camera_off_gpio_table));
}

static void __init msm_fb_init(void)
{
	msm_fb_register_device("mdp", 0);
	msm_fb_register_device("ebi2", 0);
	msm_fb_register_device("pmdh", &mddi_pdata);
	msm_fb_register_device("emdh", 0);
	msm_fb_register_device("tvenc", &tvenc_pdata);
}

static void __init halibut_init(void)
{
	socinfo_init();

#if defined(CONFIG_MSM_SERIAL_DEBUGGER)
	msm_serial_debug_init(MSM_UART1_PHYS, INT_UART1,
			      "uart1_clk", 1);
#endif
	if (machine_is_msm7201a_ffa() || machine_is_qst1105_ffa()) {
		smc91x_resources[0].start = 0x98000300;
		smc91x_resources[0].end = 0x98000400;
		smc91x_resources[1].start = MSM_GPIO_TO_INT(85);
		smc91x_resources[1].end = MSM_GPIO_TO_INT(85);
	}

	msm_init_gpio();
	platform_add_devices(devices, ARRAY_SIZE(devices));
	halibut_init_keypad(machine_is_msm7201a_ffa()
			    || machine_is_msm7601a_ffa()
			    || machine_is_qst1105_ffa());
	msm_add_devices();
	i2c_register_board_info(0, i2c_devices, ARRAY_SIZE(i2c_devices));
	config_camera_off_gpios();
	halibut_init_mmc();
	bt_power_init();
	msm_fb_init();
}

static void __init msm_halibut_allocate_memory_regions(void)
{
	void *addr, *addr_1m_aligned;
	unsigned long size;

	size = MSM_PMEM_MDP_SIZE;
	addr = alloc_bootmem(size);
	android_pmem_pdata.start = __pa(addr);
	android_pmem_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for pmem\n", size, addr, __pa(addr));

	size = MSM_PMEM_ADSP_SIZE;
	addr = alloc_bootmem(size);
	android_pmem_adsp_pdata.start = __pa(addr);
	android_pmem_adsp_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for adsp pmem\n", size, addr, __pa(addr));

	/* The GPU1 area must be aligned to a 1M boundary */
	/* XXX For now allocate an extra 1M, use the aligned part, waste the extra memory */
	size = MSM_PMEM_GPU1_SIZE + 0x100000 - PAGE_SIZE;
	addr = alloc_bootmem(size);
	addr_1m_aligned = (void *)(((unsigned int)addr + 0x100000) & 0xfff00000);
	size = MSM_PMEM_GPU1_SIZE;
	android_pmem_gpu1_pdata.start = __pa(addr_1m_aligned);
	android_pmem_gpu1_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for gpu1 pmem\n", size, addr_1m_aligned, __pa(addr_1m_aligned));

	size = MSM_FB_SIZE;
	addr = alloc_bootmem(size);
	msm_fb_resources[0].start = __pa(addr);
	msm_fb_resources[0].end = msm_fb_resources[0].start + size - 1;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for fb\n", size, addr, __pa(addr));
}

static void __init halibut_map_io(void)
{
	msm_map_common_io();
	msm_map_halibut_io();
	msm_clock_init(&halibut_clock_data);
	msm_halibut_allocate_memory_regions();
}

MACHINE_START(HALIBUT, "Halibut Board (QCT SURF7200A)")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7201A_SURF, "QCT SURF7201A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7201A_FFA, "QCT FFA7201A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7601A_SURF, "QCT SURF7601A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7601A_FFA, "QCT FFA7601A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(QST1105_SURF, "QCT QST1105 SURF")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x00200100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(QST1105_FFA, "QCT QST1105 FFA")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x00200100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7501A_SURF, "QCT SURF7501A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(MSM7501A_FFA, "QCT FFA7501A")
/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= halibut_map_io,
	.init_irq	= halibut_init_irq,
	.init_machine	= halibut_init,
	.timer		= &msm_timer,
MACHINE_END

