/* linux/arch/arm/mach-msm/board-qsd8x50.c
 *
 * Copyright (C) 2007 Google, Inc.
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * Author: Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/usb/mass_storage_function.h>
#include <linux/input.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>
#include <asm/mach/mmc.h>

#include <asm/arch/board.h>
#include <asm/arch/msm_iomap.h>
#include <asm/arch/msm_fb.h>
#include <asm/arch/sirc.h>
#include <asm/arch/vreg.h>
#include <asm/arch/mpp.h>
#include <asm/arch/msm_i2ckbd.h>
#include <asm/arch/msm_touchpad.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <linux/delay.h>
#include <linux/bootmem.h>

#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#include "clock.h"
#include <linux/i2c.h>
#include <asm/arch/gpio.h>
#include "proc_comm.h"
#include "socinfo.h"

/* XXX These sizes might not be hardcoded in the future, but
 * instead be obtained via tags or the kernel command line.
 */
#define MSM_PMEM_MDP_SIZE	0x800000
#define MSM_PMEM_ADSP_SIZE	0x800000
#define MSM_PMEM_GPU1_SIZE	0x800000
#define MSM_FB_SIZE		0x800000
#define MSM_AUDIO_SIZE		0x200000

#define MSM_SPI_PHYS            0xA1200000
#define MSM_SPI_SIZE            SZ_4K

void halibut_init_keypad(int ffa);

static struct resource smc91x_resources[] = {
	[0] = {
		.start	= 0x70000300,
		.end	= 0x70000400,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= MSM_GPIO_TO_INT(156),
		.end	= MSM_GPIO_TO_INT(156),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device smc91x_device = {
	.name		= "smc91x",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(smc91x_resources),
	.resource	= smc91x_resources,
};

static struct resource s1r72v05_resources[] = {
	[0] = {
		.start = 0x88000000,
		.end = 0x88000000 + 0xFF,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = MSM_GPIO_TO_INT(99),
		.end = 99,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device s1r72v05_device = {
	.name		= "s1r72v05",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(s1r72v05_resources),
	.resource	= s1r72v05_resources,
};

static struct resource s1r72v05_comet_resources[] = {
	[0] = {
		.start = 0x70000000,
		.end   = 0x70000000 + 0xFF,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = MSM_GPIO_TO_INT(99),
		.end = 99,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device s1r72v05_comet_device = {
	.name		= "s1r72v05",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(s1r72v05_comet_resources),
	.resource	= s1r72v05_comet_resources,
};

static struct resource msm_serial0_resources[] = {
	{
		.start	= INT_UART3,
		.end	= INT_UART3,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= MSM_UART3_PHYS,
		.end	= MSM_UART3_PHYS + MSM_UART3_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct platform_device msm_serial0_device = {
	.name	= "msm_serial",
	.id	= 2,
	.num_resources	= ARRAY_SIZE(msm_serial0_resources),
	.resource	= msm_serial0_resources,
};

static struct resource usb_resources[] = {
	{
		.start  = MSM_HSUSB_PHYS,
		.end    = MSM_HSUSB_PHYS + MSM_HSUSB_SIZE,
		.flags  = IORESOURCE_MEM,
	},
	{
		.start  = INT_USB_HS,
		.end    = INT_USB_HS,
		.flags  = IORESOURCE_IRQ,
	},
};

/* dynamic composition */
static struct usb_composition usb_func_composition[] = {
	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9012,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 2,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9013,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 3,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"mass_storage",  2,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9014,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 2,
		.funcs[0]           = {"mass_storage",  0,       1},
		.funcs[1]           = {"ethernet",         1, 2},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9016,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 3,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"nmea",          2,  1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0x9017,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 4,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"modem",         1, 1},
		.funcs[2]           = {"nmea",          2,  1},
		.funcs[3]           = {"mass_storage",  3,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0xF000,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 1,
		.funcs[0]           = {"mass_storage",  0,   1},
	},

	{
		.vendor_id          = 0x5c6,
		.product_id         = 0xF009,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 1,
		.funcs[0]           = {"ethernet",         0, 2},
	},

	{
		.vendor_id          = 0x18d1,
		.product_id         = 0xD00D,
		.product_name       = "Qualcomm HSUSB Device",
		.serial_number      = "1234567890ABCDEF",
		.manufacturer_name  = "Qualcomm Incorporated",
		.num_functions      = 5,
		.funcs[0]           = {"diag",          0,  1},
		.funcs[1]           = {"adb",           1,  1},
		.funcs[2]           = {"modem",         2,  1},
		.funcs[3]           = {"nmea",          3,  1},
		.funcs[4]           = {"mass_storage",  4,  1},
	},

};
static struct usb_mass_storage_platform_data usb_mass_storage_pdata = {
	.nluns          = 0x02,
	.buf_size       = 16384,
	.vendor         = "GOOGLE",
	.product        = "Mass storage",
	.release        = 0xffff,
};

static struct platform_device mass_storage_device = {
	.name           = "mass_storage",
	.id             = -1,
	.dev            = {
		.platform_data          = &usb_mass_storage_pdata,
	},
};

static struct msm_hsusb_platform_data msm_hsusb_pdata = {
	.version	= 0x0100,
	.phy_info       = (USB_PHY_INTEGRATED | USB_PHY_MODEL_180NM),
	.compositions	= usb_func_composition,
	.num_compositions = ARRAY_SIZE(usb_func_composition),
};

static u64 dma_mask = 0xffffffffULL;
static struct platform_device msm_hsusb_device = {
	.name           = "msm_hsusb",
	.id             = -1,
	.num_resources  = ARRAY_SIZE(usb_resources),
	.resource       = usb_resources,
	.dev            = {
		.dma_mask		= &dma_mask,
		.coherent_dma_mask      = 0xffffffffULL,
		.platform_data          = &msm_hsusb_pdata,
	},
};

static struct android_pmem_platform_data android_pmem_pdata = {
	.name = "pmem",
	.no_allocator = 1,
	.cached = 1,
};

static struct android_pmem_platform_data android_pmem_adsp_pdata = {
	.name = "pmem_adsp",
	.no_allocator = 0,
	.cached = 0,
};

static struct android_pmem_platform_data android_pmem_gpu1_pdata = {
	.name = "pmem_gpu1",
	.no_allocator = 1,
	.cached = 0,
};

static struct platform_device android_pmem_device = {
	.name = "android_pmem",
	.id = 0,
	.dev = { .platform_data = &android_pmem_pdata },
};

static struct platform_device android_pmem_adsp_device = {
	.name = "android_pmem",
	.id = 1,
	.dev = { .platform_data = &android_pmem_adsp_pdata },
};

static struct platform_device android_pmem_gpu1_device = {
	.name = "android_pmem",
	.id = 3,
	.dev = { .platform_data = &android_pmem_gpu1_pdata },
};

static struct resource msm_fb_resources[] = {
	{
		.flags  = IORESOURCE_DMA,
	}
};

static struct platform_device msm_fb_device = {
	.name   = "msm_fb",
	.id     = 0,
	.num_resources  = ARRAY_SIZE(msm_fb_resources),
	.resource       = msm_fb_resources,
};

static struct resource msm_audio_resources[] = {
       {
		.flags  = IORESOURCE_DMA,
       },
};

static struct platform_device msm_audio_device = {
	.name   = "msm_audio",
	.id     = 0,
	.num_resources  = ARRAY_SIZE(msm_audio_resources),
	.resource       = msm_audio_resources,
};

static struct resource qsd_spi_resources[] = {
	{
		.name   = "spi_irq_in",
		.start	= INT_SPI_INPUT,
		.end	= INT_SPI_INPUT,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_out",
		.start	= INT_SPI_OUTPUT,
		.end	= INT_SPI_OUTPUT,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_err",
		.start	= INT_SPI_ERROR,
		.end	= INT_SPI_ERROR,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_base",
		.start	= MSM_SPI_PHYS,
		.end	= MSM_SPI_PHYS + MSM_SPI_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.name   = "spi_clk",
		.start	= 17,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_mosi",
		.start	= 18,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_miso",
		.start	= 19,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_cs0",
		.start	= 20,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_pwr",
		.start	= 21,
		.end	= 0,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_cs0",
		.start	= 22,
		.end	= 0,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device qsd_device_spi = {
	.name	        = "spi_qsd",
	.id	        = 0,
	.num_resources	= ARRAY_SIZE(qsd_spi_resources),
	.resource	= qsd_spi_resources,
};

static struct resource qsd_spi_comet_resources[] = {
	{
		.name   = "spi_irq_in",
		.start	= INT_SPI_INPUT,
		.end	= INT_SPI_INPUT,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_out",
		.start	= INT_SPI_OUTPUT,
		.end	= INT_SPI_OUTPUT,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_err",
		.start	= INT_SPI_ERROR,
		.end	= INT_SPI_ERROR,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_base",
		.start	= MSM_SPI_PHYS,
		.end	= MSM_SPI_PHYS + MSM_SPI_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.name   = "spi_clk",
		.start	= 17,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_mosi",
		.start	= 18,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_miso",
		.start	= 19,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_cs0",
		.start	= 20,
		.end	= 1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.name   = "spi_irq_cs0",
		.start	= 106,
		.end	= 0,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device qsd_spi_comet_device = {
	.name	        = "spi_qsd",
	.id	        = 0,
	.num_resources	= ARRAY_SIZE(qsd_spi_comet_resources),
	.resource	= qsd_spi_comet_resources,
};

static struct msm_camera_device_platform_data msm_camera_device_data[2] = {
	{
	.sensor_reset	= 17,
	.sensor_pwd	  = 85,
	.vcm_pwd      = 0,
	.sensor_name  = "mt9d112",
	},
	{
	.sensor_reset	= 17,
	.sensor_pwd	  = 85,
	.vcm_pwd      = 0,
	.sensor_name  = "mt9t013",
	},
};

static struct platform_device msm_camera_device = {
	.name		= "msm_camera",
	.id		= 0,
	.dev		= {
		.platform_data = &msm_camera_device_data[0],
	},
};

#ifdef CONFIG_BT
static struct platform_device msm_bt_power_device = {
	.name = "bt_power",
};

enum {
	BT_SYSRST,
	BT_WAKE,
	BT_HOST_WAKE,
	BT_PWR_EN,
	BT_RFR,
	BT_CTS,
	BT_RX,
	BT_TX,
	BT_PCM_DOUT,
	BT_PCM_DIN,
	BT_PCM_SYNC,
	BT_PCM_CLK,
};

static unsigned bt_config_power_on[] = {
	GPIO_CFG(18, 0, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* SYSRST */
	GPIO_CFG(19, 0, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* WAKE */
	GPIO_CFG(21, 0, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* HOST_WAKE */
	GPIO_CFG(22, 0, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PWR_EN */
	GPIO_CFG(43, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* RFR */
	GPIO_CFG(44, 2, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* CTS */
	GPIO_CFG(45, 2, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* Rx */
	GPIO_CFG(46, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* Tx */
	GPIO_CFG(68, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_DOUT */
	GPIO_CFG(69, 1, GPIO_INPUT,  GPIO_NO_PULL, GPIO_2MA),	/* PCM_DIN */
	GPIO_CFG(70, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_SYNC */
	GPIO_CFG(71, 2, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA),	/* PCM_CLK */
};
static unsigned bt_config_power_off[] = {
	GPIO_CFG(18, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* SYSRST */
	GPIO_CFG(19, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* WAKE */
	GPIO_CFG(21, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* HOST_WAKE */
	GPIO_CFG(22, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PWR_EN */
	GPIO_CFG(43, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* RFR */
	GPIO_CFG(44, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* CTS */
	GPIO_CFG(45, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* Rx */
	GPIO_CFG(46, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* Tx */
	GPIO_CFG(68, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_DOUT */
	GPIO_CFG(69, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_DIN */
	GPIO_CFG(70, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_SYNC */
	GPIO_CFG(71, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA),	/* PCM_CLK */
};


static int bluetooth_power(int on)
{
	int pin, rc;

	printk(KERN_DEBUG "%s\n", __func__);

	if (on) {
		for (pin = 0; pin < ARRAY_SIZE(bt_config_power_on); pin++) {
			rc = gpio_tlmm_config(bt_config_power_on[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, bt_config_power_on[pin], rc);
				return -EIO;
			}
		}

		gpio_set_value(22, on); /* PWR_EN */
		gpio_set_value(18, on); /* SYSRST */

	} else {
		gpio_set_value(18, on); /* SYSRST */
		gpio_set_value(22, on); /* PWR_EN */

		for (pin = 0; pin < ARRAY_SIZE(bt_config_power_off); pin++) {
			rc = gpio_tlmm_config(bt_config_power_off[pin],
					      GPIO_ENABLE);
			if (rc) {
				printk(KERN_ERR
				       "%s: gpio_tlmm_config(%#x)=%d\n",
				       __func__, bt_config_power_off[pin], rc);
				return -EIO;
			}
		}

	}

	return 0;
}

static void __init bt_power_init(void)
{
	msm_bt_power_device.dev.platform_data = &bluetooth_power;
}
#else
#define bt_power_init(x) do {} while (0)
#endif

static struct resource bluesleep_resources[] = {
	{
		.name	= "gpio_host_wake",
		.start	= 21,
		.end	= 21,
		.flags	= IORESOURCE_IO,
	},
	{
		.name	= "gpio_ext_wake",
		.start	= 19,
		.end	= 19,
		.flags	= IORESOURCE_IO,
	},
	{
		.name	= "host_wake",
		.start	= MSM_GPIO_TO_INT(21),
		.end	= MSM_GPIO_TO_INT(21),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device msm_bluesleep_device = {
	.name = "bluesleep",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(bluesleep_resources),
	.resource	= bluesleep_resources,
};

static struct platform_device *devices[] __initdata = {
	&msm_fb_device,
	&msm_serial0_device,
	&msm_hsusb_device,
	&mass_storage_device,
	&smc91x_device,
	&s1r72v05_device,
	&android_pmem_device,
	&android_pmem_adsp_device,
	&android_pmem_gpu1_device,
	&qsd_device_spi,
	&msm_camera_device,
	&msm_audio_device,
#ifdef CONFIG_BT
	&msm_bt_power_device,
#endif
	&msm_bluesleep_device,
};

static struct platform_device *devices_comet[] __initdata = {
#ifdef CONFIG_FB_MSM
	&msm_fb_device,
#endif
	&msm_serial0_device,
	&msm_hsusb_device,
	&mass_storage_device,
	&s1r72v05_comet_device,
	&android_pmem_device,
	&android_pmem_adsp_device,
	&android_pmem_gpu1_device,
	&qsd_spi_comet_device,
	&msm_camera_device,
	&msm_audio_device,
};

/*
 * All SD slots are the same(ish)
 */

#ifdef HAVE_SDCC_SLOT_INSERT_DETECTION
static unsigned int qsd8x50_sdcc_slot_status(struct device *dev)
{
	/*
	 * TODO: Hook this up to the FPGA for reading slot status
	 * For now return 0.
	 */
	return 0;
}
#endif

static unsigned int vreg_enabled;
static unsigned sdcc_cfg_data[][6] = {
	/* SDC1 configs */
	{
	GPIO_CFG(51, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(52, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(53, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(54, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(55, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(56, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	},
	/* SDC2 configs */
	{
	GPIO_CFG(62, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	GPIO_CFG(63, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(64, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(65, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(66, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(67, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	},
	/* SDC3 configs */
	{
	GPIO_CFG(88, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	GPIO_CFG(89, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(90, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(91, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(92, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(93, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	},
	/* SDC4 configs */
	{
	GPIO_CFG(19, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(20, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(21, 3, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(107, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(108, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_8MA),
	GPIO_CFG(109, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_8MA),
	}
};

static int msm_sdcc_setup_gpio(int dev_id, unsigned enable)
{
	int i, rc;
	for (i = 0; i < ARRAY_SIZE(sdcc_cfg_data[dev_id - 1]); i++) {
		rc = gpio_tlmm_config(sdcc_cfg_data[dev_id - 1][i],
			enable ? GPIO_ENABLE : GPIO_DISABLE);
		if (rc) {
			printk(KERN_ERR "%s: gpio_tlmm_config(%#x)=%d\n",
				__func__, sdcc_cfg_data[dev_id - 1][i], rc);
		}
	}
	return rc;
}

static int msm_sdcc_setup_power(int dev_id, int on)
{
	int rc = 0;
	struct mpp *mpp_mmc;
	struct vreg *vreg_mmc;

	rc = msm_sdcc_setup_gpio(dev_id, on);
	if (rc)
		return -EIO;

	if ((on && vreg_enabled) || (!on && !vreg_enabled))
		return 0;

	if (machine_is_qsd8x50_ffa()) {
		mpp_mmc = mpp_get(NULL, "mpp3");
		if (!mpp_mmc) {
			printk(KERN_ERR "%s: mpp get failed\n", __func__);
			return -EIO;
		}

		rc = mpp_config_digital_out(mpp_mmc,
		     MPP_CFG(MPP_DLOGIC_LVL_MSMP,
		     on ? MPP_DLOGIC_OUT_CTRL_HIGH : MPP_DLOGIC_OUT_CTRL_LOW));
		if (rc) {
			printk(KERN_ERR
				"%s: Failed to configure mpp (%d)\n",
				__func__, rc);
			return rc;
		}
	} else {
		vreg_mmc = vreg_get(NULL, "gp5");
		if (IS_ERR(vreg_mmc)) {
			printk(KERN_ERR "%s: vreg get failed (%ld)\n",
			       __func__, PTR_ERR(vreg_mmc));
			return PTR_ERR(vreg_mmc);
		}

		rc = on ? vreg_enable(vreg_mmc) : vreg_disable(vreg_mmc);
		if (rc) {
			printk(KERN_ERR "%s: Failed to configure vreg (%d)\n",
				__func__, rc);
			return rc;
		}
	}
	vreg_enabled = on;
	return 0;
}

static struct mmc_platform_data qsd8x50_sdcc_data = {
	.ocr_mask	= MMC_VDD_27_28 | MMC_VDD_28_29,
#ifdef HAVE_SDCC_SLOT_INSERT_DETECTION
	.status		= qsd8x50_sdcc_slot_status,
#endif
	.setup_power	= msm_sdcc_setup_power,
};

extern struct sys_timer msm_timer;

static struct clk qsd8x50_clocks[] = {
	CLOCK("acpu_clk",	ACPU_CLK,	0),
	CLOCK("adm_clk",	ADM_CLK,	0),
	CLOCK("ebi1_clk",	EBI1_CLK,	MIN_SET | NO_SET),
	CLOCK("ebi2_clk",	EBI2_CLK,	0),
	CLOCK("ecodec_clk",	ECODEC_CLK,	0),
	CLOCK("emdh_clk",	EMDH_CLK,	OFF),
	CLOCK("gp_clk",		GP_CLK,		0),
	CLOCK("grp_clk",	GRP_CLK,	OFF),
	CLOCK("i2c_clk",	I2C_CLK,	0),
	CLOCK("icodec_rx_clk",	ICODEC_RX_CLK,	0),
	CLOCK("icodec_tx_clk",	ICODEC_TX_CLK,	0),
	CLOCK("imem_clk",	IMEM_CLK,	OFF),
	CLOCK("mdc_clk",	MDC_CLK,	0),
	CLOCK("mdp_clk",	MDP_CLK,	OFF),
	CLOCK("mdp_lcdc_pclk_clk",	MDP_LCDC_PCLK_CLK,	0),
	CLOCK("mdp_lcdc_pad_pclk_clk",	MDP_LCDC_PAD_PCLK_CLK,	0),
	CLOCK("mdp_vsync_clk",	MDP_VSYNC_CLK,	0),
	CLOCK("pbus_clk",	PBUS_CLK,	0),
	CLOCK("pcm_clk",	PCM_CLK,	0),
	CLOCK("pmdh_clk",	PMDH_CLK,	OFF | MIN_MAX),
/* sdac_clk does not work currently
	CLOCK("sdac_clk",	SDAC_CLK,	OFF),
*/
	CLOCK("sdc1_clk",	SDC1_CLK,	OFF),
	CLOCK("sdc1_pclk",	SDC1_PCLK,	OFF),
	CLOCK("sdc2_clk",	SDC2_CLK,	OFF),
	CLOCK("sdc2_pclk",	SDC2_PCLK,	OFF),
	CLOCK("sdc3_clk",	SDC3_CLK,	OFF),
	CLOCK("sdc3_pclk",	SDC3_PCLK,	OFF),
	CLOCK("sdc4_clk",	SDC4_CLK,	OFF),
	CLOCK("sdc4_pclk",	SDC4_PCLK,	OFF),
	CLOCK("spi_clk",	SPI_CLK,	OFF),
	CLOCK("tsif_clk",	TSIF_CLK,	0),
	CLOCK("tsif_ref_clk",	TSIF_REF_CLK,	0),
	CLOCK("tv_dac_clk",	TV_DAC_CLK,	0),
	CLOCK("tv_enc_clk",	TV_ENC_CLK,	0),
	CLOCK("uart1_clk",	UART1_CLK,	OFF),
	CLOCK("uart2_clk",	UART2_CLK,	0),
	CLOCK("uart3_clk",	UART3_CLK,	OFF),
	CLOCK("uart1dm_clk",	UART1DM_CLK,	OFF),
	CLOCK("uart2dm_clk",	UART2DM_CLK,	0),
	CLOCK("usb_hs_clk",	USB_HS_CLK,	OFF),
	CLOCK("usb_hs_pclk",	USB_HS_PCLK,	OFF),
	CLOCK("usb_otg_clk",	USB_OTG_CLK,	0),
	CLOCK("vdc_clk",	VDC_CLK,	OFF | MIN_MAX),
	CLOCK("vfe_clk",	VFE_CLK,	OFF),
	CLOCK("vfe_mdc_clk",	VFE_MDC_CLK,	OFF),
	CLOCK("vfe_axi_clk",	VFE_AXI_CLK,	OFF),
};

static struct msm_clock_platform_data qsd8x50_clock_data = {
	.acpu_switch_time_us = 50,
	.max_speed_delta_khz = 256000,
	.vdd_switch_time_us = 62,
	.active_clocks = qsd8x50_clocks,
	.active_clock_count = ARRAY_SIZE(qsd8x50_clocks),
};

static struct msm_i2ckbd_platform_data msm_kybd_data = {
	.hwrepeat = 0,
	.scanset1 = 1,
	.gpioreset = 35,
};

static struct msm_touchpad_platform_data msm_touchpad_data = {
	.gpioirq     = 38,
	.gpiosuspend = 34,
};

static struct i2c_board_info msm_i2c_board_info[] __initdata = {
	{
		I2C_BOARD_INFO("msm-i2ckbd", 0x3A),
		.type           = "msm-i2ckbd",
		.irq            = 36,
		.platform_data  = &msm_kybd_data
	},
	{
		I2C_BOARD_INFO("msm_touchpad", 0x2A),
		.type          = "msm_touchpad",
		.irq           =  MSM_GPIO_TO_INT(38),
		.platform_data = &msm_touchpad_data
	},
	{
		I2C_BOARD_INFO("mt9d112", 0x78 >> 1),
	},
	{
		I2C_BOARD_INFO("mt9t013", 0x6C >> 1),
	},
};

static struct msm_touchpad_platform_data msm_touchpad_data_comet = {
	.gpioirq     = 42,
	.gpiosuspend = 34,
};

static struct i2c_board_info msm_i2c_board_info_comet[] __initdata = {
	{
		I2C_BOARD_INFO("msm-i2ckbd", 0x3A),
		.type           = "msm-i2ckbd",
		.irq            = 144,
		.platform_data  = &msm_kybd_data
	},
	{
		I2C_BOARD_INFO("msm_touchpad", 0x2A),
		.type          = "msm_touchpad",
		.irq           =  MSM_GPIO_TO_INT(42),
		.platform_data = &msm_touchpad_data_comet
	},
	{
		I2C_BOARD_INFO("mt9d112", 0x78 >> 1),
	},
	{
		I2C_BOARD_INFO("mt9t013", 0x6C >> 1),
	},
};

static struct spi_board_info msm_spi_board_info[] __initdata = {
	{
		.modalias       = "bma150",
		.mode           = SPI_MODE_3,
		.irq            = MSM_GPIO_TO_INT(22),
		.bus_num        = 0,
		.chip_select    = 0,
		.max_speed_hz   = 10000000,
	},
};

static struct spi_board_info msm_spi_board_info_comet[] __initdata = {
	{
		.modalias       = "bma150",
		.mode           = SPI_MODE_3,
		.irq            = MSM_GPIO_TO_INT(106),
		.bus_num        = 0,
		.chip_select    = 0,
		.max_speed_hz   = 10000000,
	},
};

static void __init qsd8x50_init_mmc(void)
{
	msm_add_sdcc(1, &qsd8x50_sdcc_data);

	if (machine_is_qsd8x50_surf()) {
		msm_add_sdcc(2, &qsd8x50_sdcc_data);
		msm_add_sdcc(3, &qsd8x50_sdcc_data);
		msm_add_sdcc(4, &qsd8x50_sdcc_data);
	}
}

static uint32_t camera_off_gpio_table[] = {
	/* parallel CAMERA interfaces */
	GPIO_CFG(0,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT0 */
	GPIO_CFG(1,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT1 */
	GPIO_CFG(2,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT2 */
	GPIO_CFG(3,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT3 */
	GPIO_CFG(4,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT4 */
	GPIO_CFG(5,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT5 */
	GPIO_CFG(6,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT6 */
	GPIO_CFG(7,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT7 */
	GPIO_CFG(8,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT8 */
	GPIO_CFG(9,  1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT9 */
	GPIO_CFG(10, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT10 */
	GPIO_CFG(11, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* DAT11 */
	GPIO_CFG(12, 0, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* PCLK */
	GPIO_CFG(13, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* HSYNC_IN */
	GPIO_CFG(14, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* VSYNC_IN */
	GPIO_CFG(15, 1, GPIO_OUTPUT, GPIO_PULL_UP, GPIO_4MA), /* MCLK */
};

static uint32_t camera_on_gpio_table[] = {
	/* parallel CAMERA interfaces */
	GPIO_CFG(0,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT0 */
	GPIO_CFG(1,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT1 */
	GPIO_CFG(2,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT2 */
	GPIO_CFG(3,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT3 */
	GPIO_CFG(4,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT4 */
	GPIO_CFG(5,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT5 */
	GPIO_CFG(6,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT6 */
	GPIO_CFG(7,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT7 */
	GPIO_CFG(8,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT8 */
	GPIO_CFG(9,  1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT9 */
	GPIO_CFG(10, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT10 */
	GPIO_CFG(11, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* DAT11 */
	GPIO_CFG(12, 0, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* PCLK */
	GPIO_CFG(13, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* HSYNC_IN */
	GPIO_CFG(14, 1, GPIO_INPUT, GPIO_PULL_DOWN, GPIO_2MA), /* VSYNC_IN */
	GPIO_CFG(15, 1, GPIO_OUTPUT, GPIO_NO_PULL, GPIO_2MA), /* MCLK */
};

static void config_gpio_table(uint32_t *table, int len)
{
	int n;
	unsigned id;
	for (n = 0; n < len; n++) {
		id = table[n];
		msm_proc_comm(PCOM_RPC_GPIO_TLMM_CONFIG_EX, &id, 0);
	}
}

void config_camera_on_gpios(void)
{
	config_gpio_table(camera_on_gpio_table,
		ARRAY_SIZE(camera_on_gpio_table));
}

void config_camera_off_gpios(void)
{
	config_gpio_table(camera_off_gpio_table,
		ARRAY_SIZE(camera_off_gpio_table));
}

static void __init qsd8x50_init_irq(void)
{
	msm_init_irq();
}

static void __init msm_fb_init(void)
{
	msm_fb_register_device("mdp", 0);
	msm_fb_register_device("pmdh", 0);
	msm_fb_register_device("emdh", 0);
	msm_fb_register_device("tvenc", 0);
	msm_fb_register_device("lcdc", 0);
}

static void __init qsd8x50_init(void)
{
	socinfo_init();

	if (machine_is_qsd8x50_ffa()) {
		smc91x_resources[0].start = 0x84000300;
		smc91x_resources[0].end = 0x84000400;
	}

	msm_init_sirc();
	msm_init_gpio();
	if (machine_is_qsd8x50_comet())
		platform_add_devices(devices_comet, ARRAY_SIZE(devices_comet));
	else
		platform_add_devices(devices, ARRAY_SIZE(devices));
#if defined(CONFIG_QSD_GPIO_KEYPAD)
	halibut_init_keypad(machine_is_qsd8x50_ffa());
#endif
	msm_add_devices();
	if (machine_is_qsd8x50_comet()) {
		i2c_register_board_info(0, msm_i2c_board_info_comet,
					ARRAY_SIZE(msm_i2c_board_info_comet));
		spi_register_board_info(msm_spi_board_info_comet,
					ARRAY_SIZE(msm_spi_board_info_comet));
	} else {
		i2c_register_board_info(0, msm_i2c_board_info,
					ARRAY_SIZE(msm_i2c_board_info));
		spi_register_board_info(msm_spi_board_info,
					ARRAY_SIZE(msm_spi_board_info));
	}
	config_camera_off_gpios();
	qsd8x50_init_mmc();
	bt_power_init();
	msm_fb_init();
}

static void __init msm_qsd8x50_allocate_memory_regions(void)
{
	void *addr, *addr_1m_aligned;
	unsigned long size;

	size = MSM_PMEM_MDP_SIZE;
	addr = alloc_bootmem(size);
	android_pmem_pdata.start = __pa(addr);
	android_pmem_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for pmem\n", size, addr, __pa(addr));

	size = MSM_PMEM_ADSP_SIZE;
	addr = alloc_bootmem(size);
	android_pmem_adsp_pdata.start = __pa(addr);
	android_pmem_adsp_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for adsp pmem\n", size, addr, __pa(addr));

	/* The GPU1 area must be aligned to a 1M boundary */
	/* XXX For now allocate an extra 1M, use the aligned part, waste the extra memory */
	size = MSM_PMEM_GPU1_SIZE + 0x100000 - PAGE_SIZE;
	addr = alloc_bootmem(size);
	addr_1m_aligned = (void *)(((unsigned int)addr + 0x100000) & 0xfff00000);
	size = MSM_PMEM_GPU1_SIZE;
	android_pmem_gpu1_pdata.start = __pa(addr_1m_aligned);
	android_pmem_gpu1_pdata.size = size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for gpu1 pmem\n", size, addr_1m_aligned, __pa(addr_1m_aligned));

	size = MSM_FB_SIZE;
	addr = alloc_bootmem(size);
	msm_fb_resources[0].start = __pa(addr);
	msm_fb_resources[0].end = msm_fb_resources[0].start + size - 1;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for fb\n", size, addr, __pa(addr));

	size = MSM_AUDIO_SIZE;
	addr = alloc_bootmem(size);
	msm_audio_resources[0].start = __pa(addr);
	msm_audio_resources[0].end = msm_audio_resources[0].start + size;
	printk(KERN_INFO "allocating %lu bytes at %p (%lx physical) for audio\n", size, addr, __pa(addr));
}

static void __init qsd8x50_map_io(void)
{
	msm_map_qsd8x50_io();
	msm_clock_init(&qsd8x50_clock_data);
	msm_qsd8x50_allocate_memory_regions();
}

MACHINE_START(QSD8X50_SURF, "QSD8X50 SURF Board (QCT SURF8X50)")

/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x00200100,
	.map_io		= qsd8x50_map_io,
	.init_irq	= qsd8x50_init_irq,
	.init_machine	= qsd8x50_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(QSD8X50_FFA, "QSD8X50 FFA (QCT FFA8X50)")

/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x00200100,
	.map_io		= qsd8x50_map_io,
	.init_irq	= qsd8x50_init_irq,
	.init_machine	= qsd8x50_init,
	.timer		= &msm_timer,
MACHINE_END

MACHINE_START(QSD8X50_COMET, "QSD8X50 COMET")

/* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x00200100,
	.map_io		= qsd8x50_map_io,
	.init_irq	= qsd8x50_init_irq,
	.init_machine	= qsd8x50_init,
	.timer		= &msm_timer,
MACHINE_END
