/* arch/arm/mach-msm/mdm_notifier.c
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 * Modem Restart Notifier -- Provides notification
 *			     of modem restart events.
 */

#include <linux/notifier.h>
#include <linux/init.h>
#include <linux/debugfs.h>
#include <linux/module.h>
#include <linux/notifier.h>
#include "mdm_notifier.h"

#define DEBUG

static struct srcu_notifier_head mdm_notifier_list;
static struct workqueue_struct *mdm_notifier_wq;

static void notify_work_start_reset(struct work_struct *work)
{
	mdm_notify(0, MDM_NOTIFIER_START_RESET);
}
static DECLARE_WORK(mdm_notifier_start_reset_work, &notify_work_start_reset);

void mdm_queue_start_reset_notify(void)
{
	int ret;

	ret = queue_work(mdm_notifier_wq, &mdm_notifier_start_reset_work);

	if (!ret)
		printk(KERN_ERR "%s\n", __func__);
}
EXPORT_SYMBOL(mdm_queue_start_reset_notify);

static void notify_work_end_reset(struct work_struct *work)
{
	mdm_notify(0, MDM_NOTIFIER_END_RESET);
}
static DECLARE_WORK(mdm_notifier_end_reset_work, &notify_work_end_reset);

void mdm_queue_end_reset_notify(void)
{
	int ret;

	ret = queue_work(mdm_notifier_wq, &mdm_notifier_end_reset_work);

	if (!ret)
		printk(KERN_ERR "%s\n", __func__);
}
EXPORT_SYMBOL(mdm_queue_end_reset_notify);

int mdm_register_notifier(struct notifier_block *nb)
{
	int ret;

	ret = srcu_notifier_chain_register(
		&mdm_notifier_list, nb);

	return ret;
}
EXPORT_SYMBOL(mdm_register_notifier);

int mdm_unregister_notifier(struct notifier_block *nb)
{
	int ret;

	ret = srcu_notifier_chain_unregister(
		&mdm_notifier_list, nb);

	return ret;
}
EXPORT_SYMBOL(mdm_unregister_notifier);

void mdm_notify(struct mdm_status *status, unsigned int state)
{
	srcu_notifier_call_chain(&mdm_notifier_list, state, status);
}
EXPORT_SYMBOL(mdm_notify);

#if defined(CONFIG_DEBUG_FS)
static ssize_t debug_write(struct file *file, const char __user *buf,
			   size_t count, loff_t *ppos)
{
	mdm_notify(0, 0);
	return count;
}

static int debug_open(struct inode *inode, struct file *file)
{
	return 0;
}

static const struct file_operations debug_ops = {
	.write = debug_write,
	.open = debug_open,
};

static void mdm_notifier_debugfs_init(void)
{
	struct dentry *dent;

	dent = debugfs_create_dir("mdm_notifier", 0);
	if (IS_ERR(dent))
		return;

	debugfs_create_file("launch", 0444, dent, 0, &debug_ops);
}
#else
static void mdm_notifier_debugfs_init(void) {}
#endif

#if defined(DEBUG)
static int mdm_notifier_test_call(struct notifier_block *this,
				  unsigned long code,
				  void *_cmd)
{
	switch (code) {
	case MDM_NOTIFIER_START_RESET:
		printk(KERN_ERR "Notify: start reset\n");
		break;
	case MDM_NOTIFIER_END_RESET:
		printk(KERN_ERR "Notify: end reset\n");
		break;
	default:
		printk(KERN_ERR "Notify: general\n");
		break;
	}
	return NOTIFY_DONE;
}

static struct notifier_block nb = {
	.notifier_call = mdm_notifier_test_call,
};

static void register_test_notifier(void)
{
	mdm_register_notifier(&nb);
}
#endif

static int __init init_mdm_notifier_list(void)
{
	srcu_init_notifier_head(&mdm_notifier_list);
	mdm_notifier_debugfs_init();
#if defined(DEBUG)
	register_test_notifier();
#endif

	/* Create the workqueue */
	mdm_notifier_wq = create_singlethread_workqueue("mdm_notifier");
	if (!mdm_notifier_wq)
		return -ENOMEM;

	return 0;
}
module_init(init_mdm_notifier_list);
