/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/mutex.h>
#include <linux/dma-mapping.h>
#include <asm/arch/msm8k_cad.h>
#include <asm/arch/msm8k_ard_clk.h>
#include <asm/arch/msm8k_ard_adie.h>
#include <asm/arch/msm8k_cad_rpc_type.h>
#include <asm/arch/msm8k_cad_rpc.h>

static struct ard_session_info_struct_type ard_session[ARD_AUDIO_MAX_CLIENT];
struct ard_session_info_struct_type        *ardsession[CAD_MAX_SESSION];
struct ard_state_struct_type               ard_state;

#define CAD_FORMAT_BLK_OFFSET	0x100000

s32 cad_ard_init(struct cad_func_ptr_tbl_type **func_ptr_tbl)
{
	s32 rc;
	static struct cad_func_ptr_tbl_type ard_func = {
		ard_open, ard_close, ard_write, ard_read, ard_ioctl
	};

	*func_ptr_tbl = &ard_func;

	mutex_init(&ard_state.device_mutex);

	rc = adie_init();
	if (rc)
		return rc;

	rc = cad_rpc_init(CAD_RPC_ARM11);
	if (rc)
		return rc;

	rc = cad_rpc_reg_callback(0, ard_callback_func, NULL);
	if (rc)
		return rc;

	ard_state.state = ARD_STATE_RESET;
	ard_state.afe_enabled = ARD_FALSE;
	ard_state.device_configured = ARD_FALSE;
	ard_state.device_inuse = SND_DEVICE_HANDSET;

	return rc;
}

s32 cad_ard_dinit(void)
{
	s32 rc;

	mutex_unlock(&ard_state.device_mutex);

	rc = adie_dinit();
	if (rc)
		return rc;

	rc = cad_rpc_deinit();
	if (rc)
		return rc;

	rc = cad_rpc_dereg_callback(0, ard_callback_func);

	return rc;
}

s32 ard_open(s32 ses_id, struct cad_open_struct_type *open_param)
{
	struct cadi_open_struct_type	*op;

	s32 i, rc;
	u32 open_parm_len;

	rc = CAD_RES_SUCCESS;
	open_parm_len = 0;
	op = NULL;

	for (i = 0; i < ARD_AUDIO_MAX_CLIENT; i++) {
		if (ard_session[i].enabled == ARD_FALSE)
			break;
	}

	if (i == ARD_AUDIO_MAX_CLIENT)
		return CAD_RES_FAILURE;

	if (open_param != NULL) {
		if (open_param->op_code == CAD_OPEN_OP_DEVICE_CTRL)
			ard_session[i].session_type = DEVICE_CTRL_TYPE;
		else if ((open_param->op_code == CAD_OPEN_OP_WRITE) ||
			(open_param->op_code == CAD_OPEN_OP_READ)) {
			ard_session[i].session_type = STREAM_TYPE;
		} else {
			return CAD_RES_FAILURE;
		}

		ard_session[i].sess_open_info =
			kmalloc(sizeof(struct cadi_open_struct_type),
				GFP_KERNEL|GFP_DMA);

		if (ard_session[i].sess_open_info == NULL)
			return CAD_RES_FAILURE;

		op = (struct cadi_open_struct_type *)
				ard_session[i].sess_open_info;

		memset(op, 0, sizeof(struct cadi_open_struct_type));

		open_parm_len = sizeof(struct cad_open_struct_type);

		memcpy(&op->cad_open, open_param, open_parm_len);
	}
	ard_session[i].ev_buf =
		kmalloc(sizeof(struct cadi_evt_struct_type),
				GFP_KERNEL|GFP_DMA);

	if (ard_session[i].ev_buf == NULL) {
			kfree(ard_session[1].sess_open_info);
			ard_session[1].sess_open_info = NULL;
			return CAD_RES_FAILURE;
	}

	ard_session[i].cmd_buf = NULL;

	ard_session[i].enabled = ARD_TRUE;
	ardsession[ses_id] = &ard_session[i];

	return rc;
}


s32 ard_close(s32 ses_id)
{
	struct cad_stream_device_struct_type	*cadr_strm_device = NULL;
	struct cadi_open_struct_type		*cadr = NULL;
	struct cad_stream_config_struct_type	*cadr_config = NULL;
	s32 rc;

	ardsession[ses_id]->enabled = ARD_FALSE;

	mutex_lock(&ard_state.device_mutex);

	rc = ard_state_control(ses_id);

	mutex_unlock(&ard_state.device_mutex);

	cadr = ardsession[ses_id]->sess_open_info;
	cadr_strm_device = &(cadr->cad_device);
	cadr_config = &(cadr->cad_config);

	kfree(cadr_strm_device->p_device);
	kfree(ardsession[ses_id]->sess_open_info);
	kfree(ardsession[ses_id]->ev_buf);

	return rc;
}

s32 ard_ioctl(s32 ses_id, s32 cmd_code, void *pcmd_buf, s32 cmd_len)
{
	struct cad_open_struct_type		*p_cadr_open = NULL;
	struct cad_stream_device_struct_type	*p_cadr_strm_device = NULL;
	struct cad_stream_info_struct_type	*p_cadr_stream = NULL;
	struct cad_stream_config_struct_type	*p_cadr_config = NULL;
	struct cadi_evt_struct_type		*p_ev_buf = NULL;
	struct cadi_open_struct_type		*p_cadr = NULL;
	struct cad_device_struct_type		*def_device = NULL;
	struct cad_stream_device_struct_type	*p_strm_dev = NULL;
	void					*p_cmd_buf = NULL;
	void                                    *p_config_format_block = NULL;

	s32		rc, rpc;
	u32		cmd_buf_len;

	rc = rpc = CAD_RES_SUCCESS;
	cmd_buf_len = 0;

	switch (cmd_code) {
	case CAD_IOCTL_CMD_DEVICE_SET_GLOBAL_DEFAULT:
		if (ardsession[ses_id]->session_type == DEVICE_CTRL_TYPE) {
			def_device = (struct cad_device_struct_type *)pcmd_buf;

			mutex_lock(&ard_state.device_mutex);

			ard_state.device_inuse = def_device->ul_device;

			mutex_unlock(&ard_state.device_mutex);

			p_cadr = ardsession[ses_id]->sess_open_info;
			p_cadr_open = &(p_cadr->cad_open);
			p_ev_buf = ardsession[ses_id]->ev_buf;

			rpc = cad_rpc_open(ses_id, 1, p_cadr, p_ev_buf);
			if (rpc != CAD_RES_SUCCESS)
				return CAD_RES_FAILURE;

			cmd_buf_len = sizeof(struct cad_device_struct_type);
			if (cmd_len != cmd_buf_len)
				return CAD_RES_FAILURE;

			ard_session[ses_id].cmd_buf =
				kmalloc(sizeof(struct cad_device_struct_type),
						GFP_KERNEL|GFP_DMA);

			if (ard_session[ses_id].cmd_buf == NULL)
				return CAD_RES_FAILURE;

			p_cmd_buf = ardsession[ses_id]->cmd_buf;
			memcpy(p_cmd_buf, pcmd_buf, cmd_buf_len);

			rpc = cad_rpc_ioctl(ses_id, 1,
					CAD_IOCTL_CMD_DEVICE_SET_GLOBAL_DEFAULT,
					p_cmd_buf, cmd_buf_len, p_ev_buf);

			if (rpc != CAD_RES_SUCCESS) {
				kfree(p_cmd_buf);
				return CAD_RES_FAILURE;
			}

			kfree(p_cmd_buf);
		}
		break;

	case CAD_IOCTL_CMD_STREAM_START:
		mutex_lock(&ard_state.device_mutex);

		rc = ard_state_control(ses_id);

		mutex_unlock(&ard_state.device_mutex);
		break;

	case CAD_IOCTL_CMD_SET_STREAM_DEVICE:
		p_cadr = ardsession[ses_id]->sess_open_info;
		p_cadr_strm_device = &(p_cadr->cad_device);
		p_strm_dev = (struct cad_stream_device_struct_type *)pcmd_buf;

		p_cadr_strm_device->p_device =
			kmalloc((sizeof(u32) * p_strm_dev->ul_device_len),
				GFP_KERNEL|GFP_DMA);

		if ((pcmd_buf != NULL)
			&& (p_cadr_strm_device->p_device != NULL)) {
			memcpy(p_cadr_strm_device->p_device,
				p_strm_dev->p_device,
				(sizeof(u32) * p_strm_dev->ul_device_len));
			p_cadr_strm_device->ul_device_len
					= p_strm_dev->ul_device_len;
		} else {
			kfree(p_cadr_strm_device->p_device);
			return CAD_RES_FAILURE;
		}
		break;

	case CAD_IOCTL_CMD_SET_STREAM_INFO:
		p_cadr = ardsession[ses_id]->sess_open_info;
		p_cadr_stream = &(p_cadr->cad_stream);

		if (pcmd_buf != NULL)
			memcpy(p_cadr_stream, pcmd_buf, cmd_len);
		else
			return CAD_RES_FAILURE;
		break;

	case CAD_IOCTL_CMD_SET_STREAM_CONFIG:
		p_cadr = ardsession[ses_id]->sess_open_info;
		p_cadr_config = &(p_cadr->cad_config);

		p_cadr_config->p_format_block =
			(void *)(g_audio_base + CAD_FORMAT_BLK_OFFSET);
		p_cadr_config->ul_format_block_len = cmd_len;

		p_config_format_block =  (void *)(g_paudio_mem
						+ CAD_FORMAT_BLK_OFFSET);
		memcpy(p_config_format_block, pcmd_buf, cmd_len);
		break;

	default:
		break;
	}

	return rc;
}


s32 ard_read(s32 ses_id, struct cad_buf_struct_type *buf_ptr)
{
	return CAD_RES_SUCCESS;
}

s32 ard_write(s32 ses_id, struct cad_buf_struct_type *buf_ptr)
{
	return CAD_RES_SUCCESS;
}

void ard_callback_func(struct cadi_evt_struct_type *ev_data, void *client_data)
{
}

enum ard_state_ret_enum_type ard_state_control(s32 ses_id)
{
	enum ard_state_ret_enum_type rc;

	rc = ARD_STATE_RC_SUCCESS;

	do {
		switch (ard_state.state) {
		case ARD_STATE_RESET:
			rc = ard_state_reset(ses_id);
			break;
		case ARD_STATE_CLK_ACTIVE:
			rc = ard_state_clk_active(ses_id);
			break;
		case ARD_STATE_AFE_ACTIVE:
			rc = ard_state_afe_active(ses_id);
			break;
		case ARD_STATE_ACTIVE:
			rc = ard_state_active(ses_id);
			break;
		default:
			break;
		}
	} while (rc == ARD_STATE_RC_CONTINUE);

	return rc;
}

enum ard_state_ret_enum_type ard_state_reset(s32 ses_id)
{
	enum ard_state_ret_enum_type	rc;
	enum codec_enum_type		codec_type;

	rc = ARD_STATE_RC_SUCCESS;

	if (valid_session_present()) {
		ard_state.state = ARD_STATE_CLK_ACTIVE;
		codec_type = get_codec_type();
		ard_clk_enable(codec_type);

		rc = ARD_STATE_RC_CONTINUE;
	}
	return rc;
}

enum ard_state_ret_enum_type ard_state_clk_active(s32 ses_id)
{
	enum ard_state_ret_enum_type	rc;
	enum codec_enum_type		codec_type;
	s32 res;

	rc = ARD_STATE_RC_SUCCESS;
	codec_type = CODEC_INT;
	res = CAD_RES_SUCCESS;

	if (valid_session_present()) {

		ard_state.state = ARD_STATE_AFE_ACTIVE;

		res = qdsp6_enable(ses_id);
		if (res == CAD_RES_FAILURE) {

			ard_state.state = ARD_STATE_RESET;

			qdsp6_disable(ses_id);
			codec_type = get_codec_type();

			ard_clk_disable(codec_type);
			rc = ARD_STATE_RC_CONTINUE;
		} else {
			ard_state.afe_enabled = ARD_TRUE;
			rc = ARD_STATE_RC_CONTINUE;
		}
	} else {
		ard_state.state = ARD_STATE_RESET;
		codec_type = get_codec_type();

		ard_clk_disable(codec_type);
		rc = ARD_STATE_RC_CONTINUE;
	}

	return rc;
}

enum ard_state_ret_enum_type ard_state_afe_active(s32 ses_id)
{
	enum ard_state_ret_enum_type	rc;
	enum codec_enum_type		codec_type;
	s32 res;

	rc = ARD_STATE_RC_SUCCESS;
	codec_type = CODEC_INT;
	res = CAD_RES_SUCCESS;

	if (valid_session_present() && ard_state.afe_enabled == ARD_TRUE) {
		ard_state.state = ARD_STATE_ACTIVE;
		codec_type = get_codec_type();
		switch (codec_type) {

		case CODEC_INT:
			rc = adie_open(ses_id);
			rc = adie_enable(ses_id);
			break;
		case CODEC_AUX_PCM:
			break;
		default:
			break;
		}

		if (rc == CAD_RES_FAILURE) {
			ard_state.state = ARD_STATE_CLK_ACTIVE;
			rc = ARD_STATE_RC_CONTINUE;
		} else {
			ard_state.device_configured = ARD_TRUE;
			rc = ARD_STATE_RC_SUCCESS;
		}
	} else {
		ard_state.state = ARD_STATE_CLK_ACTIVE;
		qdsp6_disable(ses_id);
		rc = ARD_STATE_RC_CONTINUE;
	}

	return rc;
}

enum ard_state_ret_enum_type ard_state_active(s32 ses_id)
{
	enum ard_state_ret_enum_type	rc;
	enum codec_enum_type		codec_type;
	s32 res;

	rc = ARD_STATE_RC_SUCCESS;
	codec_type = CODEC_INT;
	res = CAD_RES_SUCCESS;

	if (valid_session_present()
		&& ard_state.device_configured == ARD_TRUE) {
	} else {
		codec_type = get_codec_type();
		switch (codec_type) {

		case CODEC_INT:
			rc = adie_disable(ses_id);
			rc = adie_close(ses_id);
			break;

		case CODEC_AUX_PCM:
			break;
		default:
			break;
		}

		ard_state.state = ARD_STATE_AFE_ACTIVE;
		rc = ARD_STATE_RC_CONTINUE;
	}

	return rc;
}

enum ard_ret_enum_type valid_session_present(void)
{
	u8 i = 0;
	enum ard_ret_enum_type rc;

	rc = ARD_FALSE;

	for (i = 0; i < ARD_AUDIO_MAX_CLIENT; i++) {
		if (ard_session[i].enabled == ARD_TRUE) {
			if (ard_session[i].session_type != DEVICE_CTRL_TYPE) {
				rc = ARD_TRUE;
				break;
			}
		}
	}

	if (i == ARD_AUDIO_MAX_CLIENT)
		rc = ARD_FALSE;

	return rc;
}


enum codec_enum_type get_codec_type()
{
	enum codec_enum_type rc;

	switch (ard_state.device_inuse) {
	case SND_DEVICE_HANDSET:
	case SND_DEVICE_HEADSET_MONO:
	case SND_DEVICE_HEADSET_STEREO:
		rc = CODEC_INT;
		break;
	default:
		rc = CODEC_INT;
		break;
	}

	return rc;
}


s32 qdsp6_enable(s32 ses_id)
{
	struct cadi_evt_struct_type      *p_ev_buf = NULL;
	struct cad_open_struct_type      *p_cad_open = NULL;
	struct cadi_open_struct_type     *p_cadr = NULL;
	s32 rc;

	p_cadr = ardsession[ses_id]->sess_open_info;
	p_ev_buf = ardsession[ses_id]->ev_buf;
	p_cad_open = &(p_cadr->cad_open);

	p_cadr->cad_stream.ul_ses_buf_max_size = 4096;

	rc = cad_rpc_open(ses_id, 1, p_cadr, p_ev_buf);
	if (rc)
		return rc;

	rc = cad_rpc_ioctl(ses_id, 1, CAD_IOCTL_CMD_STREAM_START,
		NULL, 0, p_ev_buf);

	if (rc)
		cad_rpc_close(ses_id, 1, p_ev_buf);

	return rc;
}


s32 qdsp6_disable(s32 ses_id)
{
	struct cadi_evt_struct_type *p_ev_buf;
	s32 rc;

	p_ev_buf = ardsession[ses_id]->ev_buf;

	rc = cad_rpc_close(ses_id, 1, p_ev_buf);

	return rc;
}
