/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <linux/kernel.h>
#include <asm/arch/msm8k_cad_ioctl.h>
#include <asm/arch/msm8k_ard_adiei.h>
#include <asm/arch/msm8k_adie_codec_rpc.h>

struct adie_state_struct_type	adie_state;

#if 0
#define D(fmt, args...) printk(KERN_INFO "adie: " fmt, ##args)
#else
#define D(fmt, args...) do {} while (0)
#endif



s32 adie_init(void)
{
	s32 dal_rc = CAD_RES_SUCCESS;

	dal_rc = daldevice_attach(DALDEVICEID_ADIE_CODEC, ADIE_CODEC_PORT_NAME,
			ADIE_CODEC_CPU, &adie_state.adie_handle);
	if (CAD_RES_SUCCESS != dal_rc)
		D("ARD: ADIE Device Attach failed\n");

	return dal_rc;
}

s32 adie_dinit(void)
{
	s32 dal_rc = CAD_RES_SUCCESS;

	dal_rc = daldevice_detach(adie_state.adie_handle);
	if (CAD_RES_SUCCESS != dal_rc)
		D("ARD: ADIE Device Detach failed\n");

	return dal_rc;
}

s32 adie_open(s32 ses_id)
{
	s32 rc = CAD_RES_SUCCESS;

	rc = daldevice_open(adie_state.adie_handle, 0);

	adie_state.state = ADIE_STATE_RESET;

	if (CAD_RES_SUCCESS != rc)
		D("ARD: ADIE Open failed\n");

	return rc;
}

s32 adie_close(s32 ses_id)
{
	s32 rc = CAD_RES_SUCCESS;

	daldevice_close(adie_state.adie_handle);

	adie_state.state = ADIE_STATE_RESET;
	return rc;
}

s32 adie_enable(s32 ses_id)
{
	s32		rc = CAD_RES_SUCCESS;
	adie_state.enable_request = ADIE_TRUE;

	rc = adie_state_control(ses_id);

	return rc;
}

s32 adie_disable(s32 ses_id)
{
	s32		rc = CAD_RES_SUCCESS;
	adie_state.enable_request = ADIE_FALSE;

	rc = adie_state_control(ses_id);

	return rc;
}

enum adie_state_ret_enum_type adie_state_control(s32 ses_id)
{
	enum adie_state_ret_enum_type	rc = ADIE_STATE_RC_SUCCESS;

	do {
		D("ARD: ses_id %d, adie state %d\n",
					ses_id, adie_state.state);
		switch (adie_state.state) {
		case ADIE_STATE_RESET:
		  rc = adie_state_reset(ses_id);
		  break;
		case ADIE_STATE_DIGITAL_ACTIVE:
		  rc = adie_state_digital_active(ses_id);
		  break;
		case ADIE_STATE_DIGITAL_ANALOG_ACTIVE:
		  rc = adie_state_digital_analog_active(ses_id);
		  break;
		default:
		  break;
		  }
	} while (rc == ADIE_STATE_RC_CONTINUE);

	return rc;
}

enum adie_state_ret_enum_type adie_state_reset(s32 ses_id)
{
	enum adie_state_ret_enum_type	rc;
	s32				dal_rc;
	enum adie_codec_path_type_enum	path_type;
	u32				path_id;
	u32				freq_plan;

	rc = ADIE_STATE_RC_SUCCESS;
	dal_rc = CAD_RES_SUCCESS;
	if (adie_state.enable_request == ADIE_TRUE) {
		path_type = ADIE_CODEC_RX;
		path_id = 16;
		freq_plan = 48000;

		dal_rc = rpc_adie_codec_set_path(adie_state.adie_handle,
				path_id,
				path_type);

		if (dal_rc != CAD_RES_SUCCESS) {
			D("ARD: ADIE Set Path failed for session %d",
				ses_id);

			return rc;
		}

		dal_rc = rpc_adie_codec_set_path_freq_plan(
				adie_state.adie_handle, path_type, freq_plan);

		rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_DIGITAL_READY);
		if (CAD_RES_SUCCESS != dal_rc)
			D("ARD: ADIE Proceed to Stage failed for session %d",
				ses_id);

		adie_state.state = ADIE_STATE_DIGITAL_ACTIVE;

		rc = ADIE_STATE_RC_CONTINUE;
	} else {
		rc = ADIE_STATE_RC_SUCCESS;
	}

	return rc;
}

enum adie_state_ret_enum_type adie_state_digital_active(s32 ses_id)
{
	enum adie_state_ret_enum_type	rc;
	enum adie_codec_path_type_enum	path_type;
	s32				dal_rc;

	rc = ADIE_STATE_RC_SUCCESS;
	dal_rc = CAD_RES_SUCCESS;

	path_type = ADIE_CODEC_RX;
	if (adie_state.enable_request == ADIE_TRUE) {

		rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_DIGITAL_ANALOG_READY);

		adie_state.state = ADIE_STATE_DIGITAL_ANALOG_ACTIVE;
		rc = ADIE_STATE_RC_CONTINUE;
	} else {

		rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_DIGITAL_OFF);

		adie_state.state = ADIE_STATE_RESET;
		rc = ADIE_STATE_RC_CONTINUE;
	}

	return rc;
}

enum adie_state_ret_enum_type adie_state_digital_analog_active(s32 ses_id)
{
	enum adie_codec_path_type_enum	path_type;
	enum adie_state_ret_enum_type   rc = ADIE_STATE_RC_SUCCESS;

	path_type = ADIE_CODEC_LB;
	if (adie_state.enable_request == ADIE_TRUE) {
	} else {

		rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_ANALOG_OFF);

		adie_state.state = ADIE_STATE_DIGITAL_ACTIVE;
		rc = ADIE_STATE_RC_CONTINUE;
	}
	return rc;
}

