/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <linux/kernel.h>
#include <linux/mutex.h>
#include <linux/clk.h>
#include <linux/init.h>
#include "clock.h"
#include <asm/arch/msm8k_ard_clk.h>

static struct clk	*rx_clk;
static struct clk	*tx_clk;


void ard_clk_enable_internal_codec_clk(void)
{
	s32 rc = CAD_RES_SUCCESS;
	rx_clk = clk_get(NULL, "icodec_rx_clk");
	tx_clk = clk_get(NULL, "icodec_tx_clk");

	if (rx_clk == NULL) {
		pr_err("ard_clk: Invalid RX clk!");
		return;
	}

	if (tx_clk == NULL) {
		pr_err("ard_clk: Invalid TX clk!");
		return;
	}

	ard_clk_set_icodec_rx_clk();
	ard_clk_set_icodec_tx_clk();

	rc = clk_enable(rx_clk);
	if (rc)
		pr_err("ard_clk: RX clk not enabled!");

	rc = clk_enable(tx_clk);
	if (rc)
		pr_err("ard_clk: TX clk not enabled!");
}


void ard_clk_disable_internal_codec_clk(void)
{
	clk_disable(rx_clk);
	clk_disable(tx_clk);

	clk_put(rx_clk);
	clk_put(tx_clk);
}


void ard_clk_set_icodec_rx_clk(void)
{
	s32 rc = CAD_RES_SUCCESS;
	/* Frequency in Hz - 48KHz */
	rc = clk_set_rate(rx_clk, 12288000);

	if (rc)
		pr_err("ard_clk: Rate on RX clk not set!");
}


void ard_clk_set_icodec_tx_clk(void)
{
	s32 rc = CAD_RES_SUCCESS;

	/* Frequency in Hz */
	rc = clk_set_rate(tx_clk, 12288000);

	if (rc)
		pr_err("ard_clk: Rate on TX clk not set!");
}


void ard_clk_enable(enum codec_enum_type ct)
{
	switch (ct) {
	case CODEC_INT:
		ard_clk_enable_internal_codec_clk();
		break;
	default:
		break;
	}
}


void ard_clk_disable(enum codec_enum_type ct)
{
	switch (ct) {
	case CODEC_INT:
		ard_clk_disable_internal_codec_clk();
		break;
	default:
		break;
	}
}
