/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <linux/module.h>
#include <linux/mutex.h>
#include <asm/arch/msm8k_cad_module.h>
#include <asm/arch/msm8k_cad_q6dec_drv.h>
#include <asm/arch/msm8k_cad_itypes.h>
#include <linux/platform_device.h>
#include <linux/io.h>

#define MODULE_NAME "CAD"

#if 0
#define D(fmt, args...) printk(KERN_INFO "msm8k_cad: " fmt, ##args)
#else
#define D(fmt, args...) do {} while (0)
#endif

#define CAD_SESSION_INUSE 1
#define CAD_SESSION_FREE  0


struct cad_session_info_struct_type {
	s8				status;
	struct mutex			sync;
	struct cad_func_ptr_tbl_type	*hw_accel;
};

struct cad_state_struct_type {
	struct mutex			sync;
	struct cad_func_ptr_tbl_type	*resource_alloc;
	struct cad_func_ptr_tbl_type	*audiodec;
	struct cad_func_ptr_tbl_type	*audioenc;
	struct cad_func_ptr_tbl_type	*voicedec;
	struct cad_func_ptr_tbl_type	*voiceenc;
	struct cad_func_ptr_tbl_type	*device_filter;
	struct cad_func_ptr_tbl_type	*ard;

	struct cad_session_info_struct_type session_info[CAD_MAX_SESSION];
};

struct cad_state_struct_type cad;
u8 *g_paudio_mem;
u32 g_audio_base;
u32 g_audio_size;

static int __init cad_probe(struct platform_device *pdev)
{
	s32 rc;
	rc = CAD_RES_SUCCESS;

	D("%s: %s called\n", MODULE_NAME, __func__);

	mutex_init(&cad.sync);

	g_audio_base = pdev->resource[0].start;
	g_audio_size = pdev->resource[0].end - pdev->resource[0].start + 1;

	g_paudio_mem = (u8 *)ioremap(g_audio_base, g_audio_size);
	if (g_paudio_mem == NULL)
		return -1;

	cad.ard			= NULL;
	cad.audiodec		= NULL;
	cad.audioenc		= NULL;
	cad.device_filter	= NULL;
	cad.voicedec		= NULL;
	cad.voiceenc		= NULL;
	cad.resource_alloc	= NULL;

	(void)cad_q6_dec_init(&(cad.audiodec));
	(void)cad_ard_init(&(cad.ard));

	return rc;
}


s32 cad_open(struct cad_open_struct_type *p_open_param)
{
	s32 handle;
	s32 handle_temp;
	s8 resource_alloc, ard, hw_accel;
	s32 rc;

	D("%s: %s called\n", MODULE_NAME, __func__);

	rc = CAD_RES_FAILURE;
	handle = handle_temp = resource_alloc = ard = hw_accel = 0;

	mutex_lock(&cad.sync);

	if (p_open_param != NULL) {
		for (handle_temp = 1; handle_temp < CAD_MAX_SESSION;
				handle_temp++) {
			if (CAD_SESSION_FREE ==
					cad.session_info[handle_temp].status) {
				if (CAD_OPEN_OP_READ == p_open_param->op_code)
					cad.session_info[handle_temp].hw_accel
							= cad.audioenc;
				else if (CAD_OPEN_OP_WRITE ==
							p_open_param->op_code)
					cad.session_info[handle_temp].hw_accel
							= cad.audiodec;
				rc = CAD_RES_SUCCESS;
				break;
			}
		}
	} else {
		rc = CAD_RES_FAILURE;
		goto done;
	}

	if (rc == CAD_RES_FAILURE)
		goto done;

	if (cad.resource_alloc != NULL) {
		if (cad.resource_alloc->pfn_open != NULL) {

			rc = cad.resource_alloc->pfn_open(handle_temp,
							p_open_param);

			if (rc == CAD_RES_FAILURE)
				goto done;
			resource_alloc = 1;
		}

	}

	if ((cad.ard != NULL) && (cad.ard->pfn_open != NULL)) {

		rc = cad.ard->pfn_open(handle_temp, p_open_param);

		if (rc == CAD_RES_FAILURE)
			goto done;

		ard = 1;
	}

	if ((cad.session_info[handle_temp].hw_accel != NULL) &&
		(cad.session_info[handle_temp].hw_accel->pfn_open != NULL)) {

		rc = cad.session_info[handle_temp].hw_accel->pfn_open(
						handle_temp, p_open_param);

		if (rc == CAD_RES_FAILURE)
			goto done;

		hw_accel = 1;
	}

	mutex_init(&cad.session_info[handle_temp].sync);

	if (rc == CAD_RES_SUCCESS) {
		handle = handle_temp;
		cad.session_info[handle].status = CAD_SESSION_INUSE;
	}

done:
	/* This could be implemented with multiple goto labels, but that would
	   be ugly.
	*/
	if (rc == CAD_RES_FAILURE) {

		if (ard && cad.ard->pfn_close)
			cad.ard->pfn_close(handle_temp);

		if (hw_accel &&
			cad.session_info[handle_temp].hw_accel->pfn_close) {

			cad.session_info[handle_temp].hw_accel->pfn_close(
								handle_temp);
		}

		if (resource_alloc && cad.resource_alloc->pfn_close)
			cad.resource_alloc->pfn_close(handle_temp);
	}
	mutex_unlock(&cad.sync);

	return handle;
}
EXPORT_SYMBOL(cad_open);




s32 cad_close(s32 driver_handle)
{
	s32 rc;

	D("%s: %s called\n", MODULE_NAME, __func__);

	rc = CAD_RES_SUCCESS;

	if (cad.session_info[driver_handle].hw_accel &&
		cad.session_info[driver_handle].hw_accel->pfn_close)
		cad.session_info[driver_handle].hw_accel->pfn_close(
								driver_handle);

	if (cad.ard && cad.ard->pfn_close)
		(void) cad.ard->pfn_close(driver_handle);

	if (cad.resource_alloc && cad.resource_alloc->pfn_close)
		(void) cad.resource_alloc->pfn_close(driver_handle);

	mutex_lock(&cad.session_info[driver_handle].sync);

	mutex_lock(&cad.sync);

	cad.session_info[driver_handle].status = CAD_SESSION_FREE;
	cad.session_info[driver_handle].hw_accel = NULL;

	mutex_unlock(&cad.session_info[driver_handle].sync);
	mutex_unlock(&cad.sync);

	return rc;
}
EXPORT_SYMBOL(cad_close);



s32 cad_read(s32 driver_handle, struct cad_buf_struct_type *p_buf)
{
	s32 data_read;

	D("%s: %s called\n", MODULE_NAME, __func__);

	data_read = -1;

	if (cad.session_info[driver_handle].hw_accel &&
		cad.session_info[driver_handle].hw_accel->pfn_read)
		data_read = cad.session_info[driver_handle].hw_accel->
					pfn_read(driver_handle, p_buf);

	return data_read;
}
EXPORT_SYMBOL(cad_read);


s32 cad_write(s32 driver_handle, struct cad_buf_struct_type *p_buf)
{
	s32 data_written;

	D("%s: %s called\n", MODULE_NAME, __func__);

	data_written = -1;

	if (cad.session_info[driver_handle].hw_accel &&
		cad.session_info[driver_handle].hw_accel->pfn_write)
		data_written = cad.session_info[driver_handle].hw_accel->
			pfn_write(driver_handle, p_buf);

	else if (cad.ard && cad.ard->pfn_write)
		data_written = cad.ard->pfn_write(driver_handle, p_buf);

	return data_written;
}
EXPORT_SYMBOL(cad_write);


s32 cad_ioctl(s32 driver_handle, u32 cmd_code, void *p_cmd_buf, u32 cmd_buf_len)
{
	s32 ret_val;

	D("%s: %s called\n", MODULE_NAME, __func__);

	ret_val = CAD_RES_SUCCESS;

	mutex_lock(&cad.session_info[driver_handle].sync);

	if (cad.ard && cad.ard->pfn_ioctl)
		ret_val = cad.ard->pfn_ioctl(driver_handle, cmd_code, p_cmd_buf,
								cmd_buf_len);

	if ((ret_val != CAD_RES_FAILURE)
		&& cad.session_info[driver_handle].hw_accel
		&& cad.session_info[driver_handle].hw_accel->pfn_ioctl)
		ret_val = cad.session_info[driver_handle].hw_accel->
				pfn_ioctl(driver_handle, cmd_code, p_cmd_buf,
								cmd_buf_len);

	mutex_unlock(&cad.session_info[driver_handle].sync);

	return ret_val;
}
EXPORT_SYMBOL(cad_ioctl);

static struct platform_driver cad_driver = {
	.probe = cad_probe,
	.driver = {
		.name = "msm_audio",
		.owner = THIS_MODULE,
	},
};

static int __init cad_init(void)
{
	s32 rc;

	rc = platform_driver_register(&cad_driver);

	return rc;
}

static void __exit cad_exit(void)
{
	s32 i = 0;

	D("%s: %s called\n", MODULE_NAME, __func__);

	mutex_unlock(&cad.sync);

	while (i < CAD_MAX_SESSION)
		mutex_unlock(&cad.session_info[i++].sync);

	(void)cad_q6_dec_dinit();
	(void)cad_ard_dinit();
	iounmap(g_paudio_mem);
}


module_init(cad_init);
module_exit(cad_exit);

MODULE_DESCRIPTION("CAD driver");
MODULE_VERSION("1.0");
MODULE_LICENSE("GPL v2");

