/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <asm/arch/msm8k_cad_q6dec_drv.h>
#include <asm/arch/msm8k_cad_q6dec_drvi.h>
#include <asm/arch/msm8k_cad_module.h>
#include <asm/arch/msm8k_cad_q6dec_session.h>
#include <linux/kernel.h>
#include <linux/err.h>
#include <linux/slab.h>
#include <linux/string.h>

static struct cad_dec_drvi_data   cad_q6_dec_data;


static s32 cad_q6_dec_open(s32 session_id,
			struct cad_open_struct_type *open_param)
{
	struct cad_dec_drvi_session_data *session = NULL;
	if (!open_param || session_id >= CAD_MAX_SESSION || session_id <= 0)
		return -1;

	if (CAD_OPEN_OP_WRITE == open_param->op_code &&
			cad_q6_dec_data.free_session_list) {
		session = cad_q6_dec_data.free_session_list;
		if (cad_q6dec_session_open(session, session_id, open_param)) {
			cad_q6dec_session_close(session);
			return -1;
		}
		cad_q6_dec_data.free_session_list =
			cad_q6_dec_data.free_session_list->next;
		session->next = cad_q6_dec_data.used_session_list;
		cad_q6_dec_data.used_session_list = session;
	}
	return 0;
}


static s32 cad_q6_dec_close(s32 session_id)
{
	struct cad_dec_drvi_session_data *session =
	cad_q6_dec_data.used_session_list;
	struct cad_dec_drvi_session_data *prev_session = NULL;
	if (session_id >= CAD_MAX_SESSION || session_id <= 0)
		return -1;

	while (session) {
		if (session->session_id == session_id) {
			if (prev_session == NULL) {
				/* first node */
				cad_q6_dec_data.used_session_list =
					session->next;
				break;
			}
			/* not first node */
			prev_session->next = session->next;
			break;
		}
		prev_session = session;
		session = session->next;
	}

	if (session) {
		cad_q6dec_session_close(session);
		session->next = cad_q6_dec_data.free_session_list;
		cad_q6_dec_data.free_session_list = session;
	}
	return 0;
}

static s32 cad_q6_dec_write(s32 session_id,
			struct cad_buf_struct_type *buf_ptr)
{
	struct cad_dec_drvi_session_data *session =
	cad_q6_dec_data.used_session_list;
	if (!buf_ptr)
		return -1;

	while (session) {
		if (session->session_id == session_id) {
			if (cad_q6dec_session_write(session, buf_ptr))
				return -1;
			break;
		}
		session = session->next;
	}
	return 0;
}

static s32 cad_q6_dec_read(s32 session_id,
			struct cad_buf_struct_type  *buf_ptr)
{
	pr_err("q6decoder read has not implemnted......\n");
	return 0;
}

static s32 cad_q6_dec_ioctl(s32         session_id,
				s32         cmd_code,
				void        *cmd_buf,
				s32         cmd_len)
{
	pr_err("q6decoder ioctl has not implemented......\n");
	return 0;
}


/* public functions */

s32 cad_q6_dec_dinit(void)
{
	struct cad_dec_drvi_session_data *session;
	while (cad_q6_dec_data.free_session_list) {
		session = cad_q6_dec_data.free_session_list->next;
		cad_q6dec_session_deinit(cad_q6_dec_data.free_session_list);
		kfree(cad_q6_dec_data.free_session_list);
		cad_q6_dec_data.free_session_list = session;
	}

	while (cad_q6_dec_data.used_session_list) {
		session = cad_q6_dec_data.used_session_list->next;
		cad_q6dec_session_deinit(cad_q6_dec_data.used_session_list);
		kfree(cad_q6_dec_data.used_session_list);
		cad_q6_dec_data.used_session_list = session;
	}

	memset(&cad_q6_dec_data, 0, sizeof(struct cad_dec_drvi_data));
	return 0;
}


s32 cad_q6_dec_init(struct cad_func_ptr_tbl_type **ppvtable)
{
	u32 i;
	struct cad_dec_drvi_session_data *pnode = NULL;
	static struct cad_func_ptr_tbl_type vtable = {
		cad_q6_dec_open,
		cad_q6_dec_close,
		cad_q6_dec_write,
		cad_q6_dec_read,
		cad_q6_dec_ioctl
	};
	*ppvtable = NULL;

	memset(&cad_q6_dec_data, 0, sizeof(struct cad_dec_drvi_data));

	/* create session list */
	for (i = 0; i < CAD_DEC_DRVI_MAX_STREAM_COUNT; i++) {
		pnode = kmalloc(sizeof(struct cad_dec_drvi_session_data),
				GFP_KERNEL|GFP_DMA);
		if (IS_ERR(pnode)) {
			cad_q6_dec_dinit();
			return -1;
		}

		memset(pnode, 0,
			sizeof(struct cad_dec_drvi_session_data));

		if (cad_q6dec_session_init(pnode)) {
			cad_q6dec_session_deinit(pnode);
			kfree(pnode);
			return CAD_RES_FAILURE;
		}

		pnode->next = cad_q6_dec_data.free_session_list;
		cad_q6_dec_data.free_session_list = pnode;
	}

	*ppvtable = &vtable;
	return 0;
}
