/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <asm/arch/msm8k_cad.h>
#include <asm/arch/msm8k_cad_q6dec_session.h>
#include <asm/arch/msm8k_cad_rpc.h>
#include <linux/slab.h>
#include <linux/dma-mapping.h>
#include <linux/err.h>
#include <linux/uaccess.h>

static void (*event_cb)(void);

void register_cb(void *cb)
{
	event_cb = cb;
}
EXPORT_SYMBOL(register_cb);


static void cad_q6dec_session_async_callback(struct cadi_evt_struct_type *evt,
							void *data)
{
	static u32 counter = 1;
	struct cad_dec_drvi_session_data *self =
		(struct cad_dec_drvi_session_data *) data;
	struct cad_dec_drvi_sesson_buffer_node *pnode = NULL;
	struct cad_dec_drvi_sesson_buffer_node *prev_node = NULL;

	if (CAD_EVT_STATUS_BUF_DONE !=  evt->cad_event_header.cmd_event_id)
		/* unknow event, and do nothing */
		return;

	mutex_lock(&self->session_mutex);
	if (CAD_DEC_DRVI_RESET == self->session_state) {
		pr_err(" the sessino is reseted!!!!!!!\n");
		mutex_unlock(&self->session_mutex);
		return;
	}

	pnode = self->used_buf_list;
	while (pnode) {
		if (pnode->buf == evt->cad_event_data.buf_data.buffer) {
			if (prev_node == NULL) {
				/* first node */
				self->used_buf_list = pnode->next;
				break;
			}
			prev_node->next = pnode->next;
			break;
		}
		prev_node = pnode;
		pnode = pnode->next;
	}

	if (pnode) {
		pr_err("========>return buffer number %d, 0x%x\n",
			counter, (u32) pnode);
		counter++;
		pnode->next = self->free_buf_list;
		self->free_buf_list = pnode;
		up(&self->free_buff_count_mutex);
		if (self->used_buf_list == NULL) {
			if (CAD_DEC_DRVI_CLOSING == self->session_state) {
				pr_err("===> Signal all buffer done event");
				up(&self->all_buf_done_mutex);
			} else {
				self->session_state = CAD_DEC_DRVI_READY;
			}
		}
	} else {
		pr_err("\n--------callback semi count skip--------\n");
	}
	mutex_unlock(&self->session_mutex);

	if (event_cb != NULL)
		event_cb();

	return;
}


static struct cad_dec_drvi_sesson_buffer_node *cad_q6dec_session_get_free_buf(
	struct cad_dec_drvi_session_data *self)
{
	struct cad_dec_drvi_sesson_buffer_node *pnode = NULL;
	mutex_lock(&self->session_mutex);
	if (self->free_buf_list) {
		pnode = self->free_buf_list;
		self->free_buf_list = self->free_buf_list->next;
		pnode->next = NULL;
	}
	mutex_unlock(&self->session_mutex);
	return pnode;
}

static s32 cad_q6dec_session_send_buf(struct cad_dec_drvi_session_data *self,
				struct cad_dec_drvi_sesson_buffer_node *pnode,
				u32 buf_len)
{
	struct cad_buf_struct_type send_buf;
	memset(&send_buf, 0, sizeof(struct cad_buf_struct_type));
	send_buf.buffer = pnode->buf;

	send_buf.phys_addr = pnode->phys_addr;

	send_buf.max_size = buf_len;

	/* add to the used list*/
	mutex_lock(&self->session_mutex);
	pnode->next = self->used_buf_list;
	self->used_buf_list = pnode;
	mutex_unlock(&self->session_mutex);
	pr_err("----> Send %d byte of data\n", buf_len);
	/* rpc wirte*/
	return cad_rpc_write(self->session_id, 0, &send_buf, NULL);
}

/*===========================================================================*/
/* These are public APIs */
s32 cad_q6dec_session_open(struct cad_dec_drvi_session_data *self,
				u32 session_id,
				struct cad_open_struct_type *open_param)
{
	u32 i;
	s32 err = 0;
	struct cad_dec_drvi_sesson_buffer_node *pnode = NULL;

	/* initialize the sempaphore and mutex */
	mutex_init(&self->session_mutex);
	sema_init(&self->free_buff_count_mutex, 0);

	/* create the shared memory region */
	self->shared_buf =  g_paudio_mem + (4096 + session_id * 4096);

	if (self->shared_buf == NULL)
		return -1;

	memset(self->shared_buf, 0, CAD_DEC_DRVI_BUFFER_NUM_PER_STREAM
		* CAD_DEC_DRVI_BUFFER_SIZE);

	/* create buffer node */
	for (i = 0; i < CAD_DEC_DRVI_BUFFER_NUM_PER_STREAM; i++) {
		pnode = kmalloc(sizeof(struct cad_dec_drvi_sesson_buffer_node),
				GFP_KERNEL|GFP_DMA);
		if (IS_ERR(pnode)) {
			err = -1;
			break;
		}
		memset(pnode, 0,
			sizeof(struct cad_dec_drvi_sesson_buffer_node));
		pnode->next = self->free_buf_list;
		self->free_buf_list = pnode;
		/* assign the shared buffer */
		pnode->buf = self->shared_buf + CAD_DEC_DRVI_BUFFER_SIZE * i;
		pnode->phys_addr = g_audio_base + (4096 + session_id * 4096)
					+ CAD_DEC_DRVI_BUFFER_SIZE * i;
	}

	if (!err)
		err = cad_rpc_reg_callback(session_id,
				cad_q6dec_session_async_callback,
				self);

	if (!err) {
		self->session_id = session_id;
		self->session_state = CAD_DEC_DRVI_READY;
		pr_err("Session open successful\n");
		return 0;
	}

	/* release shared memory */

	while (self->free_buf_list) {
		pnode = self->free_buf_list;
		self->free_buf_list = self->free_buf_list->next;
		kfree(pnode);
	}
	cad_rpc_dereg_callback(session_id, cad_q6dec_session_async_callback);
	self->session_id = 0;
	self->session_state = CAD_DEC_DRVI_RESET;
	return -1;
}

s32 cad_q6dec_session_close(struct cad_dec_drvi_session_data *self)
{
	struct cad_dec_drvi_sesson_buffer_node *pnode = NULL;

	mutex_lock(&self->session_mutex);
	if (CAD_DEC_DRVI_PROCESSING == self->session_state) {
		pr_err("-------> wait for all buf done\n");
		sema_init(&self->all_buf_done_mutex, 0);
		self->session_state = CAD_DEC_DRVI_CLOSING;
	} else {
		goto session_close;
	}
	mutex_unlock(&self->session_mutex);
	down_interruptible(&self->all_buf_done_mutex);
	mutex_lock(&self->session_mutex);
session_close:
	while (self->free_buf_list) {
		pnode = self->free_buf_list;
		self->free_buf_list = self->free_buf_list->next;
		kfree(pnode);
	}
	while (self->used_buf_list) {
		pnode = self->used_buf_list;
		self->used_buf_list = self->used_buf_list->next;
		kfree(pnode);
	}

	cad_rpc_dereg_callback(self->session_id,
			       cad_q6dec_session_async_callback);
	pr_err("Session close successful\n");
	self->session_id = 0;
	self->session_state = CAD_DEC_DRVI_RESET;
	mutex_unlock(&self->session_mutex);
	return 0;
}


s32 cad_q6dec_session_write_buf(struct cad_dec_drvi_session_data *self,
				u8  **input_buf,
				u32 *input_buf_size)
{
	static u32 counter = 1;
	s32 err = 0;
	unsigned long n;        /* to get rid of warning */
	struct cad_dec_drvi_sesson_buffer_node *pnode = NULL;
	while ((pnode = cad_q6dec_session_get_free_buf(self)) != NULL) {
		pr_err("==========> use buffer number %d, 0x%x\n",
		       counter, (u32) pnode);

		if (self->free_buff_count_mutex.count.counter == 0)
			pr_err("\nFROZE AFTER GETTING A FREE BUFFER!!!\n");

		counter++;
		/* copy the data */
		if (*input_buf_size > CAD_DEC_DRVI_BUFFER_SIZE) {

			n = copy_from_user(pnode->buf, *input_buf,
					   CAD_DEC_DRVI_BUFFER_SIZE);

			err = cad_q6dec_session_send_buf(
				self, pnode, CAD_DEC_DRVI_BUFFER_SIZE);

			*input_buf += CAD_DEC_DRVI_BUFFER_SIZE;
			*input_buf_size -= CAD_DEC_DRVI_BUFFER_SIZE;
		} else {
			n = copy_from_user(pnode->buf, *input_buf,
				*input_buf_size);
			err = cad_q6dec_session_send_buf(
				self, pnode, *input_buf_size);
			*input_buf_size = 0;
			break;
		}
		if (err)
			break;
	}
	return err;
}

s32 cad_q6dec_session_write(struct cad_dec_drvi_session_data *self,
				struct cad_buf_struct_type *buffer)
{
	s32 err = 0;
	u8 *buf = (u8 *)buffer->buffer;
	u32 buf_size = buffer->max_size;

	pr_err("========> Start send %d byte buffer data......\n", buf_size);
	while (buf_size > 0) {
		err = cad_q6dec_session_write_buf(self, &buf, &buf_size);
		if (err || (buf_size == 0))
			break;

		if (self->free_buff_count_mutex.count.counter == 0)
			pr_err("Waiting for a free buffer.\n");

		/* block for semaphore */
		down_interruptible(&self->free_buff_count_mutex);
	}
	return err;
}

s32 cad_q6dec_session_init(struct cad_dec_drvi_session_data *self)
{
	return 0;
}


s32 cad_q6dec_session_deinit(struct cad_dec_drvi_session_data *self)
{
	if (CAD_DEC_DRVI_RESET !=  self->session_state)
		cad_q6dec_session_close(self);

	return 0;
}
