/*
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 *
 */

#include <linux/kernel.h>
#include <linux/mutex.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/slab.h>
#include <linux/dma-mapping.h>
#include <asm/semaphore.h>

#include <asm/arch/msm8k_cad_rpc.h>
#include <asm/arch/msm8k_cad_rpc_type.h>
#include <asm/arch/dal.h>

#define  SYNC_EVENT_CLEAN_UP_TIMEOUT      1000      /* timeout value in us */
#define  SYNC_EVENT_WAIT_TIMEOUT          50000000 /* timeout value in us */

/* this will be replace by include file daldeviceid.h later*/
#define DALDEVICEID_AUDIO_QDSP          0x02000028


/* rpc table index */
enum {
	CAD_RPC_Q6_OPEN = DALDEVICE_FIRST_DEVICE_API_IDX,
	CAD_RPC_Q6_WRITE,
	CAD_RPC_Q6_READ,
	CAD_RPC_Q6_IOCTL,
	CAD_RPC_Q6_INIT,
	CAD_RPC_Q6_CLOSE,
	CAD_RPC_Q6_FLUSH_BUF
};

struct callback_function_node;
struct callback_function_node {
	RPC_CB_FCN                      cb_funct;
	void                            *client_data;
	struct callback_function_node   *next;
};


struct async_evt_node {
	struct cadi_evt_struct_type	*evt;
	struct async_evt_node		*next;
};

struct cad_rpc_data_struct {
	enum cad_rpc_process_type       processor_id;
	struct mutex                    resource_mutex;
	u32                             initialized;
	void				*remote_handle_list[CAD_MAX_SESSION];

	/* 0 = WL wakeup event, 1...CAD_MAX_SESSION = normal sync wakeup evnt*/
	struct semaphore		sem_list[CAD_MAX_SESSION];
	/* sync mutex for each session call */
	struct mutex			dal_remote_mutex_list[CAD_MAX_SESSION];

	struct async_evt_node		*async_evt_queue;
	struct cadi_evt_struct_type	sync_evt_queue[CAD_MAX_SESSION];

	struct callback_function_node   *dal_callback_list[CAD_MAX_SESSION];

	/*	pointer to async kthread	*/
	struct task_struct              *async_task;
	s32 (*async_cb)(void *data);
};


static struct cad_rpc_data_struct cad_rpc_data;

static void remote_cb_function(void *context, u32 dwParam,
					void *evt_buf, u32 len)
{
	struct cad_rpc_data_struct *self = context;
	struct cadi_evt_struct_type *evt = evt_buf;
	struct async_evt_node *node = NULL;
	if (dwParam == (u32)&self->sem_list[0]) {
	/* async event */
		node = kmalloc(sizeof(*node), GFP_KERNEL|GFP_DMA);
		if (!node) {
			pr_err("Can not allocate memory for async event");
			return;
		}
		memset(node, 0, sizeof(*node));
		node->evt = kmalloc(sizeof(*node->evt), GFP_KERNEL|GFP_DMA);
		if (!node->evt) {
			kfree(node);
			return;
		}

		memset(node->evt, 0, sizeof(*node->evt));
		memcpy(node->evt, evt, sizeof(*node->evt));

		mutex_lock(&cad_rpc_data.resource_mutex);
		node->next = self->async_evt_queue;
		self->async_evt_queue = node;
		mutex_unlock(&cad_rpc_data.resource_mutex);

		up((struct semaphore *)dwParam);
	} else {
		if (dwParam != (u32)&self->sem_list
			[evt->cad_event_header.cad_handle]) {

			pr_err("wrong sync event received!!!");
			return;
		}
		/* sync event */
		mutex_lock(&cad_rpc_data.resource_mutex);
		memcpy(&self->sync_evt_queue[evt->cad_event_header.cad_handle],
			evt, sizeof(*evt));
		mutex_unlock(&cad_rpc_data.resource_mutex);
		up((struct semaphore *)dwParam);
	}
	return;
}


/* main entry function for the thread */
static s32 thread_main(void *data)
{
	struct cad_rpc_data_struct *this = (struct cad_rpc_data_struct *)data;

	pr_err("Starting async handler thread......\n");
	for (;;) {
		down_interruptible(&cad_rpc_data.sem_list[0]);
		this->async_cb(this);
	}
	/* wait on mutiple event for event*/
	return 0;
}

static s32 cad_rpc_get_async_evnt(struct async_evt_node **data)
{
	mutex_lock(&cad_rpc_data.resource_mutex);
	if (cad_rpc_data.async_evt_queue == NULL) {
		pr_err("Not more async event!!!");
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return CAD_RES_FAILURE;
	}
	*data = cad_rpc_data.async_evt_queue;
	cad_rpc_data.async_evt_queue = cad_rpc_data.async_evt_queue->next;
	mutex_unlock(&cad_rpc_data.resource_mutex);
	return CAD_RES_SUCCESS;
}


static s32 cad_rpc_async_callback(void *data)
{
	static u32  counter = 1;  /* this is the test counter */
	struct callback_function_node *pnode = NULL;
	struct cad_rpc_data_struct  *me = (struct cad_rpc_data_struct *)data;
	struct async_evt_node *event_payload;
	pr_err("<-------------- ARM Async callback function fired.......");

	mutex_lock(&me->resource_mutex);
	if (!me->initialized) {
		mutex_unlock(&me->resource_mutex);
		return CAD_RES_FAILURE;
	}
	mutex_unlock(&me->resource_mutex);

	while (CAD_RES_SUCCESS == cad_rpc_get_async_evnt(&event_payload)) {
		pr_err("---> process number %d event\n", counter++);
		if (event_payload->evt->cad_event_header.cad_handle > 0
			&& event_payload->evt->cad_event_header.cad_handle
				< CAD_MAX_SESSION) {

			mutex_lock(&me->resource_mutex);
			pnode = me->dal_callback_list[event_payload->evt->
				cad_event_header.cad_handle];

			while (pnode) {
				pnode->cb_funct(event_payload->evt,
					pnode->client_data);

				pnode = pnode->next;
			}
			mutex_unlock(&me->resource_mutex);
		}
		kfree(event_payload->evt);
		kfree(event_payload);
	}
	return 0;
}

static s32 cad_rpc_get_remote_handle(u32 session_id)
{
	s32 err = 0;
	struct cad_rpc_config_info info;
	if (session_id >= CAD_MAX_SESSION)
		return CAD_RES_FAILURE;

	memset(&info, 0, sizeof(struct cad_rpc_config_info));

	mutex_lock(&cad_rpc_data.resource_mutex);
	if (!cad_rpc_data.initialized) {
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return CAD_RES_FAILURE;
	}

	/* already exist the handle */
	if (cad_rpc_data.remote_handle_list[session_id]) {
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return 0;
	}

	/* get device handle */
	/* <peter> N-way SMD dependent.*/
	err = daldevice_attach(DALDEVICEID_AUDIO_QDSP,
		"DAL_AQ_AUD",
		DALRPC_DEST_QDSP,
		&(cad_rpc_data.remote_handle_list[session_id]));
	if (err) {
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return err;
	}

	/*  get physical addresss of the buffer */
	info.session_id = session_id;
	info.processor_id = cad_rpc_data.processor_id;
	info.cb_evt = dalrpc_alloc_cb(cad_rpc_data.
		remote_handle_list[session_id],
		remote_cb_function, &cad_rpc_data);
	info.local_trigger_evt = (u32) &cad_rpc_data.sem_list[session_id];

	/* initlize the rpc call */
	err = dalrpc_fcn_5(CAD_RPC_Q6_INIT,
		cad_rpc_data.remote_handle_list[session_id],
		(void *)&info,
		sizeof(struct cad_rpc_config_info));
	mutex_unlock(&cad_rpc_data.resource_mutex);
	return err;
}

/* public functions */
s32 cad_rpc_init(u32 processor_id)
{
	u32 i;
	memset(&cad_rpc_data, 0, sizeof(cad_rpc_data));

	if (processor_id >= CAD_RPC_PROCESSPR_MAX)
		return CAD_RES_FAILURE;

	cad_rpc_data.processor_id = processor_id;

	cad_rpc_data.async_cb = cad_rpc_async_callback;

	/* create mutex for data resource */
	mutex_init(&cad_rpc_data.resource_mutex);

	for (i = 0; i < CAD_MAX_SESSION; i++) {
		mutex_init(&cad_rpc_data.dal_remote_mutex_list[i]);
		sema_init(&cad_rpc_data.sem_list[i], 0);
	}

	cad_rpc_data.initialized = 1;

	/* handle zero for async rpc dh */
	if (cad_rpc_get_remote_handle(0)) {
		cad_rpc_deinit();
		return -1;
	}

	/* create kthread for async handling */
	cad_rpc_data.async_task = kthread_run(thread_main,
					&cad_rpc_data, "cad_rpc_task");
	if (IS_ERR(cad_rpc_data.async_task))
		return -1;

	pr_err("DALRPC Interface Initialized!!!");
	return 0;
}


/* this function is called when everything is going away.*/
s32 cad_rpc_deinit()
{
	u32 i;
	struct callback_function_node  *pnode = NULL;

	pr_err("cad deinit function fired   ....... \n");
	/* stop async thread */
	kthread_stop(cad_rpc_data.async_task);

	for (i = 0; i < CAD_MAX_SESSION; i++) {

		/* release remote device handle*/
		if (cad_rpc_data.remote_handle_list[i])
			daldevice_detach(cad_rpc_data.remote_handle_list[i]);

		/* clean callback  function array */
		while (cad_rpc_data.dal_callback_list[i]) {
			pnode = cad_rpc_data.dal_callback_list[i];
			cad_rpc_data.dal_callback_list[i] =
			(cad_rpc_data.dal_callback_list[i])->next;
			kfree(pnode);
		}
	}

	/*free shared memory first, then close file handle */

	pr_err("DeInit the rpc resource interface!!!!!\n");
	return 0;
}

s32 cad_rpc_reg_callback(u32 session_id, RPC_CB_FCN cbFCN, void *client_data)
{
	s32 err = 0;
	struct callback_function_node  *pnode = NULL;
	if (!cbFCN && (session_id >= CAD_MAX_SESSION))
		return -1;

	mutex_lock(&cad_rpc_data.resource_mutex);
	if (cad_rpc_data.dal_callback_list[session_id]) {
		/* check if duplicated callback registration */
		pnode = cad_rpc_data.dal_callback_list[session_id];
		while (pnode) {
			if (pnode->cb_funct == cbFCN) {
				/* duplicated */
				err = -1;
				break;
			}
			pnode = pnode->next;
		}
	}
	if (err) {
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return err;
	}
	pnode = kmalloc(sizeof(struct callback_function_node),
				GFP_KERNEL|GFP_DMA);
	if (IS_ERR(pnode)) {
		mutex_unlock(&cad_rpc_data.resource_mutex);
		return -1;
	}
	memset(pnode, 0, sizeof(struct callback_function_node));
	pnode->cb_funct = cbFCN;
	pnode->client_data = client_data;
	if (cad_rpc_data.dal_callback_list[session_id]) {
		/* not first one */
		pnode->next = cad_rpc_data.dal_callback_list[session_id];
		cad_rpc_data.dal_callback_list[session_id] = pnode;
	} else {
		/* first one */
		cad_rpc_data.dal_callback_list[session_id] = pnode;
	}

	mutex_unlock(&cad_rpc_data.resource_mutex);
	pr_err("Registered the async callback function!!!");
	return 0;
}

s32 cad_rpc_dereg_callback(u32 session_id, RPC_CB_FCN cbFCN)
{
	struct callback_function_node *pnode = NULL;
	struct callback_function_node *pprev = NULL;
	if (!cbFCN && session_id >= CAD_MAX_SESSION)
		return -1;

	mutex_lock(&cad_rpc_data.resource_mutex);
	pnode = cad_rpc_data.dal_callback_list[session_id];
	while (pnode) {
		if (pnode->cb_funct == cbFCN) {
			if (pprev == NULL) {
				/* first node */
				cad_rpc_data.dal_callback_list[session_id] =
					pnode->next;
				kfree(pnode);
				break;
			}
			/* remove the none first node */
			pprev->next = pnode->next;
			kfree(pnode);
			break;
		}
		pprev = pnode;
		pnode = pnode->next;
	}
	mutex_unlock(&cad_rpc_data.resource_mutex);
	pr_err("DeRegistered the async callback function!!!");
	return 0;
}

/*===========================================================================*/
/* RPC Interface functions: */
/*===========================================================================*/
s32 cad_rpc_open(u32 session_id,     /* session handle */
		u32 block_flag,     /* 0=none block, 1=block */
		struct cadi_open_struct_type *open_buf,
		struct cadi_evt_struct_type *ret_status)
{
	s32 err = 0;
	if ((session_id == 0) || cad_rpc_get_remote_handle(session_id))
		return -1;

	if (!open_buf && !ret_status)
		return -1;

	pr_err("DALRPC Open function entering!!!");
	mutex_lock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	pr_err("DALRPC Open function entering!!!");

	/* clean up any pending signal */

	/* making rpc open call*/
	err = dalrpc_fcn_5(CAD_RPC_Q6_OPEN,
		cad_rpc_data.remote_handle_list[session_id],
		(void *)open_buf,
		sizeof(struct cadi_open_struct_type));
	if (!err) {
		pr_err("DALRPC Open function start wait!!!");
		err = down_interruptible(&cad_rpc_data.sem_list[session_id]);
		if (err != -ETIME) {
			pr_err("Got wakeup signal for Open blocking!!!");
			memcpy(ret_status,
				&cad_rpc_data.sync_evt_queue[session_id],
				sizeof(*ret_status));

			memset(&cad_rpc_data.sync_evt_queue[session_id], 0,
				sizeof(*ret_status));
		}
	}
	mutex_unlock(&cad_rpc_data.dal_remote_mutex_list[session_id]);
	return err;
}

s32 cad_rpc_read(u32 session_id,
			u32 block_flag,    /* none block, 1=block */
			struct cad_buf_struct_type *read_buf,
			struct cadi_evt_struct_type *ret_status)
{
	s32 err = 0;

	if ((session_id == 0) || cad_rpc_get_remote_handle(session_id))
		return -1;

	if (!read_buf && !ret_status)
		return -1;

	mutex_lock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	pr_err("DALRPC READ function Entering!!!");

	/* making rpc read call*/
	err = dalrpc_fcn_5(CAD_RPC_Q6_READ,
			cad_rpc_data.remote_handle_list[session_id],
			(void *)read_buf,
			sizeof(struct cad_buf_struct_type));

	mutex_unlock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	return err;
}

s32 cad_rpc_write(u32 session_id,
			u32 block_flag,   /* 0=none block, 1=block */
			struct cad_buf_struct_type *write_buf,
			struct cadi_evt_struct_type *ret_status)
{
	s32 err = 0;

	if ((session_id == 0) || cad_rpc_get_remote_handle(session_id))
		return -1;

	if (!write_buf && !ret_status)
		return -1;

	mutex_lock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	pr_err("DALRPC WRite function Entering!!!");

	/* making rpc write call */
	err = dalrpc_fcn_5(CAD_RPC_Q6_WRITE,
			cad_rpc_data.remote_handle_list[session_id],
			(void *)write_buf,
			sizeof(struct cad_buf_struct_type));

	mutex_unlock(&cad_rpc_data.dal_remote_mutex_list[session_id]);
	return err;
}

s32 cad_rpc_ioctl(u32 session_id,
			u32 block_flag,   /* 0=none block, 1=block */
			u32 cmd_code,
			u8  *cmd_buf,
			u32 cmd_buf_len,
			struct cadi_evt_struct_type *ret_status)
{
	s32 err = 0;
	if ((session_id == 0) || cad_rpc_get_remote_handle(session_id))
		return -1;

	if (!cmd_buf && !ret_status)
		return -1;

	mutex_lock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	pr_err("DALRPC IOCTL function Entering!!!");

	/* clean up any delayed signal */

	err = dalrpc_fcn_6(CAD_RPC_Q6_IOCTL,
			cad_rpc_data.remote_handle_list[session_id],
			cmd_code,
			(void *)cmd_buf,
			cmd_buf_len);
	if (!err) {
		pr_err("DALRPC IOCTL function start wait!!!");
		err = down_interruptible(&cad_rpc_data.sem_list[session_id]);
		if (err != -ETIME) {
			pr_err("Got wake up signal for IOCTL blocking!!!");
			memset(ret_status, 0,
				sizeof(struct cadi_evt_struct_type));
			memcpy(ret_status,
				&cad_rpc_data.sync_evt_queue[session_id],
				sizeof(*ret_status));
			memset(&cad_rpc_data.sync_evt_queue[session_id], 0,
				sizeof(*ret_status));
		}
	}
	mutex_unlock(&cad_rpc_data.dal_remote_mutex_list[session_id]);
	return err;
}

s32 cad_rpc_close(u32 session_id,
		u32 block_flag,   /* 0=none block, 1=block */
		struct cadi_evt_struct_type *ret_status)
{
	s32 err = 0;
	if ((session_id == 0) || cad_rpc_get_remote_handle(session_id))
		return -1;

	if (!ret_status)
		return -1;

	mutex_lock(&cad_rpc_data.dal_remote_mutex_list[session_id]);

	pr_err("DALRPC CLOSE function Entering!!!");

	/* clean up any delayed signal */

	/* making rpc close call */
	err = dalrpc_fcn_0(CAD_RPC_Q6_CLOSE,
			cad_rpc_data.remote_handle_list[session_id],
			session_id);
	if (!err) {
		pr_err("DALRPC Close function start wait!!!");
		err = down_interruptible(&cad_rpc_data.sem_list[session_id]);
		if (err != -ETIME) {
			pr_err("Got wake up signal for Close blocking!!!");
			memset(ret_status, 0,
				sizeof(struct cadi_evt_struct_type));
			memcpy(ret_status,
				&cad_rpc_data.sync_evt_queue[session_id],
				sizeof(*ret_status));
			memset(&cad_rpc_data.sync_evt_queue[session_id], 0,
				sizeof(*ret_status));
		}
	}
	mutex_unlock(&cad_rpc_data.dal_remote_mutex_list[session_id]);
	return err;
}
