/* arch/arm/mach-msm/mpp.c
 *
 * Copyright (c) 2008 QUALCOMM Incorporated.
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 */

/* Qualcomm PMIC Multi-Purpose Pin Configurations */

#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/init.h>
#include <linux/debugfs.h>

#include "proc_comm.h"

#define  PCOM_HUAWEI_CMD PCOM_CUSTOMER_CMD1

typedef enum
{
  PROC_COMM_HUAWEI_IS_POWEROFF_CHARGING,
  PROC_COMM_HUAWEI_IS_LOW_POWER,
  PROC_COMM_HUAWEI_WPS_LED_ONOFF,
#ifdef FEATURE_FLUX_COLLECT
  PROC_COMM_HUAWEI_POWER_OFF=4,
#endif
  PROC_COMM_HUAWEI_NUM_CMDS
}proc_comm_huawei_cmd_type;


static struct huawei_cmd {
	const char *name;
	unsigned id;
	int status;
};

#define CMD(_name, _id, _status) { .name = _name, .id = _id, .status = _status}

static struct huawei_cmd huawei_cmds[] = {
  CMD("is_poweroff_charging", 0, 0),
  CMD("is_low_power", 1, 0),
	CMD("wps_led_onoff", 2, 0),
#ifdef FEATURE_FLUX_COLLECT
  CMD("is_power_off", PROC_COMM_HUAWEI_POWER_OFF, 0),
#endif
};

struct huawei_cmd *huawei_cmd_get(struct device *dev, const char *id)
{
	int n;
	for (n = 0; n < ARRAY_SIZE(huawei_cmds); n++) {
		if (!strcmp(huawei_cmds[n].name, id))
			return huawei_cmds + n;
	}
	return NULL;
}
EXPORT_SYMBOL(huawei_cmd_get);

int rpc_huawei_cmd(proc_comm_huawei_cmd_type cmd, unsigned param)
{
	int err;
	err = msm_proc_comm(PCOM_HUAWEI_CMD, &cmd, &param);
	huawei_cmds[cmd].status = param;
	return err;
}
EXPORT_SYMBOL(rpc_huawei_cmd);

#if defined(CONFIG_DEBUG_FS)
static int rpc_huawei_cmd_debug_set(void *data, u64 val)
{
	int err;
	struct huawei_cmd *cmd = data;

	err = rpc_huawei_cmd(cmd->id, (unsigned)val);
	if (err) {
		printk(KERN_ERR
			   "%s: rpc_huawei_cmd \
			   [%s(%d) = 0x%x] failed\n",
			   __func__, cmd->name, cmd->id, (unsigned)val);
	}
	return 0;
}

static int rpc_huawei_cmd_debug_get(void *data, u64 *val)
{
	struct huawei_cmd *cmd = data;
	//int status = cmd->status;
 
	*val = cmd->status;
 
	return 0;
}

DEFINE_SIMPLE_ATTRIBUTE(rpc_huawei_cmd_fops, rpc_huawei_cmd_debug_get, rpc_huawei_cmd_debug_set, "%llu\n");

static int __init rpc_huawei_cmd_debug_init(void)
{
	struct dentry *dent;
	int n;

	dent = debugfs_create_dir("rpc_huawei_cmd", 0);
	if (IS_ERR(dent))
		return 0;

	for (n = 0; n < ARRAY_SIZE(huawei_cmds); n++)
		debugfs_create_file(huawei_cmds[n].name, 0644, dent, huawei_cmds + n,
				    &rpc_huawei_cmd_fops);

	return 0;
}

device_initcall(rpc_huawei_cmd_debug_init);
#endif
