/* drivers/char/diag/diagfwd.c */

/* Copyright (c) 2008 QUALCOMM USA, INC. 
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/workqueue.h>
#include <linux/diagchar.h>
#include <asm/arch/usbdiag.h>
#include <asm/arch/msm_smd.h>
#include "diagmem.h"
#include "diagchar.h"
#include "diagfwd.h"
#include "diagchar_hdlc.h"


MODULE_DESCRIPTION("Diag Char Driver");
MODULE_LICENSE("GPL v2");
MODULE_VERSION("1.0");

/* Size of the USB buffers used for read and write*/
#define USB_MAX_BUF 4096

/* Size of the buffer used for deframing a packet
  reveived from the PC tool*/
#define HDLC_MAX 4096

/* Number of maximum USB requests that the USB layer should handle at
   one time. */
#define MAX_DIAG_USB_REQUESTS 12

static void diag_smd_send_req(void)
{
	void *buf;

	if (driver->ch && (!driver->in_busy)) {
		int r = smd_read_avail(driver->ch);

		if (r > USB_MAX_BUF) {
			printk(KERN_INFO "diag dropped num bytes = %d\n", r);
			return;
		}
		if (r > 0) {

			buf = driver->usb_buf_in;
			if (!buf) {
				printk(KERN_INFO "Out of diagmem\n");
			} else {
				smd_read_from_cb(driver->ch, buf, r);
				driver->in_busy = 1;
				diag_write(buf, r);
			}
		}
	}
}

static void diag_print_mask_table(void)
{
/* Enable this to print mask table when updated */
#ifdef MASK_DEBUG
	int first;
	int last;
	uint8_t *ptr = driver->msg_masks;
	int i = 0;

	while (*(uint32_t *)(ptr + 4)) {
		first = *(uint32_t *)ptr;
		ptr += 4;
		last = *(uint32_t *)ptr;
		ptr += 4;
		printk(KERN_INFO "SSID %d - %d\n", first, last);
		for (i = 0 ; i <= last - first ; i++)
			printk(KERN_INFO "MASK:%x\n", *((uint32_t *)ptr + i));
		ptr += ((last - first) + 1)*4;

	}
#endif
}

static void diag_update_msg_mask(int start, int end , uint8_t *buf)
{
	int found = 0;
	int first;
	int last;
	uint8_t *ptr = driver->msg_masks;
	mutex_lock(&driver->diagchar_mutex);
	/* First SSID can be zero : So check that last is non-zero */

	while (*(uint32_t *)(ptr + 4)) {
		first = *(uint32_t *)ptr;
		ptr += 4;
		last = *(uint32_t *)ptr;
		ptr += 4;
		if (start >= first && start <= last) {
			ptr += (start - first)*4;
			if (end <= last)
					memcpy(ptr, buf ,
						   (((end - start)+1)*4));
			else
				printk(KERN_INFO "Unable to copy mask "
						 "change \n");

			found = 1;
			break;
		} else {
			ptr += ((last - first) + 1)*4;
		}
	}
	/* Entry was not found - add new table */
	if (!found) {
		memcpy(ptr, &(start) , 4);
		ptr += 4;
		memcpy(ptr, &(end), 4);
		ptr += 4;
		memcpy(ptr, buf , ((end - start) + 1)*4);

	}
	mutex_unlock(&driver->diagchar_mutex);
	diag_print_mask_table();

}

static void diag_update_event_mask(uint8_t *buf, int toggle, int num_bits)
{
	uint8_t *ptr = driver->event_masks;
	uint8_t *temp = buf + 2;

	mutex_lock(&driver->diagchar_mutex);
	if (!toggle)
		memset(ptr, 0 , EVENT_MASK_SIZE);
	else
				memcpy(ptr, temp , num_bits/8 + 1);
	mutex_unlock(&driver->diagchar_mutex);
}

static void diag_update_log_mask(uint8_t *buf, int num_items)
{
		uint8_t *ptr = driver->log_masks;
		uint8_t *temp = buf;

		mutex_lock(&driver->diagchar_mutex);
		memcpy(ptr, temp , (num_items+7)/8);
		mutex_unlock(&driver->diagchar_mutex);
}

void diag_update_userspace_clients(unsigned int type)
{
	int i;

	mutex_lock(&driver->diagchar_mutex);
	for (i = 0; i < driver->num_clients; i++)
		if (driver->client_map[i] != 0)
			driver->data_ready[i] |= type;
	wake_up_interruptible(&driver->wait_q);
    mutex_unlock(&driver->diagchar_mutex);
}

static void diag_process_apps_pkt(unsigned char *buf, int len)
{
	uint16_t start;
	uint16_t end;

	/* event mask */
	if ((*buf == 0x60) && (*(++buf) == 0x0)) {
		diag_update_event_mask(buf, 0, 0);
		diag_update_userspace_clients(EVENT_MASKS_TYPE);
	}

	/* check for set event mask */
	if (*buf == 0x82) {
		buf += 4;
		diag_update_event_mask(buf, 1, *(uint16_t *)buf);
		diag_update_userspace_clients(
		EVENT_MASKS_TYPE);
	}

	/* log mask */
	if (*buf == 0x73) {
		buf += 4;
		if (*(int *)buf == 3) {
			buf += 4;
			/*printk(KERN_INFO "equip id %d \n", *(int *)buf); */
			buf += 4;
			/*printk(KERN_INFO "num items %d \n", *(int *)buf); */
			diag_update_log_mask(buf+4, *(int *)buf);
			diag_update_userspace_clients(LOG_MASKS_TYPE);
		}
	}
	/* Check for set message mask  */
	if ((*buf == 0x7d) && (*(++buf) == 0x4)) {
		buf++;
		start = *(uint16_t *)buf;
		buf += 2;
		end = *(uint16_t *)buf;
		buf += 4;
		diag_update_msg_mask((uint32_t)start, (uint32_t)end , buf);
		diag_update_userspace_clients(MSG_MASKS_TYPE);
	}
	/* Set all run-time masks
	if ((*buf == 0x7d) && (*(++buf) == 0x5)) {
		TO DO
	} */

}

static void diag_process_hdlc(void *data, unsigned len)
{
	struct diag_hdlc_decode_type hdlc;
	int ret;

	hdlc.dest_ptr = driver->hdlc_buf;
	hdlc.dest_size = USB_MAX_BUF;
	hdlc.src_ptr = data;
	hdlc.src_size = len;
	hdlc.src_idx = 0;
	hdlc.dest_idx = 0;
	hdlc.escaping = 0;

	ret = diag_hdlc_decode(&hdlc);

	if (ret)
		diag_process_apps_pkt(driver->hdlc_buf, hdlc.dest_idx - 3);

	/* ignore 2 bytes for CRC, one for 7E and send */
	if ((driver->ch) && (ret) && (hdlc.dest_idx > 3))
		smd_write(driver->ch, driver->hdlc_buf, hdlc.dest_idx - 3);
}

int diagfwd_connect(void)
{
	diag_open(driver->poolsize + 2); /* 2 requests for ARM9 traffic */

	driver->usb_connected = 1;
	driver->in_busy = 0;

	diag_read(driver->usb_buf_out, USB_MAX_BUF);
	return 0;
}

int diagfwd_disconnect(void)
{
	driver->usb_connected = 0;
	driver->in_busy = 1;

	diag_close();
	/* TBD - notify and flow control SMD */
	return 0;
}

int diagfwd_write_complete(unsigned char *buf, int len, int status)
{
	/*Determine if the write complete is for data from ARM9 or ARM11 */
	/* Need a context variable here instead */
	if (buf != (void *)driver->usb_buf_in)
		diagmem_free(driver, buf);
	else {
		driver->in_busy = 0;
		diag_smd_send_req();
	}
	return 0;
}

int diagfwd_read_complete(unsigned char *buf, int len, int status)
{
	driver->read_len = len;
	schedule_work(&(driver->diag_read_work));
	return 0;
}

static struct diag_operations diagfwdops = {
	.diag_connect = diagfwd_connect,
	.diag_disconnect = diagfwd_disconnect,
	.diag_char_write_complete = diagfwd_write_complete,
	.diag_char_read_complete = diagfwd_read_complete
};

static void diag_smd_notify(void *ctxt, unsigned event)
{
	diag_smd_send_req();
}

static int diag_smd_probe(struct platform_device *pdev)
{
	smd_open("DIAG", &driver->ch, driver, diag_smd_notify);
	return 0;
}

static struct platform_driver msm_smd_ch1_driver = {

	.probe = diag_smd_probe,
	.driver = {
		   .name = "DIAG",
		   .owner = THIS_MODULE,
		   },
};

void diag_read_work_fn(struct work_struct *work)
{
	diag_process_hdlc(driver->usb_buf_out, driver->read_len);
	diag_read(driver->usb_buf_out, USB_MAX_BUF);
}

void diagfwd_init(void)
{
	if (driver->usb_buf_in == NULL &&
	     (driver->usb_buf_in = kzalloc(USB_MAX_BUF, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->usb_buf_out  == NULL &&
	     (driver->usb_buf_out = kzalloc(USB_MAX_BUF, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->hdlc_buf == NULL
	    && (driver->hdlc_buf = kzalloc(HDLC_MAX, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->msg_masks == NULL
	    && (driver->msg_masks = kzalloc(MSG_MASK_SIZE, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->log_masks == NULL &&
	    (driver->log_masks = kzalloc(LOG_MASK_SIZE, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->event_masks == NULL &&
	    (driver->event_masks = kzalloc(EVENT_MASK_SIZE,
					    GFP_KERNEL)) == NULL)
		goto err;
	if (driver->client_map == NULL &&
	    (driver->client_map = kzalloc
	     (driver->num_clients, GFP_KERNEL)) == NULL)
		goto err;
	if (driver->data_ready == NULL &&
	     (driver->data_ready = kzalloc(driver->num_clients,
					    GFP_KERNEL)) == NULL)
		goto err;

	driver->diag_wq = create_singlethread_workqueue("diag_wq");
	INIT_WORK(&(driver->diag_read_work), diag_read_work_fn);

	diag_usb_register(&diagfwdops);

	platform_driver_register(&msm_smd_ch1_driver);

	return;
err:
		printk(KERN_INFO "\n Could not initialize diag buffer \n");
		kfree(driver->usb_buf_in);
		kfree(driver->usb_buf_out);
		kfree(driver->hdlc_buf);
		kfree(driver->msg_masks);
		kfree(driver->log_masks);
		kfree(driver->event_masks);
		kfree(driver->client_map);
		kfree(driver->data_ready);
}

void diagfwd_exit(void)
{
	smd_close(driver->ch);
	driver->ch = 0;		/*SMD can make this NULL */

	if (driver->usb_connected)
		diag_close();

	platform_driver_unregister(&msm_smd_ch1_driver);

	diag_usb_unregister();

	kfree(driver->usb_buf_in);
	kfree(driver->usb_buf_out);
	kfree(driver->hdlc_buf);
	kfree(driver->msg_masks);
	kfree(driver->log_masks);
	kfree(driver->event_masks);
	kfree(driver->client_map);
	kfree(driver->data_ready);
}
