#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/miscdevice.h>
#include <linux/taskstats_kern.h>
#include <linux/delay.h>

#include <asm/uaccess.h>
#include <asm/arch/mpp.h>
#include "proc_comm.h"

#define BUFF_SIZE  10

extern struct mpp *mpp_get(struct device *dev, const char *id);
extern int mpp_config_digital_out(struct mpp *mpp, unsigned config);

#define MPP5_UP "up"
#define MPP5_DOWN "down"
#define MPP5_RESET "reset"

static int wifi_reset_open(struct inode *inode, struct file *file)
{
    return 0;
}

static ssize_t
wifi_reset_write(struct file *file, const char __user *data, size_t len,
                     loff_t *ppos)
{
    char buff[BUFF_SIZE];
    int ret = -1;
    unsigned config = 0;
    struct mpp *p_mpp = NULL;
	int i = 0;

    memset(buff, 0, BUFF_SIZE);
    ret = copy_from_user(buff, data, len);
    if (ret)
    {
        printk(KERN_ALERT "wifi_reset_write copy_from_user failed.\n");
        return ret;
    }

    for ( ; ; ++i)
	{
	    if (buff[i] == '\0')
		{
			break;
		}
		if (buff[i] == '\n')
		{
			buff[i] = '\0';
			break;
		}
	}
    p_mpp = mpp_get(NULL, "mpp5");
    if (!p_mpp)
    {
        printk(KERN_ALERT "wifi_reset_write, mpp_get failed.\n");
        return -1;
    }

    // for config, the low 16 bits stands for the output logic, such as low, high
    // the corresponding datatype in ARM9 is pm_mpp_dlogic_out_ctrl_type
    // the higher 16 bits stands for pm_mpp_dlogic_lvl_type in ARM9
    config = 1 << 16; // 1 stands for PM_MPP_DLOGIC_LVL_MSMP

    if (!strcmp(buff, "up"))
    {
        config |= 1;  
        mpp_config_digital_out(p_mpp, config);
    } else if (!strcmp(buff, "down"))
    {
        mpp_config_digital_out(p_mpp, config);
    } else if (!strcmp(buff, "reset"))
    {
        mpp_config_digital_out(p_mpp, config);
		// need to adjust this value to make the reset
        mdelay(2);
        config |= 1;
        mpp_config_digital_out(p_mpp, config);
    } else
    {
       printk(KERN_ALERT "wifi_reset_write wrong command. the comm is %s.\n", buff);
       return -1;
    }
    
    return len;
}


// forced to read all in one time
static ssize_t
wifi_reset_read(struct file *file, char __user *buf, size_t count,
                         loff_t *ppos)
{
    return 0;
}

static const struct file_operations wifi_reset_fops =
{
    .owner = THIS_MODULE,
    .open = wifi_reset_open,
    .write = wifi_reset_write,
    .read = wifi_reset_read,
};

static struct miscdevice wifi_reset_miscdev =
{
    .minor = MISC_DYNAMIC_MINOR,
    .name = "wifi_reset",
    .fops = &wifi_reset_fops,
};

static int __init wifi_reset_init(void)
{
    int ret;
    ret = misc_register(&wifi_reset_miscdev);
    if (ret)
    {
        printk(KERN_ALERT "misc_register failed.\n");
        return -1;
    }
    return 0;
}

static void __exit wifi_reset_exit(void)
{
    misc_deregister(&wifi_reset_miscdev);
}

module_init(wifi_reset_init);
module_exit(wifi_reset_exit);
