/*
 * drivers/input/misc/msm_handset.c
 *
 * Driver for MSM handset events driver.
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org.
 */
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/input.h>

struct input_dev *ip_dev;
static int ip_dev_reg;

#define DRIVER_NAME	"msm-handset"

static int __init hs_probe(struct platform_device *pdev)
{
	int rc;

	ip_dev = input_allocate_device();
	if (!ip_dev) {
		rc = -ENOMEM;
		return rc;
	}

	ip_dev->name		= DRIVER_NAME;
	ip_dev->phys		= "msm-handset/input0";
	ip_dev->id.vendor	= 0x0001;
	ip_dev->id.product	= 1;
	ip_dev->id.version	= 1;

	ip_dev->evbit[0] = BIT(EV_SW);
	/* TODO: set this based on switchmap/keymap coming
	 * from board files.
	 */
	set_bit(SW_HEADPHONE_INSERT, ip_dev->swbit);

	rc = input_register_device(ip_dev);
	if (rc) {
		dev_err(&ip_dev->dev,
				"hs_probe: input_register_device rc=%d\n", rc);
		goto err_reg_input_dev;
	}

	ip_dev_reg = 1;

	return 0;

err_reg_input_dev:
	kfree(ip_dev);
	return rc;
}

static int hs_remove(struct platform_device *pdev)
{
	input_unregister_device(ip_dev);
	kfree(ip_dev);
	ip_dev_reg = 0;
	return 0;
}

struct input_dev *msm_get_handset_input_dev(void)
{
	return ip_dev_reg ? ip_dev : NULL;
}
EXPORT_SYMBOL(msm_get_handset_input_dev);

struct platform_driver hs_driver = {
	.probe		= hs_probe,
	.remove		= __devexit_p(hs_remove),
	.driver		= {
		.name	= DRIVER_NAME,
		.owner	= THIS_MODULE,
	},
};

static struct platform_device *hs_pdev;

static int __init hs_init(void)
{
	int rc;

	hs_pdev = platform_device_register_simple(DRIVER_NAME,
					-1, NULL, 0);

	if (IS_ERR(hs_pdev)) {
		rc = PTR_ERR(hs_pdev);
		goto err_reg_dev;
	}

	rc = platform_driver_register(&hs_driver);
	if (rc)
		goto err_reg_drv;

	return 0;

err_reg_drv:
	platform_device_unregister(hs_pdev);
err_reg_dev:
	return rc;
}
module_init(hs_init);

static void __exit hs_exit(void)
{
	platform_driver_unregister(&hs_driver);
	platform_device_unregister(hs_pdev);
}
module_exit(hs_exit);

MODULE_LICENSE("GPL v2");
MODULE_ALIAS("platform:msm-handset");
