/*
 * Copyright (c) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 */

#include <linux/delay.h>
#include <asm/arch/msm_iomap.h>
#include <linux/clk.h>
#include <asm/arch/gpio.h>
#include <asm/arch/board.h>
#include <asm/arch/io.h>
#include <asm/arch/msm_camio.h>

#define out_dword(addr, val) \
	(*((unsigned long  *)(addr)) = ((unsigned long)(val)))

#define out_dword_masked_ns(io, mask, val, current_reg_content)     \
	((void) out_dword(io, ((current_reg_content & (uint32_t)(~(mask))) | \
	((uint32_t)((val) & (mask))))))

#define __inpdw(port) (*((uint32_t *) (port)))
#define in_dword_masked(addr, mask) (__inpdw(addr) & (uint32_t)mask)

#define HWIO_MDDI_CAMIF_CFG_ADDR MSM_MDC_BASE
#define HWIO_MDDI_CAMIF_CFG_RMSK 0x1fffff
#define HWIO_MDDI_CAMIF_CFG_IN \
	in_dword_masked(HWIO_MDDI_CAMIF_CFG_ADDR, HWIO_MDDI_CAMIF_CFG_RMSK)

#define HWIO_MDDI_CAMIF_CFG_OUTM(m,v) \
	out_dword_masked_ns(HWIO_MDDI_CAMIF_CFG_ADDR,\
	m, v, HWIO_MDDI_CAMIF_CFG_IN);
#define __msmhwio_outm(hwiosym, mask, val) HWIO_##hwiosym##_OUTM(mask, val)
#define HWIO_OUTM(hwiosym, mask, val) __msmhwio_outm(hwiosym, mask, val)

#define HWIO_MDDI_CAMIF_CFG_CAM_SEL_BMSK 0x2
#define HWIO_MDDI_CAMIF_CFG_CAM_PCLK_SRC_SEL_BMSK 0x60000
#define HWIO_MDDI_CAMIF_CFG_CAM_PCLK_INVERT_BMSK 0x80000
#define HWIO_MDDI_CAMIF_CFG_CAM_PAD_REG_SW_RESET_BMSK 0x100000

#define HWIO_MDDI_CAMIF_CFG_CAM_SEL_SHFT 0x1
#define HWIO_MDDI_CAMIF_CFG_CAM_PCLK_SRC_SEL_SHFT 0x11
#define HWIO_MDDI_CAMIF_CFG_CAM_PCLK_INVERT_SHFT 0x13
#define HWIO_MDDI_CAMIF_CFG_CAM_PAD_REG_SW_RESET_SHFT 0x14

#define __msmhwio_shft(hwio_regsym, hwio_fldsym) \
	HWIO_##hwio_regsym##_##hwio_fldsym##_SHFT
#define HWIO_SHFT(hwio_regsym, hwio_fldsym) \
	__msmhwio_shft(hwio_regsym, hwio_fldsym)

#define __msmhwio_fmsk(hwio_regsym, hwio_fldsym) \
	HWIO_##hwio_regsym##_##hwio_fldsym##_BMSK
#define HWIO_FMSK(hwio_regsym, hwio_fldsym) \
	__msmhwio_fmsk(hwio_regsym, hwio_fldsym)

#define HWIO_APPS_RESET_ADDR (MSM_CLK_CTL_BASE + 0x00000210)
#define HWIO_APPS_RESET_RMSK 0x1fff
#define HWIO_APPS_RESET_VFE_BMSK 1
#define HWIO_APPS_RESET_VFE_SHFT 0
#define HWIO_APPS_RESET_IN \
	in_dword_masked(HWIO_APPS_RESET_ADDR, HWIO_APPS_RESET_RMSK)
#define HWIO_APPS_RESET_OUTM(m,v) \
	out_dword_masked_ns(HWIO_APPS_RESET_ADDR, m, v, HWIO_APPS_RESET_IN)

static struct clk *camio_vfe_mdc_clk;
static struct clk *camio_mdc_clk;
static struct clk *camio_vfe_clk;

void msm_camio_gpio_enable(void)
{
	config_camera_on_gpios();
}

void msm_camio_gpio_disable()
{
	config_camera_off_gpios();
}

int msm_camio_clk_enable(enum msm_camio_clk_type clktype)
{
	int rc = -1;
	struct clk *clk = NULL;

	switch (clktype) {
	case CAMIO_VFE_MDC_CLK:
		clk = camio_vfe_mdc_clk = clk_get(NULL, "vfe_mdc_clk");
		break;

	case CAMIO_MDC_CLK:
		clk = camio_mdc_clk = clk_get(NULL, "mdc_clk");
		break;

	case CAMIO_VFE_CLK:
		clk = camio_vfe_clk = clk_get(NULL, "vfe_clk");
		break;

	default:
		break;
	}

	if (clk != NULL && clk != ERR_PTR(-ENOENT)) {
		clk_enable(clk);
		rc = 0;
	}

	return rc;
}

int msm_camio_clk_disable(enum msm_camio_clk_type clktype)
{
	int rc = -1;
	struct clk *clk = NULL;

	switch (clktype) {
	case CAMIO_VFE_MDC_CLK:
		clk = camio_vfe_mdc_clk;
		break;

	case CAMIO_MDC_CLK:
		clk = camio_mdc_clk;
		break;

	case CAMIO_VFE_CLK:
		clk = camio_vfe_clk;
		break;

	default:
		break;
	}

	if (clk != NULL && clk != ERR_PTR(-ENOENT)) {
		clk_disable(clk);
		clk_put(clk);
		rc = 0;
	}

	return rc;
}

void msm_camio_clk_rate_set(int rate)
{
	struct clk *clk = camio_vfe_clk;
	if (clk != ERR_PTR(-ENOENT))
		clk_set_rate(clk, rate);
}

void msm_camio_enable(void)
{
	msm_camio_gpio_enable();
	msm_camio_clk_enable(CAMIO_VFE_MDC_CLK);
	msm_camio_clk_enable(CAMIO_MDC_CLK);
	msm_camio_clk_enable(CAMIO_VFE_CLK);
}

void msm_camio_disable(void)
{
	msm_camio_gpio_disable();
	msm_camio_clk_disable(CAMIO_VFE_MDC_CLK);
	msm_camio_clk_disable(CAMIO_MDC_CLK);
	msm_camio_clk_disable(CAMIO_VFE_CLK);
}

void msm_camio_camif_pad_reg_reset(void)
{
	/* select CLKRGM_VFE_SRC_CAM_VFE_SRC:  internal source */
	msm_camio_clk_sel(MSM_CAMIO_CLK_SRC_INTERNAL);

	HWIO_OUTM(MDDI_CAMIF_CFG,
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_SEL) |
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PCLK_SRC_SEL) |
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PCLK_INVERT),
		1 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_SEL) |
		3 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PCLK_SRC_SEL) |
		0 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PCLK_INVERT));

	mdelay(10);

	HWIO_OUTM(MDDI_CAMIF_CFG,
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET),
		1 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET));

	mdelay(10);
	HWIO_OUTM(MDDI_CAMIF_CFG,
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET),
		0 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET));

	mdelay(10);

	/* select CLKRGM_VFE_SRC_TLMM_CAM_PCLK: external source */
	msm_camio_clk_sel(MSM_CAMIO_CLK_SRC_EXTERNAL);
}

void msm_camio_vfe_blk_reset(void)
{
	HWIO_OUTM(APPS_RESET, HWIO_FMSK(APPS_RESET, VFE),
		1 << HWIO_SHFT(APPS_RESET, VFE));

	mdelay(10);

	HWIO_OUTM(APPS_RESET, HWIO_FMSK(APPS_RESET, VFE),
		0 << HWIO_SHFT(APPS_RESET, VFE));

	mdelay(10);
}

void msm_camio_camif_pad_reg_reset_2(void)
{
	HWIO_OUTM(MDDI_CAMIF_CFG,
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET),
		1 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET));

	mdelay(10);

	HWIO_OUTM(MDDI_CAMIF_CFG,
		HWIO_FMSK(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET),
		0 << HWIO_SHFT(MDDI_CAMIF_CFG, CAM_PAD_REG_SW_RESET));

	mdelay(10);
}

void msm_camio_clk_sel(enum msm_camio_clk_src_type srctype)
{
	struct clk *clk = NULL;

	clk = camio_vfe_clk;

	if (clk != NULL && clk != ERR_PTR(-ENOENT)) {
		switch (srctype) {
		case MSM_CAMIO_CLK_SRC_INTERNAL:
			clk_set_flags(clk, 0x00000100 << 1);
			break;

		case MSM_CAMIO_CLK_SRC_EXTERNAL:
			clk_set_flags(clk, 0x00000100);
			break;

		default:
			break;
		}
	}
}
