/*
 * Copyright (C) 2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 */

#include <linux/msm_adsp.h>
#include <asm/arch/msm_adsp.h>
#include <linux/uaccess.h>
#include <linux/android_pmem.h>
#include <asm/arch/msm_camio.h>
#include "msm_vfe7x.h"

#define QDSP_CMDQUEUE 25

#define VFE_RESET_CMD 0
#define VFE_START_CMD 1
#define VFE_STOP_CMD  2
#define VFE_FRAME_ACK 20
#define STATS_WE_ACK  22

#define MSG_SNAPSHOT  2
#define MSG_OUTPUT1   6
#define MSG_OUTPUT2   7
#define MSG_STATS_AF  8
#define MSG_STATS_WE  9

#define VFE_ADSP_EVENT 0xFFFFFFFF

static struct msm_adsp_module *qcam_mod;
static struct msm_adsp_module *vfe_mod;
static struct msm_vfe_resp *resp;
static void *extdata;
static uint32_t extlen;

static void vfe_7x_convert(struct msm_vfe_phy_info *pinfo,
	enum vfe_resp_msg_t	type, void *data, void **ext, int32_t *elen)
{
	switch (type) {
	case VFE_MSG_OUTPUT1:
	case VFE_MSG_OUTPUT2: {
		pinfo->y_phy = ((struct vfe_endframe *)data)->y_address;
		pinfo->cbcr_phy =
			((struct vfe_endframe *)data)->cbcr_address;

		CDBG("vfe_7x_convert, y_phy = 0x%x, cbcr_phy = 0x%x\n",
				 pinfo->y_phy, pinfo->cbcr_phy);

		((struct vfe_frame_extra *)extdata)->bl_evencol =
		((struct vfe_endframe *)data)->blacklevelevencolumn;

		((struct vfe_frame_extra *)extdata)->bl_oddcol =
		((struct vfe_endframe *)data)->blackleveloddcolumn;

		((struct vfe_frame_extra *)extdata)->g_def_p_cnt =
		((struct vfe_endframe *)data)->greendefectpixelcount;

		((struct vfe_frame_extra *)extdata)->r_b_def_p_cnt =
		((struct vfe_endframe *)data)->redbluedefectpixelcount;

		*ext  = extdata;
		*elen = extlen;
	}
		break;

	case VFE_MSG_STATS_AF:
	case VFE_MSG_STATS_WE:
		pinfo->sbuf_phy = *(uint32_t *)data;
		break;

	default:
		break;
	} /* switch */
}

static void vfe_7x_ops(void *driver_data, unsigned id, size_t len,
		void (*getevent)(void *ptr, size_t len))
{
	uint32_t evt_buf[3];
	struct msm_vfe_resp_t *rp;

	rp = kmalloc(sizeof(struct msm_vfe_resp_t), GFP_ATOMIC);
	if (!rp) {
		CDBG("rp: cannot allocate buffer\n");
		return;
	}

	if (id == VFE_ADSP_EVENT) {
		/* event */
		getevent(evt_buf, sizeof(evt_buf));

		rp->type           = VFE_EVENT;
		rp->evt_msg.type   = MSM_CAMERA_EVT;
		rp->evt_msg.len    = 0;
		rp->evt_msg.msg_id = evt_buf[0];

		resp->vfe_resp(rp, MSM_CAM_Q_VFE_EVT);
	} else {
		/* messages */

		rp->evt_msg.type   = MSM_CAMERA_MSG;
		rp->evt_msg.msg_id = id;
		rp->evt_msg.len    = len;

		rp->evt_msg.data = kmalloc(rp->evt_msg.len, GFP_ATOMIC);
		if (!(rp->evt_msg.data))
			return;

		getevent(rp->evt_msg.data, len);

		switch (rp->evt_msg.msg_id) {
		case MSG_SNAPSHOT:
			rp->type = VFE_MSG_SNAPSHOT;
			break;

		case MSG_OUTPUT1:
			rp->type = VFE_MSG_OUTPUT1;
			vfe_7x_convert(&(rp->phy), VFE_MSG_OUTPUT1,
				rp->evt_msg.data, &(rp->extdata),
				&(rp->extlen));
			break;

		case MSG_OUTPUT2:
			rp->type = VFE_MSG_OUTPUT2;
			vfe_7x_convert(&(rp->phy), VFE_MSG_OUTPUT2,
					rp->evt_msg.data, &(rp->extdata),
					&(rp->extlen));
			break;

		case MSG_STATS_AF:
			rp->type = VFE_MSG_STATS_AF;
			vfe_7x_convert(&(rp->phy), VFE_MSG_STATS_AF,
					rp->evt_msg.data, NULL, NULL);
			break;

		case MSG_STATS_WE:
			rp->type = VFE_MSG_STATS_WE;
			vfe_7x_convert(&(rp->phy), VFE_MSG_STATS_WE,
					rp->evt_msg.data, NULL, NULL);

			CDBG("MSG_STATS_WE: phy = 0x%x\n", rp->phy.sbuf_phy);
			break;

		default:
			rp->type = VFE_MSG_GENERAL;
			break;
		}

		resp->vfe_resp(rp, MSM_CAM_Q_VFE_MSG);
	}
}

static struct msm_adsp_ops vfe_7x_sync = {
	.event = vfe_7x_ops,
};

static int vfe_7x_enable(struct camera_enable_cmd_t *enable)
{
	int rc = -EFAULT;

	if (!strcmp(enable->name, "QCAMTASK"))
		rc = msm_adsp_enable(qcam_mod);
	else if (!strcmp(enable->name, "VFETASK"))
		rc = msm_adsp_enable(vfe_mod);

	return rc;
}

static int vfe_7x_disable(struct camera_enable_cmd_t *enable)
{
	int rc = -EFAULT;

	if (!strcmp(enable->name, "QCAMTASK"))
		rc = msm_adsp_disable(qcam_mod);
	else if (!strcmp(enable->name, "VFETASK"))
		rc = msm_adsp_disable(vfe_mod);

	return rc;
}

static void vfe_7x_release(void)
{
	msm_adsp_disable(qcam_mod);
	msm_adsp_disable(vfe_mod);

	msm_adsp_put(qcam_mod);
	msm_adsp_put(vfe_mod);

	msm_camio_disable();

	kfree(extdata);
	extlen = 0;

}

static int vfe_7x_init(struct msm_vfe_resp *presp)
{
	int rc = 0;

	/* 1. Bring up all the required GPIOs and Clocks */
	msm_camio_enable();
	msm_camio_camif_pad_reg_reset();

	extdata =
		kmalloc(sizeof(struct vfe_frame_extra), GFP_ATOMIC);
	if (!extdata)
		return -ENOMEM;

	extlen = sizeof(struct vfe_frame_extra);

	if (presp && presp->vfe_resp)
		resp = presp;
	else
		return -EFAULT;

	rc = msm_adsp_get("QCAMTASK", &qcam_mod, &vfe_7x_sync, NULL);
	if (rc)
		return rc;

	rc = msm_adsp_get("VFETASK", &vfe_mod, &vfe_7x_sync, NULL);
	return rc;
}

static int vfe_7x_config_axi(enum vfeoutput_mode_t mode,
	struct axidata_t *ad, struct axiout_t *ao)
{
	struct msm_pmem_region *regptr;
	int rc = 0;

	if (mode == OUTPUT_1 || mode == OUTPUT_1_AND_2_SNAP) {
		regptr = ad->region;

		CDBG("bufnum1 = %d\n", ad->bufnum1);
		CDBG("config_axi: O1, phy = 0x%lx, y_off = %d, cbcr_off = %d\n",
			regptr->paddr, regptr->y_off, regptr->cbcr_off);

		switch (ad->bufnum1) {
		case 1: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output1buffer8_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer8_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
		} /* 1 */
			break;

		case 2: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer3_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer3_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer4_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer4_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
			ao->output1buffer5_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer5_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer6_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer6_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
			ao->output1buffer7_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer7_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer8_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
		} /* 2 */
			break;

		case 3: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer4_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer4_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer5_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer5_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
			ao->output1buffer6_y_phy =
				ao->output1buffer3_y_phy;
			ao->output1buffer6_cbcr_phy =
				ao->output1buffer3_cbcr_phy;
			ao->output1buffer7_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer7_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer8_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
		} /* 3 */
			break;

		case 4: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer5_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer5_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer6_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer6_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
			ao->output1buffer7_y_phy =
				ao->output1buffer3_y_phy;
			ao->output1buffer7_cbcr_phy =
				ao->output1buffer3_cbcr_phy;
			ao->output1buffer8_y_phy =
				ao->output1buffer4_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer4_cbcr_phy;
		} /* 4 */
			break;

		case 5: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer6_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer6_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer7_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer7_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
			ao->output1buffer8_y_phy =
				ao->output1buffer3_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer3_cbcr_phy;
		} /* 5 */
			break;

		case 6: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer7_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer7_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
			ao->output1buffer8_y_phy =
				ao->output1buffer2_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer2_cbcr_phy;
		} /* 6 */
			break;

		case 7: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output1buffer8_y_phy =
				ao->output1buffer1_y_phy;
			ao->output1buffer8_cbcr_phy =
				ao->output1buffer1_cbcr_phy;
		} /* 7 */
			break;

		case 8: {
			ao->output1buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output1buffer8_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output1buffer8_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
		} /* 8 */
			break;

		default:
			break;
		} /* switch (bufnum) */
	} /* if OUTPUT1 or Both */

	if (mode == OUTPUT_2 || mode == OUTPUT_1_AND_2_SNAP) {

		regptr = &(ad->region[ad->bufnum1]);

		CDBG("bufnum2 = %d\n", ad->bufnum2);
		CDBG("config_axi: O2, phy = 0x%lx, y_off = %d, cbcr_off = %d\n",
			regptr->paddr, regptr->y_off, regptr->cbcr_off);

		switch (ad->bufnum2) {
		case 1: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			ao->output2buffer8_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer8_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
		} /* 1 */
			break;

		case 2: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer3_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer3_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer4_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer4_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
			ao->output2buffer5_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer5_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer6_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer6_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
			ao->output2buffer7_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer7_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer8_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
		} /* 2 */
			break;

		case 3: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer4_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer4_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer5_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer5_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
			ao->output2buffer6_y_phy =
				ao->output2buffer3_y_phy;
			ao->output2buffer6_cbcr_phy =
				ao->output2buffer3_cbcr_phy;
			ao->output2buffer7_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer7_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer8_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
		} /* 3 */
			break;

		case 4: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;

			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;

			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;

			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer5_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer5_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer6_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer6_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
			ao->output2buffer7_y_phy =
				ao->output2buffer3_y_phy;
			ao->output2buffer7_cbcr_phy =
				ao->output2buffer3_cbcr_phy;
			ao->output2buffer8_y_phy =
				ao->output2buffer4_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer4_cbcr_phy;
		} /* 4 */
			break;

		case 5: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer6_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer6_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer7_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer7_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
			ao->output2buffer8_y_phy =
				ao->output2buffer3_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer3_cbcr_phy;
		} /* 5 */
			break;

		case 6: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer7_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer7_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
			ao->output2buffer8_y_phy =
				ao->output2buffer2_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer2_cbcr_phy;
		} /* 6 */
			break;

		case 7: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;

			ao->output2buffer8_y_phy =
				ao->output2buffer1_y_phy;
			ao->output2buffer8_cbcr_phy =
				ao->output2buffer1_cbcr_phy;
		} /* 7 */
			break;

		case 8: {
			ao->output2buffer1_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer1_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer2_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer2_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer3_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer3_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer4_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer4_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer5_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer5_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer6_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer6_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer7_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer7_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
			regptr++;
			ao->output2buffer8_y_phy =
				regptr->paddr + regptr->y_off;
			ao->output2buffer8_cbcr_phy =
				regptr->paddr + regptr->cbcr_off;
		} /* 8 */
			break;
		} /* switch (bufnum) */
	}

	return rc;
}

static int vfe_7x_config(struct msm_vfe_cfg_cmd_t *cmd, void *data)
{
	struct msm_pmem_region *regptr;
	unsigned char buf[256];

	struct vfe_stats_we_ack_t sack;
	struct axidata_t *axid;
	uint32_t i;

	struct vfe_stats_we_cfg_t *scfg_t = NULL;
	struct axiout_t *axio = NULL;
	void   *cmd_data = NULL;
	void   *cmd_data_alloc = NULL;
	long rc = 0;
	struct msm_vfe_command_7k *vfecmd;

	static uint8_t vfestopped;

	vfecmd =
			kmalloc(sizeof(struct msm_vfe_command_7k),
				GFP_ATOMIC);
	if (!vfecmd) {
		CDBG("vfecmd alloc failed!\n");
		return -ENOMEM;
	}

	if (cmd->cmd_type != CMD_FRAME_BUF_RELEASE &&
	    cmd->cmd_type != CMD_STATS_BUF_RELEASE) {
		if (copy_from_user(vfecmd,
				(void *)(cmd->value),
				sizeof(struct msm_vfe_command_7k))) {
			rc = -EFAULT;
			goto config_failure;
		}
	}

	switch (cmd->cmd_type) {
	case CMD_STATS_ENABLE:
	case CMD_STATS_AXI_CFG: {
		axid = (struct axidata_t *)data;
		if (!axid) {
			rc = -EFAULT;
			goto config_failure;
		}

		scfg_t =
			kmalloc(sizeof(struct vfe_stats_we_cfg_t),
				GFP_ATOMIC);
		if (!scfg_t) {
			rc = -EFAULT;
			goto config_failure;
		}

		if (copy_from_user(scfg_t,
					(void *)(vfecmd->value),
					vfecmd->length)) {

			rc = -EFAULT;
			goto config_done;
		}

		CDBG("STATS_ENABLE: bufnum = %d, enabling = %d\n",
			axid->bufnum1, scfg_t->wb_expstatsenable);

		if (axid->bufnum1 > 0) {
			regptr = axid->region;

			for (i = 0; i < axid->bufnum1; i++) {

		CDBG("STATS_ENABLE, phy = 0x%lx\n",
			regptr->paddr);

				scfg_t->wb_expstatoutputbuffer[i] =
					(void *)regptr->paddr;
				regptr++;
			}

			cmd_data = scfg_t;

		} else {
			rc = -EINVAL;
			goto config_done;
		}
	}
		break;

	case CMD_STATS_AF_AXI_CFG:
		break;

	case CMD_FRAME_BUF_RELEASE: {
		struct msm_frame_t *b;
		unsigned long p;
		struct vfe_outputack_t fack;
		if (!data)  {
			rc = -EFAULT;
			goto config_failure;
		}

		b = (struct msm_frame_t *)(cmd->value);
		p = *(unsigned long *)data;

		fack.header = VFE_FRAME_ACK;

		fack.output2newybufferaddress =
			(void *)(p + b->y_off);

		fack.output2newcbcrbufferaddress =
			(void *)(p + b->cbcr_off);

		vfecmd->queue = QDSP_CMDQUEUE;
		vfecmd->length = sizeof(struct vfe_outputack_t);
		cmd_data = &fack;
	}
		break;

	case CMD_SNAP_BUF_RELEASE:
		break;

	case CMD_STATS_BUF_RELEASE: {
		if (!data) {
			rc = -EFAULT;
			goto config_failure;
		}

		sack.header = STATS_WE_ACK;
		sack.wb_expnewbufferaddress = (void *)*(uint32_t *)data;

		vfecmd->queue  = QDSP_CMDQUEUE;
		vfecmd->length = sizeof(struct vfe_stats_we_ack_t);
		cmd_data = &sack;
	}
		break;

	case CMD_GENERAL:
	case CMD_STATS_DISABLE: {
		if (vfecmd->length > 256) {
			cmd_data_alloc =
			cmd_data = kmalloc(vfecmd->length, GFP_ATOMIC);
			if (!cmd_data) {
				rc = -EFAULT;
				goto config_failure;
			}
		} else
			cmd_data = buf;

		if (copy_from_user(cmd_data,
					(void *)(vfecmd->value),
					vfecmd->length)) {

			rc = -EFAULT;
			goto config_done;
		}

		if (vfecmd->queue == QDSP_CMDQUEUE) {
			switch (*(uint32_t *)cmd_data) {
			case VFE_RESET_CMD:
			msm_camio_vfe_blk_reset();
			msm_camio_camif_pad_reg_reset_2();
			vfestopped = 0;
			break;

			case VFE_START_CMD:
			msm_camio_camif_pad_reg_reset_2();
			vfestopped = 0;
			break;

			case VFE_STOP_CMD:
			vfestopped = 1;
			goto config_send;

			default:
			break;
			}
		} /* QDSP_CMDQUEUE */
	}
		break;

	case CMD_AXI_CFG_OUT1: {
		axid = (struct axidata_t *)data;
		if (!axid) {
			rc = -EFAULT;
			goto config_failure;
		}

		axio = kmalloc(sizeof(struct axiout_t), GFP_ATOMIC);
		if (!axio) {
			rc = -ENOMEM;
			goto config_failure;
		}

		if (copy_from_user(axio, (void *)(vfecmd->value),
					sizeof(struct axiout_t))) {
			rc = -EFAULT;
			goto config_done;
		}

		vfe_7x_config_axi(OUTPUT_1, axid, axio);

		cmd_data = axio;
	}
		break;

	case CMD_AXI_CFG_OUT2:
	case CMD_RAW_PICT_AXI_CFG: {
		axid = (struct axidata_t *)data;
		if (!axid) {
			rc = -EFAULT;
			goto config_failure;
		}

		axio = kmalloc(sizeof(struct axiout_t), GFP_ATOMIC);
		if (!axio) {
			rc = -ENOMEM;
			goto config_failure;
		}

		if (copy_from_user(axio, (void *)(vfecmd->value),
					sizeof(struct axiout_t))) {
			rc = -EFAULT;
			goto config_done;
		}

		vfe_7x_config_axi(OUTPUT_2, axid, axio);
		cmd_data = axio;
	}
		break;

	case CMD_AXI_CFG_SNAP_O1_AND_O2: {
		axid = (struct axidata_t *)data;
		if (!axid) {
			rc = -EFAULT;
			goto config_failure;
		}

		axio = kmalloc(sizeof(struct axiout_t), GFP_ATOMIC);
		if (!axio) {
			rc = -EFAULT;
			goto config_failure;
		}

		if (copy_from_user(axio, (void *)(vfecmd->value),
					sizeof(struct axiout_t))) {
			rc = -EFAULT;
			goto config_done;
		}

		vfe_7x_config_axi(OUTPUT_1_AND_2_SNAP, axid, axio);

		cmd_data = axio;
	}
		break;

	default:
		break;
	} /* switch */

	if (vfestopped)
		goto config_done;

config_send:
	CDBG("send adsp command = %d\n", *(uint32_t *)cmd_data);
	rc = msm_adsp_write(vfe_mod, vfecmd->queue,
				cmd_data, vfecmd->length);

config_done:
	if (cmd_data_alloc != NULL)
		kfree(cmd_data_alloc);

config_failure:
	if (scfg_t != NULL)
		kfree(scfg_t);
	if (axio != NULL)
		kfree(axio);
	if (vfecmd != NULL)
		kfree(vfecmd);
	return rc;
}

void msm_camvfe_init(struct msm_camvfe_fn_t *fptr)
{
	fptr->vfe_init    = vfe_7x_init;
	fptr->vfe_enable  = vfe_7x_enable;
	fptr->vfe_config  = vfe_7x_config;
	fptr->vfe_disable = vfe_7x_disable;
	fptr->vfe_release = vfe_7x_release;
}
