/* linux/drivers/net/msm_rmnet.c
 *
 * Virtual Ethernet Interface for MSM7K Networking
 *
 * Copyright (C) 2007 Google, Inc.
 * Author: Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>

#include <asm/arch/msm_smd.h>

#include <linux/netlink.h>
#include <linux/icmp.h>
#include <linux/udp.h>
#include <linux/ip.h>
#include <linux/in.h>
extern int vote_for_sleep(int, int);
#define SMD_PORT_ETHER0 11

static int diagstate = 0;
char DIAL_str[] = "DEMAND_GPIO";
#define MAX_TRIES   20000
#define TRY_CYCLE   5000
#define	DHCP_SERVER_PORT  67
#define	DHCP_CLIENT_PORT  68

struct rmnet_private
{
	smd_channel_t *ch;
	struct net_device_stats stats;
	const char *chname;
    struct sk_buff *skb;
    spinlock_t lock;
    struct tasklet_struct tsklt;    
};

static int is_broadcast(unsigned  char *addr)
{
	return(*(unsigned  short*)(&addr[0]) == 0xffff &&
	       *(unsigned  short *)(&addr[2]) == 0xffff &&
	       *(unsigned  short *)(&addr[4]) == 0xffff ) ;
}

static int count_this_packet(void *_hdr, int len)
{
	struct ethhdr *hdr = _hdr;

	if (len < ETH_HLEN)
		return 1;

       if(is_broadcast(hdr->h_dest) || is_broadcast(hdr->h_source))
       {
            printk("Ethernet Broadcast Packet: don't add to flow Stat.!\n");
            return 0;
       }
	if (hdr->h_proto == htons(ETH_P_ARP))
		return 0;

	return 1;
}
#ifdef FEATURE_WIFI_INTERRUPT
typedef enum
{
    WLAN_AWAKE,
    WLAN_AWAKING,
    WLAN_SLEEPING,
    WLAN_ASLEEP
}wlan_wow_state_t;
wlan_wow_state_t g_wlan_wow_state = WLAN_AWAKE;
void set_wlan_wow_state(wlan_wow_state_t stat)
{
   g_wlan_wow_state = stat;
}
typedef int (*wlan_callback)(void);
extern wlan_callback wlan_resume_func;
EXPORT_SYMBOL(set_wlan_wow_state);
#endif
/* Called in soft-irq context */
static void smd_net_data_handler(unsigned long arg)
{
	struct net_device *dev = (struct net_device *) arg;
	struct rmnet_private *p = netdev_priv(dev);
	struct sk_buff *skb;
	void *ptr = 0;
	int sz;

	for (;;) {
		sz = smd_cur_packet_size(p->ch);
		if (sz == 0) break;
		if (smd_read_avail(p->ch) < sz) break;

		if (sz > 1514) {
			pr_err("rmnet_recv() discarding %d len\n", sz);
			ptr = 0;
		} else {
			skb = dev_alloc_skb(sz + NET_IP_ALIGN);
			if (skb == NULL) {
				pr_err("rmnet_recv() cannot allocate skb\n");
			} else {
				skb->dev = dev;
				skb_reserve(skb, NET_IP_ALIGN);
				ptr = skb_put(skb, sz);
				if (smd_read(p->ch, ptr, sz) != sz) {
					pr_err("rmnet_recv() smd lied "
					       "about avail?!");
					ptr = 0;
					dev_kfree_skb_irq(skb);
				} else {
					skb->protocol = eth_type_trans(skb, dev);
					if (count_this_packet(ptr, skb->len)) {
						p->stats.rx_packets++;
						p->stats.rx_bytes += skb->len;
					}
					netif_rx(skb);
#ifdef FEATURE_WIFI_INTERRUPT
          if(WLAN_ASLEEP == g_wlan_wow_state)
          {
            if(wlan_resume_func);
            {
              wlan_resume_func();
            }
          }
#endif
				}
				continue;
			}
		}
		if (smd_read(p->ch, ptr, sz) != sz)
			pr_err("rmnet_recv() smd lied about avail?!");
	}
}

static DECLARE_TASKLET(smd_net_data_tasklet, smd_net_data_handler, 0);

static int _rmnet_xmit(struct sk_buff *skb, struct net_device *dev)
{
    struct rmnet_private *p = netdev_priv(dev);
    smd_channel_t *ch = p->ch;
    int smd_ret;
    struct icmphdr *icmph;
    struct udphdr *udph;
    struct iphdr *iph;    

    smd_ret = smd_write(ch, skb->data, skb->len);
    if (smd_ret != skb->len) {
           pr_err("%s: smd_write returned error %d", __func__, smd_ret);
           goto xmit_out;
    }

    {
        iph = ip_hdr(skb);
        if(iph->protocol == IPPROTO_ICMP) 
        {
            icmph = icmp_hdr(skb);
            if((NULL != icmph)&&(0 == icmph->type)&&(0 == icmph->code))
            {
                printk("ping-ack packet don't count\n");
                dev_kfree_skb_irq(skb);
                return 0;
            }
        }
        else if(iph->protocol == IPPROTO_UDP)
        {
            udph = udp_hdr(skb);
            if((NULL != udph) && ((DHCP_CLIENT_PORT == ntohs(udph->source)) && (DHCP_SERVER_PORT == ntohs(udph->dest))))
            {
                printk("DHCP Packet: don't add to flow Stat.!\n");
                dev_kfree_skb_irq(skb);
                return 0;
            }
        }

        if (count_this_packet(skb->data, skb->len)) {
            if(1==diagstate)
            {
                syswatch_nl_send(DIAL_str, GPIO_TRIGGER);
                diagstate = 0;
            }            
            p->stats.tx_packets++;
            p->stats.tx_bytes += skb->len;
        }
    }
xmit_out:
       /* data xmited, safe to release skb */
       dev_kfree_skb_irq(skb);
       return 0;
}


static void _rmnet_resume_flow(unsigned long param)
{
       struct net_device *dev = (struct net_device *)param;
       struct rmnet_private *p = netdev_priv(dev);
       struct sk_buff *skb = NULL;
       unsigned long flags;

       /* xmit and enable the flow only once even if
          multiple tasklets were scheduled by smd_net_notify */
       spin_lock_irqsave(&p->lock, flags);
       if (p->skb && (smd_write_avail(p->ch) >= p->skb->len)) {
               skb = p->skb;
               p->skb = NULL;
               spin_unlock_irqrestore(&p->lock, flags);
               _rmnet_xmit(skb, dev);
               netif_wake_queue(dev);
               
       } else
               spin_unlock_irqrestore(&p->lock, flags);
}
static DECLARE_TASKLET(smd_resume_data_tasklet, _rmnet_resume_flow, 0);

static void smd_net_notify(void *_dev, unsigned event)
{
       struct rmnet_private *p = netdev_priv((struct net_device *)_dev);

	if (event != SMD_EVENT_DATA)
		return;
            
       spin_lock(&p->lock);
       if (p->skb && (smd_write_avail(p->ch) >= p->skb->len))
        {           
               smd_resume_data_tasklet.data =(unsigned long) _dev;
               tasklet_schedule(&smd_resume_data_tasklet);
        }
       spin_unlock(&p->lock);

       if (smd_read_avail(p->ch) &&
           (smd_read_avail(p->ch) >= smd_cur_packet_size(p->ch))) {
               smd_net_data_tasklet.data = (unsigned long) _dev;
               tasklet_schedule(&smd_net_data_tasklet);
       }

}

static int rmnet_open(struct net_device *dev)
{
	int r;
	struct rmnet_private *p = netdev_priv(dev);

	printk(KERN_INFO "rmnet_open()\n");
	pr_info("rmnet_open()\n");
	if (!p->ch) {
		r = smd_open(p->chname, &p->ch, dev, smd_net_notify);

		if (r < 0)
			return -ENODEV;
	}

	netif_start_queue(dev);
	return 0;
}

/*lint -e529*/
static int rmnet_stop(struct net_device *dev)
{
        struct rmnet_private *p = netdev_priv(dev); 

        pr_info("rmnet_stop()\n");
        netif_stop_queue(dev);
        tasklet_disable(&p->tsklt);

        return 0;
}
/*lint -e529*/

static void sleep_cycle(int count)
{
    while(count-- > 0)
    {
        count--;
    }
}
static int try_to_xmit(struct sk_buff *skb, struct net_device *dev)
{
    struct rmnet_private *p = netdev_priv(dev);
    smd_channel_t *ch = p->ch;

    int tries = 0;
    while( tries++ < MAX_TRIES)
    {
        if (smd_write(ch, skb->data, skb->len) == skb->len)
        {
            return 0;
        }
        else
        {
            //pr_err("rmnet fifo full, try again!\n");
            sleep_cycle(TRY_CYCLE);
        }
    }
    return -1;
}

static int rmnet_xmit(struct sk_buff *skb, struct net_device *dev)
{
    struct rmnet_private *p = netdev_priv(dev);
    smd_channel_t *ch = p->ch;
    unsigned long flags;

    if (netif_queue_stopped(dev)) {
            pr_err("fatal: rmnet_xmit called when netif_queue is stopped");
            return 0;
    }

    spin_lock_irqsave(&p->lock, flags);
    if (smd_write_avail(ch) < skb->len) {
           netif_stop_queue(dev);
           p->skb = skb;
           spin_unlock_irqrestore(&p->lock, flags);
           return 0;
    }
    spin_unlock_irqrestore(&p->lock, flags);

    _rmnet_xmit(skb, dev);

    return 0;
}

static struct net_device_stats *rmnet_get_stats(struct net_device *dev)
{
	struct rmnet_private *p = netdev_priv(dev);
	return &p->stats;
}

static void rmnet_set_multicast_list(struct net_device *dev)
{
}

static void rmnet_tx_timeout(struct net_device *dev)
{
	pr_info("rmnet_tx_timeout()\n");
}

/* ûRMNET·ȡͳϢ */
#define SIOCGRMNETSTATS (SIOCDEVPRIVATE + 0)
#define SIOCDEMANDON  (SIOCDEVPRIVATE + 1)
#define SIOCDEMANDOFF  (SIOCDEVPRIVATE + 2)
#define SIOCDEMANDFLUSH (SIOCDEVPRIVATE + 3)

/* RMNETӿڵͳƽṹݸû */
struct rmnet_user_stats
{
    unsigned long rx_packets;         /* հĸ */
    unsigned long tx_packets;         /* Ͱĸ */
    unsigned long rx_bytes;           /* հֽ */
    unsigned long tx_bytes;           /* Ͱֽ */
    unsigned long rx_errors;          /* մĸ */
    unsigned long tx_errors;          /* ʹĸ */
    unsigned long rx_dropped;         /* ն */
    unsigned long tx_dropped;         /* Ͷ */
    unsigned long sip_packets;        /* sip */
    unsigned long sip_bytes;          /* sipֽ */
    unsigned long voice_packets;      /* voice */
    unsigned long voice_bytes;        /* voiceֽ */
};

static int msm_rmnet_ioctl (struct net_device *net, struct ifreq *rq, int cmd)
{
    void *addr = (void *) rq->ifr_data;
    struct rmnet_user_stats stats;
    struct net_device_stats *pstats;
    
    switch (cmd)
    {
    case SIOCGRMNETSTATS:
        pstats = rmnet_get_stats(net);
        memset(&stats, 0, sizeof(struct rmnet_user_stats)); //lint !e522
        stats.rx_packets = pstats->rx_packets;
        stats.tx_packets = pstats->tx_packets;
        stats.rx_bytes      = pstats->rx_bytes;
        stats.tx_bytes      = pstats->tx_bytes;
        stats.rx_errors     = pstats->rx_errors;
        stats.tx_errors     = pstats->tx_errors;
        stats.rx_dropped    = pstats->rx_dropped;
        stats.tx_dropped    = pstats->tx_dropped;
        stats.sip_packets   = pstats->multicast;
        stats.sip_bytes     = pstats->collisions;
        stats.voice_packets = pstats->rx_compressed;
        stats.voice_bytes   = pstats->tx_compressed;
        if (copy_to_user(addr, &stats, sizeof(stats)))
        {
            return -EFAULT;
        }
        break;
    case  SIOCDEMANDON:
        diagstate = 1;
        break;
        
    case  SIOCDEMANDOFF:
        diagstate = 0;
        break;
    case  SIOCDEMANDFLUSH:
        pstats = rmnet_get_stats(net);
        memset(pstats, 0, sizeof(struct rmnet_user_stats)); //lint !e522
        break;
    default:
        return -EOPNOTSUPP;
    }
    return 0;
}
static void __init rmnet_setup(struct net_device *dev)
{
	dev->open = rmnet_open;
	dev->stop = rmnet_stop;
	dev->hard_start_xmit = rmnet_xmit;
	dev->get_stats = rmnet_get_stats;
	dev->set_multicast_list = rmnet_set_multicast_list;
	dev->tx_timeout = rmnet_tx_timeout;

    dev->watchdog_timeo = 1000;

	ether_setup(dev);

	dev->change_mtu = 0; /* ??? */

	dev->do_ioctl = msm_rmnet_ioctl;

	random_ether_addr(dev->dev_addr);
}


static const char *ch_name[3] = {
	"DATA5",
	"DATA6",
	"DATA7",
};

static int __init rmnet_init(void)
{
	int ret;
	struct net_device *dev;
	struct rmnet_private *p;
	unsigned n;

	for (n = 0; n < 3; n++) {
		dev = alloc_netdev(sizeof(struct rmnet_private),
				   "rmnet%d", rmnet_setup);

		if (!dev)
			return -ENOMEM;

		p = netdev_priv(dev);
		p->chname = ch_name[n];

        p->skb = NULL;
        spin_lock_init(&p->lock);
        tasklet_init(&p->tsklt, _rmnet_resume_flow,
                   (unsigned long)dev);

		ret = register_netdev(dev);
		if (ret) {
			free_netdev(dev);
			return ret;
		}
	}
	return 0;
}


module_init(rmnet_init);
