#ifndef	__AUDIOFIFO_H__
#define	__AUDIOFIFO_H__	1

#include <linux/interrupt.h>	/* irqreturn_t */
struct completion;		/* see <linux/completion.h> */

/* physical limit for DMA engine (in samples) */
#define AUDIO_FIFO_MAX_LEN	1024 * 1024

typedef void (*tAudioFIFOCallBack) (void *pData);

typedef struct tAudioFIFO {
	/* Control */
	int bInit;
	int *pBuf;
	int len;
	int nFirst;
	int nLast;
	int inCount;
	int outCount;
	int outChunk;
	/* Status */
	int underruns;
	/* Intr Handler */
	int *pBaseAddr;
	int irqOffset;
	tAudioFIFOCallBack callBack;
	void *pCBData;
} tAudioFIFO;

/* should be same as is raptor_reg.h */
#define AUDIO_BLOCK_BASE_PHYS	0xA0700000
#define AUDIO_BLOCK_BASE_SIZE	0x00001000
#define AUDIO_IRQ_SIZE			0x0014
#define AUDIO_IRQ_STAT			0x0010
#define AUDIO_IRQ_RAW_STAT		0x0014
#define AUDIO_IRQ_CLEAR			0x0018
#define IRQ_AUDIO_OUT0			(32 + 27)
#define IRQ_AUDIO_OUT1			(32 + 28)

tAudioFIFO *GetAudioFIFO(void);
/* All lengths, counts, etc are in samples (which are stored in int's) */
static inline int GetAudioFIFOCount(tAudioFIFO *pCtrl)
{
	return (int)(pCtrl->inCount - pCtrl->outCount);
}
static inline int GetAudioFIFOSpace(tAudioFIFO *pCtrl)
{
	return pCtrl->len - GetAudioFIFOCount(pCtrl);
}

int InitAudioFIFO(tAudioFIFO *pCtrl, int *pBuf, int length);
int InitAudioFIFOHandler(tAudioFIFO *pCtrl, int *pBaseAddr, int irqNr,
			 tAudioFIFOCallBack callBack, void *pCBData);
void RestartAudioFIFO(tAudioFIFO *pCtrl);
void ClearAudioBuf(tAudioFIFO *pCtrl);
int RequestAudioFIFOWriteBuffer(tAudioFIFO *pCtrl, int **pBuf, int reqLen);
int DeliverAudioFIFOWriteBuffer(tAudioFIFO *pCtrl, int *pBuf, int len);
irqreturn_t HandleAudioFIFOInterrupt(int irqnr, void *devid,
				     struct pt_regs *regs);

#endif /*      __AUDIOFIFO_H__ */

