/* drivers/titan/audioSS.c
 *
 * Copyright (C) 2004,2008 QUALCOMM USA, INC.
 * 
 * All source code in this file is licensed under the following license
 * except where indicated.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org
 */
#include <linux/types.h>	/* <windows.h>  UINT8, DWORD, etc */
#include <linux/memory.h>	/* memcpy */
#include <linux/device.h>	/* struct device (dma_alloc_coherent) */
#include <linux/dma-mapping.h> /* enum dma_data_direction(dma_alloc_coherent)*/
#include <linux/soundcard.h>	/* audio_buf_info */
#include <linux/uaccess.h>	/* copy_from_user */
#include <linux/dma-mapping.h>	/* dma_alloc_coherent, dma_free_coherent */
#include <linux/io.h>		/* readl(), writel() */
#include "audiohw.h"		/* uint8, ... */
#include "raptor2.h"

#ifndef AUDIO_LOW_POWER
#define AUDIO_LOW_POWER	0
#endif

#include "audioSS.h"
#include "audioFIFO.h"
#if AUDIO_LOW_POWER
#include "AudioLowPower.h"
#endif

#include "platform.h"

/* This table was created using mndfind.pl
* The "Target clock frequency" was set to "(sample rate) * 512 * (0.8)"
* The * (0.8) was added because the output of the mndfind.pl script
* was creating icodex_rx frequencies that were 20% too fast on the test SURF
* This was verified using the "testclocks.cmm" script on the mARM Trace32
*/
AudioSSConfigType AudioSSConfig = {
	4,			/* Clock Source: PLL0 - MODEM PLL */
	{
	 {48000, 0x01, 0xfff5, 0xfff6}
	 ,			/* SR_48000 */
	 {44100, 0x93, 0xf9bf, 0xfa52}
	 ,			/* SR_44100 */
	 {32000, 0x01, 0xfff0, 0xfff1}
	 ,			/* SR_32000 */
	 {24000, 0x01, 0xffeb, 0xffec}
	 ,			/* SR_24000 */
	 {22050, 0x93, 0xf37f, 0xf412}
	 ,			/* SR_22050 */
	 {16000, 0x01, 0xffe1, 0xffe2}
	 ,			/* SR_16000 */
	 {12000, 0x01, 0xffd7, 0xffd8}
	 ,			/* SR_12000 */
	 {11025, 0x93, 0xe6ff, 0xe792}
	 ,			/* SR_11025 */
	 {8000, 0x01, 0xffc3, 0xffc4}	/* SR_8000 */
	 }
};

/* Configuration macros: */
#define SPOOF_RATES				(0 | NO_8x_UPSAMPLE)
#define DFLT_FRAMES_PER_KSEC	31000
#define PREFILL_FRAMES			2

/* Porting macros: */
/* Configure SCORPION PORT */
#define UNREACHABLE(a)		/* so compiler won't complain */
#define OUTP32(a, b)		WRITEL(b, a)
#define INP32(a)			READL(a)
#define INP8(a)			(uint8)READB(a)

/* Normally, use copy_to_user/copy_from_user. For short->int expansion, however,
 * I need a faster method: just give me an accessible ptr!  In 8K implementation
 *the user address space is already mapped and is accessible (in this writing)
 */
#define GET_ACCESS_TO_USER_DATA(p)	p

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif

/*
* globals:
*/
static tWBCodec gWBCodec = {
	.bConfig = 0,
	.count = 0
};

static tAudioConfig gAudioConfig = {
	.dmaOn = 0,
	.ctl = 0x013CF,
#if (IRQ_AUDIO_OUTn == IRQ_AUDIO_OUT0)
	.irqEnable0 = 0x01,	/* periodCnt Chan 0 */
	.irqEnable1 = 0,	/* none */
#else
	.irqEnable0 = 0,	/* none */
	.irqEnable1 = 0x01,	/* periodCnt Chan 0 */
#endif
	.bDisableOnEmpty = FALSE,
	.dma[0] =		/* DMA channel configurations */
	{
	 .chan = 0,
	 .ctl = 2,		/* WB, 2 samples */
	 .base = NULL,
	 .len = 0,
	 .periodLen = 0,
	 .defaultSample = 0},
	.dma[1] = {.chan = -1}
};

static tAudioSS gAudioSS = {
	.samplesPerSec = 0,
	.bStereo = TRUE,
	.framesPerKSec = 0,
	.bNonBlock = FALSE,
	.pAudioFIFO = NULL,
	.pAudioConfig = NULL,
	.pWBCodec = NULL,
	.mSecPlayed = 0
};

static int *gpAudioBuf;	/* virtual address of buffer */
static int *gAudioBufHandle;	/* physical address of buffer */
static int gAudioBufLen;

/*
* statics:
*/
static unsigned char gWBCodecRegisters[] = {
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
	0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
	0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
	0x48, 0x49, 0x4A, 0x4B, 0x4C
};

#if 0
static unsigned char gWBCodecMic1Ear[] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x71, 0x01, 0xFF,
	0x7F, 0x03, 0x00, 0x00, 0xB4, 0xB4, 0x00, 0x80,
	0x05, 0x1C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x01, 0x01, 0x00
};
#endif

/* Note: WBCodec has filter with programmable
*      cutoff frequency.  I believe the filter is
*      digital and assumes the input sample rate
*      is always 48kHz (upsampled externally).  We
*      will adjust the conversion clock, so we will
*      always allow the maximum bandwidth setting.
*/
static unsigned char gWBCodecHeadphone[] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x69, 0x69, 0x00,
	0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x10, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x06, 0x00, 0x00
};

void *audioBaseAddress;
void *audioClockCtlBase;
void *audioSSBIBase;

/* OUTP32/INP32 virtual address references:
* For Linux Kernel, most IO in addr range 0xAxxxxxxx have already been mapped
* to fixed address, for which the macro, PA_TO_VA, can be used
* For others, we need to use ioremap/iounmap
*/
/*===========================================================================
FUNCTION static void SetupIOVirtualAddresses(void)

DESCRIPTION  STUBBED

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
static void SetupIOVirtualAddresses(void)
{
	audioBaseAddress = ioremap(AUDIO_BASE_HW, 0x1000);
	audioClockCtlBase = ioremap(CLK_CTL_BASE_HW, 0x1000);
	audioSSBIBase = ioremap(SSBI_BASE_HW, 0x1000);
}

/*===========================================================================
FUNCTION static void ReturnIOVirtualAddresses(void)

DESCRIPTION STUBBED

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
static void ReturnIOVirtualAddresses(void)
{
	iounmap(audioBaseAddress);
	iounmap(audioClockCtlBase);
	iounmap(audioSSBIBase);
	audioBaseAddress = 0;
	audioClockCtlBase = 0;
	audioSSBIBase = 0;
}

/*===========================================================================
FUNCTION static int IsPrefilled( tAudioSS *pSS )

DESCRIPTION Returns if the Audion FIFO is prefilled.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
static int IsPrefilled(tAudioSS *pSS)
{
	int frameSize = pSS->pAudioFIFO->outChunk;	/* in samples */
	int count = GetAudioFIFOCount(pSS->pAudioFIFO);	/* in samples */
	int retVal = ((frameSize * PREFILL_FRAMES) <= count) ? 1 : 0;
	return retVal;
}


/*///////////////////
* WB Codec Code
*
* Copied with some modification from Nabil's rap_marm_audio.c file
*/

/*===========================================================================
FUNCTION void ssbi_init(uint8 which_chain)

DESCRIPTION   Initialize the ssbi, single wire serial interface so we can
   control the audio codec.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void ssbi_init(uint8 which_chain)
{
	uint32 ssbi_data_pden = 0x00;	/* [bit 16] */
	uint32 activate_reserve = 0x01;	/* [bit 15] */
	uint32 disable_term_sym = 0x00;	/* [bit 14] */
	uint32 slave_id = 0x00;	/* [bit 13:8] */
	uint32 ftm_mode = 0x00;	/* [bit 7] */
	uint32 sel_rd_data = 0x00;	/* [bits 6:5] */
	uint32 enable_ssbi_int = 0x01;	/* [bit 4] */
    uint32 ssbi_data_del = 0x01;	/* [bits 3:2] */
	uint32 idle_syms = 0x01;	/* [bits 1:0] */
	static uint32 reg_val;	/* static for mirroring last value written? */
	static char done;	/* SPL: TODO: RPC SBI calls */

	if (1 == done)
		return;
	done = 1;

	which_chain = CODEC_SSBI;
	if (which_chain == CODEC_SSBI) {
		/* Enable the clocks */
		OUTP32(HWIO_SBI_NS_REG_ADDR, (INP32(HWIO_SBI_NS_REG_ADDR) |
			     HWIO_SBI_NS_REG_CODEC_SSBI_ROOT_ENA_BMSK
			      |
			     HWIO_SBI_NS_REG_CODEC_SSBI_CLK_BRANCH_ENA_BMSK));

	}

	reg_val = ssbi_data_pden << 16 |
	    activate_reserve << 15 |
	    disable_term_sym << 14 |
	    slave_id << 8 |
	    ftm_mode << 7 |
	    sel_rd_data << 5 |
	    enable_ssbi_int << 4 | ssbi_data_del << 2 | idle_syms << 0;

	switch (which_chain) {
	case CODEC_SSBI:
		OUTP32(HWIO_CODEC_SSBI_CTL_ADDR, reg_val);
		break;

	default:
		break;
	}
}

/*===========================================================================
FUNCTION void ssbi_write(uint8 which_chain, uint32 reg_add, uint32 reg_data)

DESCRIPTION   Write a value out the ssbi interface to the reg passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void ssbi_write(uint8 which_chain, uint32 reg_add, uint32 reg_data)
{
	uint32 send_term_sym = 0x0;
	uint32 wakeup_slave = 0x0;
	uint32 use_enable = 0x0;
	uint32 rdwrn = 0x0;
	uint8 status = 0;
	static uint32 reg_value; /* static for mirroring last value written? */

	reg_value = send_term_sym << 27 |
	    wakeup_slave << 26 |
	    use_enable << 25 | rdwrn << 24 | reg_add << 16 | reg_data << 0;

/* This waits until the read operation is done, before leaving the function.*/
	while (!status) {
		status = INP8(HWIO_CODEC_SSBI_STATUS_ADDR);
		status = status & (1 << 1);
	}

	switch (which_chain) {
	case CODEC_SSBI:
		OUTP32(HWIO_CODEC_SSBI_CMD_ADDR, reg_value);
		break;

	default:
		break;
	}
}

/*===========================================================================
FUNCTION int InitWBCodec( tWBCodec* pWBCodec )

DESCRIPTION  Using the ssbi interface write commands to initialze the wide
   band codec.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int InitWBCodec(tWBCodec *pWBCodec)
{
	int i;

	ssbi_init(CODEC_SSBI);
	/* Titan */
	ssbi_write(CODEC_SSBI, 0x5A, 0x01);	/* CODEC enable */
	ssbi_write(CODEC_SSBI, 0x04, 0xC0);
	ssbi_write(CODEC_SSBI, 0x14, 0xFF);	/* Rx Left and Right */
	ssbi_write(CODEC_SSBI, 0x18, 0xE0);	/* Ear1 enable */
	ssbi_write(CODEC_SSBI, 0x19, 0xF0);	/* HPH  enable */
	ssbi_write(CODEC_SSBI, 0x1A, 0x48);	/* Select stereo HPH and Ear1 */

	pWBCodec->bConfig = 1;
	return 0;
}

/*===========================================================================
FUNCTION int TurnoffWBCodec( tWBCodec* pWBCodec )

DESCRIPTION  Using the ssbi interface turn off the wide band codec.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int TurnoffWBCodec(tWBCodec *pWBCodec)
{

	pWBCodec->bConfig = 0;
	return 0;
}

/*===========================================================================
FUNCTION int InitAudioClocks( int samplesPerSecond )

DESCRIPTION  Enable and initialize the clocks for the audio output based on
   the samplesPerSecond variable passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int InitAudioClocks(int samplesPerSecond)
{
	static unsigned int md, ns;
	AudioSSSamplingRateType sr = (AudioSSSamplingRateType) 0;

	while (sr < SR_COUNT
	       && AudioSSConfig.config[sr].rate != samplesPerSecond)
		sr++;

	if (sr == SR_COUNT)
		sr = SR_8000;


	md = (AudioSSConfig.config[sr].m << 16) | AudioSSConfig.config[sr].
	    not_2d;

	/* Set NS upper 16 bits */
	ns = (AudioSSConfig.config[sr].not_nm << 16) |
	    (1 << HWIO_ICODEC_RX_NS_REG_ICODEC_RX_CLK_DIV_SHFT)|
	    (HWIO_ICODEC_RX_NS_REG_ICODEC_2X_RX_CLK_BRANCH_ENA_BMSK) |
	    (HWIO_ICODEC_RX_NS_REG_ICODEC_RX_ROOT_ENA_BMSK) |
	    (HWIO_ICODEC_RX_NS_REG_ICODEC_RX_CLK_BRANCH_ENA_BMSK) |
	    (HWIO_ICODEC_RX_NS_REG_MNCNTR_EN_BMSK) |
	    (2 << HWIO_ICODEC_RX_NS_REG_MNCNTR_MODE_SHFT) |
	    AudioSSConfig.clock_source;

	OUTP32(HWIO_ROW_RESET_ADDR,
	       (INP32(HWIO_ROW_RESET_ADDR) | HWIO_ROW_RESET_ICODEC_RX_BMSK));
	OUTP32(HWIO_ICODEC_RX_MD_REG_ADDR, md);
	OUTP32(HWIO_ICODEC_RX_NS_REG_ADDR, ns);
	OUTP32(HWIO_ROW_RESET_ADDR,
	       (INP32(HWIO_ROW_RESET_ADDR) & ~HWIO_ROW_RESET_ICODEC_RX_BMSK));
	return 0;
}

/* Audio Block Code */

/*===========================================================================
FUNCTION void InitAudioDMA(tAudioConfig *pAudio)

DESCRIPTION  Initialize the audio DMA, clear any old information and leave
   the DMA in the OFF state.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void InitAudioDMA(tAudioConfig *pAudio)
{
	/* disable and clear any prior Audio interrupt settings */
	OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(0), 0);
	OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(1), 0);
	OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(0), 0xFFFFFFFF);
	OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(1), 0xFFFFFFFF);

	OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(0), 0);	/* DMA off */
	OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(1), 0);	/* DMA off */
	OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(2), 0);	/* DMA off */
	OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(3), 0);	/* DMA off */
	pAudio->dmaOn = 0;
}

/*===========================================================================
FUNCTION int SetupAudioDMA(tAudioConfig *pAudio)

DESCRIPTION  Setup the audio DMA based one the information for  the audio
  channel passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int SetupAudioDMA(tAudioConfig *pAudio)
{
	int n;

	/* DMA Channel setup */
	n = pAudio->dma[0].chan;
	if (n >= 0) {
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[0].ctl & (~1));
		OUTP32(HWIO_AUDIO_DMA_BASEa_ADDR(n), pAudio->dma[0].base);
		OUTP32(HWIO_AUDIO_DMA_BUFF_LENa_ADDR(n), pAudio->dma[0].len);
		OUTP32(HWIO_AUDIO_DMA_PER_LENa_ADDR(n),
		       pAudio->dma[0].periodLen);
		OUTP32(HWIO_AUDIO_DMA_DEFa_ADDR(n),
		       pAudio->dma[0].defaultSample);
	}
	n = pAudio->dma[1].chan;
	if (n >= 0) {
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[1].ctl & (~1));
		OUTP32(HWIO_AUDIO_DMA_BASEa_ADDR(n), pAudio->dma[1].base);
		OUTP32(HWIO_AUDIO_DMA_BUFF_LENa_ADDR(n), pAudio->dma[1].len);
		OUTP32(HWIO_AUDIO_DMA_PER_LENa_ADDR(n),
		       pAudio->dma[1].periodLen);
		OUTP32(HWIO_AUDIO_DMA_DEFa_ADDR(n),
		       pAudio->dma[1].defaultSample);
	}
	pAudio->dmaOn = 0;
	pAudio->bDisableOnEmpty = FALSE;

	/* Setup Audio Block interrupt source */
	if (pAudio->irqEnable0) {
		OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(0), 0xFFFFFFFF);
	OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(0), pAudio->irqEnable0);
	}
	if (pAudio->irqEnable1) {
		OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(1), 0xFFFFFFFF);
		OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(1), pAudio->irqEnable1);
	}
	/* Configure AudioSS control reg */
	switch (pAudio->interface) {
	case AUDIOSS_IF_WB:
		OUTP32(HWIO_AUDIO_WB_CTL_ADDR, INP32(HWIO_AUDIO_WB_CTL_ADDR)
				 | 0x01000000);	/* RX_RESET */
		OUTP32(HWIO_AUDIO_WB_CTL_ADDR, pAudio->ctl | 0x01000000);
		OUTP32(HWIO_AUDIO_WB_CTL_ADDR, pAudio->ctl);
		break;
	case AUDIOSS_IF_I2S:
		OUTP32(HWIO_AUDIO_I2S_CTL_ADDR, pAudio->ctl);
		break;
	case AUDIOSS_IF_PCM:
		OUTP32(HWIO_AUDIO_PCM_CTL_ADDR, pAudio->ctl);
		break;
	}

	return 0;
}

/*===========================================================================
FUNCTION void EnableAudioDMA( tAudioConfig *pAudio )

DESCRIPTION  Enable the audio DMA for the channel passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void EnableAudioDMA(tAudioConfig *pAudio)
{
	int n;
	/* Start channel(s) */
	n = pAudio->dma[0].chan;
	if (n >= 0)
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[0].ctl | 1);
	n = pAudio->dma[1].chan;
	if (n >= 0)
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[1].ctl | 1);
	pAudio->dmaOn = 1;
}

/*===========================================================================
FUNCTION void DisableAudioDMA( tAudioConfig *pAudio )

DESCRIPTION  Disable the audio DMA for the channel number passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void DisableAudioDMA(tAudioConfig *pAudio)
{
	int n;

	/* Stop channel(s) */
	n = pAudio->dma[0].chan;
	if (n >= 0)
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[0].ctl & ~1);
	n = pAudio->dma[1].chan;
	if (n >= 0)
		OUTP32(HWIO_AUDIO_DMA_CTLa_ADDR(n), pAudio->dma[1].ctl & ~1);
	pAudio->dmaOn = 0;
	pAudio->bDisableOnEmpty = FALSE;

	/* Note: when buffer reenabled, current pointer will be reset
		to beginning of DMA buffer */
}

/*===========================================================================
FUNCTION int TeardownAudioDMA(tAudioConfig *pAudio)

DESCRIPTION  Teardown the audio DMA (shut it down) for the channel
   number passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int TeardownAudioDMA(tAudioConfig *pAudio)
{
	/* Stop channel(s) */
	DisableAudioDMA(pAudio);

	switch (pAudio->interface) {
	case AUDIOSS_IF_WB:
		/*OUTP32(HWIO_AUDIO_WB_CTL_ADDR,
				(pAudio->ctl | HWIO_AUDIO_WB_CTL_TX_RESET_BMSK
					| HWIO_AUDIO_WB_CTL_RX_RESET_BMSK) ); */
		break;
	case AUDIOSS_IF_I2S:
		/*OUTP32(HWIO_AUDIO_I2S_CTL_ADDR, 0); */
		break;
	case AUDIOSS_IF_PCM:
		/*OUTP32(HWIO_AUDIO_PCM_CTL_ADDR, 0x00028000); */
		break;
	}

	/* Audio SubSystem (Audio Block) teardown */
	if (pAudio->irqEnable0) {
		OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(0), 0);
		OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(0), 0xFFFFFFFF);
	}
	if (pAudio->irqEnable1) {
		OUTP32(HWIO_AUDIO_IRQ_ENa_ADDR(1), 0);
		OUTP32(HWIO_AUDIO_IRQ_CLEARa_ADDR(1), 0xFFFFFFFF);
	}

	return 0;
}

/*===========================================================================
FUNCTION int ReconfigAudioDMA(int newPeriod)

DESCRIPTION  Reconfigure the audio DMA for the new parameters passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int ReconfigAudioDMA(int newPeriod)
{
	if (newPeriod > 0) {
		int n;
		int audioFIFOLen = (gAudioBufLen / newPeriod) * newPeriod;

		if (InitAudioFIFO(gAudioSS.pAudioFIFO, gpAudioBuf, audioFIFOLen)
		    < 0)
			while (1) ;
#if AUDIO_LOW_POWER
		if (gAudioSS.framesPerKSec == 0) {
	/* Set up Audio-only playback so that we only interrupt
				when the FIFO is nearly empty */
	/*int bufferLowLevel = audioFIFOLen >> 1; */
#if NO_SW_8x_UPSAMPLE
			int bufferLowLevel = gAudioSS.samplesPerSec * 03 / 100;
#else
		int bufferLowLevel = gAudioSS.samplesPerSec * 8 * 03 / 100;
#endif
			bufferLowLevel = ((bufferLowLevel + newPeriod - 1)
						/ newPeriod) * newPeriod;
			if (audioFIFOLen > bufferLowLevel)
				newPeriod = audioFIFOLen - bufferLowLevel;
		}
#endif
		gAudioSS.pAudioFIFO->outChunk = newPeriod;

		n = gAudioSS.pAudioConfig->dma[0].chan;
		if (n >= 0) {
			gAudioSS.pAudioConfig->dma[0].len = audioFIFOLen - 1;
			gAudioSS.pAudioConfig->dma[0].periodLen = newPeriod - 1;
		}
		n = gAudioSS.pAudioConfig->dma[1].chan;
		if (n >= 0) {
			gAudioSS.pAudioConfig->dma[1].len = audioFIFOLen - 1;
			gAudioSS.pAudioConfig->dma[1].periodLen = newPeriod - 1;
		}

		gAudioSS.mSecPlayed = 0;

		SetupAudioDMA(gAudioSS.pAudioConfig);
	} else
		TeardownAudioDMA(gAudioSS.pAudioConfig);

	return 0;
}

/* Audio Subsystem Code */

/*===========================================================================
FUNCTION void FrameCompleteCallBack(void *pData)

DESCRIPTION  Handler for when a frame is finished being processed by the
   audio block

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void FrameCompleteCallBack(void *pData)
{
	tAudioSS *pSS = (tAudioSS *) pData;
	if ((pSS->pAudioConfig->bDisableOnEmpty)
	    && (0 == GetAudioFIFOCount(pSS->pAudioFIFO))) {
		DisableAudioDMA(pSS->pAudioConfig);
/* Reset FIFO pointer's in anticipation of starting up again,
		but leave counts intact*/
		RestartAudioFIFO(pSS->pAudioFIFO);
		pSS->pAudioConfig->bDisableOnEmpty = FALSE;
	}
	complete(&pSS->writeReady);
}


/*===========================================================================
FUNCTION tAudioSS* GetAudioSS(void)

DESCRIPTION  Return the audio ss pointer.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
tAudioSS *GetAudioSS(void)
{
	return &gAudioSS;
}

/*===========================================================================
FUNCTION tAudioSS* InitAudioSS(int prefBufLen)

DESCRIPTION  Initialize the audion subsystem, setup the clocks, sample rates,
   stereo, etc.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
tAudioSS *InitAudioSS(int prefBufLen)
{
	int status;

	/* Initialize AudioSS settings */
	gAudioSS.samplesPerSec = 0;
	gAudioSS.bStereo = TRUE;
	gAudioSS.framesPerKSec = 0;
	gAudioSS.bNonBlock = FALSE;
	init_completion(&gAudioSS.writeReady);
	gAudioSS.pAudioFIFO = GetAudioFIFO();
	gAudioSS.pAudioConfig = &gAudioConfig;
	gAudioSS.pWBCodec = &gWBCodec;
	gAudioSS.mSecPlayed = 0;

	/* Map HW addresses */
	SetupIOVirtualAddresses();

	/* Allocate output buffer */
	if (NULL == gpAudioBuf) {
		gAudioBufLen = prefBufLen;

	/* Keep trying smaller sizes from MAX until we get something */
		do {
			if (NULL !=
			    (gpAudioBuf =
			     dma_alloc_coherent(NULL,
						gAudioBufLen * sizeof(int),
						(dma_addr_t
						 *) (&gAudioBufHandle),
						GFP_KERNEL)))
				break;
			gAudioBufLen >>= 1;
		} while (gAudioBufLen > 0);

		if (NULL == gpAudioBuf)
			return NULL;
	}

	/* Initialize audio output pipeline */

	InitAudioFIFO(gAudioSS.pAudioFIFO, gpAudioBuf, gAudioBufLen);
	ClearAudioBuf(gAudioSS.pAudioFIFO);	/* prefill audio FIFO w/ 0's */
	InitAudioFIFOHandler(gAudioSS.pAudioFIFO,
			     (int *)AUDIO_BASE,
			     IRQ_AUDIO_OUTn, FrameCompleteCallBack, &gAudioSS);
	/* Install interrupt handler */
	status = request_irq(IRQ_AUDIO_OUTn,
			     HandleAudioFIFOInterrupt,
			     0, "AudioSS_rx", gAudioSS.pAudioFIFO);
	if (status != 0) {
		/* deallocate memory? or hold onto it? */
	/* We don't expect request_irq to fail, so just hold onto it */
		return NULL;
	}

	InitAudioDMA(gAudioSS.pAudioConfig);

#if (0 == NO_SW_8x_UPSAMPLE)
	vInitUpsampleBuf();
#endif

	return &gAudioSS;
}

/*===========================================================================
FUNCTION int ConfigAudioSS( tAudioSS *pAudioSS )

DESCRIPTION  Configure the audio subsystem based on the ppointer to the
   subsystem passed.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int ConfigAudioSS(tAudioSS *pAudioSS)
{
	int len;
	int period;
	unsigned int dmaCtlAddr = 0;

	/* Configure WB Codec */
	len = sizeof(gWBCodecHeadphone);
	pAudioSS->pWBCodec->count = len;
	memcpy(pAudioSS->pWBCodec->reg, gWBCodecRegisters, len);
	memcpy(pAudioSS->pWBCodec->val, gWBCodecHeadphone, len);
	InitWBCodec(pAudioSS->pWBCodec);
#if NO_8x_UPSAMPLE
	/* convert sample rate to equivalent pre-upsampled rate */
	InitAudioClocks(pAudioSS->samplesPerSec >> 3);
#else
	InitAudioClocks(pAudioSS->samplesPerSec);
#endif

	/* Configure pAudioConfig */
	pAudioSS->pAudioConfig->interface = AUDIOSS_IF_WB;
	pAudioSS->pAudioConfig->ctl = 0x01000 |	/* SEL_SATURN */
	    0x00380 |	/* DITHER: bypass, delayed sum, incl delay regs */
#if (0 == NO_HW_8x_UPSAMPLE)
	    0x00010 |		/* 8x Upsampler (RX_FILT_EN) */
#endif
	    0x00040;		/* SATURN2 (FLIP_SEND2XD) */
	if (pAudioSS->bStereo)
		pAudioSS->pAudioConfig->ctl |= 0x00003;
	/* else, replicate mono on both left, right*/

#if (IRQ_AUDIO_OUTn == IRQ_AUDIO_OUT0)
	pAudioSS->pAudioConfig->irqEnable0 = 1;
	pAudioSS->pAudioConfig->irqEnable1 = 0;
#else
	pAudioSS->pAudioConfig->irqEnable0 = 0;
	pAudioSS->pAudioConfig->irqEnable1 = 1;
#endif

	dmaCtlAddr = HWIO_AUDIO_DMA_CTLa_ADDR(0);
	pAudioSS->pAudioConfig->dma[0].chan = 0;
	pAudioSS->pAudioConfig->dma[0].ctl = 6;	/* WB, 4 sample watermark */
	pAudioSS->pAudioConfig->dma[0].base = gAudioBufHandle;
	pAudioSS->pAudioConfig->dma[0].defaultSample = 0;

	pAudioSS->pAudioConfig->dma[1].chan = -1;

	if (0 >= pAudioSS->framesPerKSec)
		pAudioSS->framesPerKSec = DFLT_FRAMES_PER_KSEC;
#if NO_SW_8x_UPSAMPLE
	period = (pAudioSS->samplesPerSec * 1000) / pAudioSS->framesPerKSec;
#else
	period = (pAudioSS->samplesPerSec * 8 * 1000) / pAudioSS->framesPerKSec;
#endif
	ReconfigAudioDMA(period);	/* also configures pAudioFIFO */

	return 0;
}

/*===========================================================================
FUNCTION int GetAudioSSFrameSize( tAudioSS *pSS )

DESCRIPTION  Return the audio subsystem frame size based on the kind of
  sampling.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int GetAudioSSFrameSize(tAudioSS *pSS)
{
	int retVal = pSS->pAudioFIFO->outChunk * 2;	/* bytes/PCM */
	if (pSS->bStereo)
		retVal *= 2;	/* PCM/samples */
#if (0 == NO_SW_8x_UPSAMPLE)
	retVal /= 8;		/* hide 8x upsampling */
#endif
	return retVal;
}

/*===========================================================================
FUNCTION void GetAudioSSBufInfo( tAudioSS *pSS, struct audio_buf_info *pInfo )

DESCRIPTION Return information about the audion buffer, up sampling, size, etc.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void GetAudioSSBufInfo(tAudioSS *pSS, struct audio_buf_info *pInfo)
{
	if (pSS->pAudioFIFO->outChunk == 0)
		pSS->pAudioFIFO->outChunk = 2;

	pInfo->fragstotal = pSS->pAudioFIFO->len / pSS->pAudioFIFO->outChunk;
	pInfo->fragsize = pSS->pAudioFIFO->outChunk * 2;	/* PCM size */
	pInfo->bytes = GetAudioFIFOSpace(pSS->pAudioFIFO) * 2;	/* PCM size */
#if (0 == NO_SW_8x_UPSAMPLE)
	pInfo->fragsize /= 8;	/* hide 8x upsampling */
	pInfo->bytes /= 8;	/* hide 8x upsampling */
#endif
	if (pSS->bStereo) {
		pInfo->fragsize *= 2;	/* PCM/sample */
		pInfo->bytes *= 2;	/* PCM/sample */
	}

	if (0 == pInfo->fragsize)
		pInfo->fragsize = 1;

	pInfo->fragments = pInfo->bytes / pInfo->fragsize;
}


/*===========================================================================
FUNCTION int WriteAudioSSMono( tAudioSS *pSS, short *pData, int len )

DESCRIPTION  Set the audio subsystem to mono.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int WriteAudioSSMono(tAudioSS *pSS, short *pData, int len)
{
	int retVal = 0;
	int bufLen, inLen;
	int *pBuf;

#if NO_SW_8x_UPSAMPLE
	while (len) {
		int n;
		bufLen =
		    RequestAudioFIFOWriteBuffer(pSS->pAudioFIFO, &pBuf, len);
		if (0 > bufLen)
			return -1;

		inLen = bufLen;
		if (0 == inLen) {
			DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf,
						bufLen);
			if (pSS->bNonBlock) {
				break;
			} else {
				wait_for_completion(&pSS->writeReady);
				continue;
			}
		}
		len -= inLen;
		retVal += inLen;
		{
			short *pTmp = GET_ACCESS_TO_USER_DATA(pData);
			for (n = 0; n < bufLen; n++)
				pBuf[n] = pTmp[n];

		}
		pData += inLen;
		DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf, bufLen);
	}
#else
	while (len) {
		bufLen =
		    RequestAudioFIFOWriteBuffer(pSS->pAudioFIFO, &pBuf,
						len << 3);
		if (0 > bufLen)
			return -1;

		inLen = bufLen >> 3;
		/* assert( (inLen <<3) == bufLen ); */
		if (0 == inLen) {
			DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf,
							bufLen);
			if (pSS->bNonBlock) {
				break;
			} else {
				wait_for_completion(&pSS->writeReady);
				continue;
			}
		}
		len -= inLen;
		retVal += inLen;
		{
			short *pTmp = GET_ACCESS_TO_USER_DATA(pData);
			vUpsample8Mono(pBuf, pTmp, inLen);
		}
		pData += inLen;
		DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf, bufLen);
	}
#endif

	if ((!(pSS->pAudioConfig->dmaOn)) && (IsPrefilled(pSS)))
		EnableAudioDMA(pSS->pAudioConfig);

	return retVal;
}


/*===========================================================================
FUNCTION int WriteAudioSSStereo( tAudioSS *pSS, int *pData, int len )

DESCRIPTION  Set the audio subsystem to stereo.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int WriteAudioSSStereo(tAudioSS *pSS, int *pData, int len)
{
	int retVal = 0;
	int bufLen, inLen;
	int *pBuf;
	int status;

#if NO_SW_8x_UPSAMPLE
	while (len) {
		bufLen =
		    RequestAudioFIFOWriteBuffer(pSS->pAudioFIFO, &pBuf, len);
		if (0 > bufLen)
			return -1;

		inLen = bufLen;
		if (0 == inLen) {
			DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf,
							 bufLen);
			if (pSS->bNonBlock) {
				break;
			} else {
				wait_for_completion(&pSS->writeReady);
				continue;
			}
		}
		len -= inLen;
		retVal += inLen;
		status = copy_from_user(pBuf, pData, bufLen * sizeof(int));
		pData += bufLen;
		DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf, bufLen);
	}
#else
	while (len) {
		bufLen =
		    RequestAudioFIFOWriteBuffer(pSS->pAudioFIFO, &pBuf,
						len << 3);
		if (0 > bufLen)
			return -1;

		inLen = bufLen >> 3;
		/* assert( (inLen <<3) == bufLen ); */
		if (0 == inLen) {
			DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf,
							 bufLen);
			if (pSS->bNonBlock) {
				break;
			} else {
				wait_for_completion(&pSS->writeReady);
				continue;
			}
		}
		len -= inLen;
		retVal += inLen;
		{
			int *pTmp = GET_ACCESS_TO_USER_DATA(pData);
			vUpsample8Stereo(pBuf, pTmp, inLen);
		}
		pData += inLen;
		DeliverAudioFIFOWriteBuffer(pSS->pAudioFIFO, pBuf, bufLen);
	}
#endif

	if ((!(pSS->pAudioConfig->dmaOn)) && (IsPrefilled(pSS)))
		EnableAudioDMA(pSS->pAudioConfig);

	return retVal;
}

/*===========================================================================
FUNCTION int FlushAudioSS( tAudioSS *pSS )

DESCRIPTION  Flush that data in the audio subsystem, dma, buffers, etc.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int FlushAudioSS(tAudioSS *pSS)
{
	int count;
	int padLen;
	int *pBuf;

	/* have DMA turn off when FIFO empty */
	pSS->pAudioConfig->bDisableOnEmpty = TRUE;

	count = GetAudioFIFOCount(pSS->pAudioFIFO);
	if (0 >= count) {
		DisableAudioDMA(pSS->pAudioConfig);
		return 0;
	}
	/* zero-pad for up to one frame/interrupt period (outChunk) */
	padLen =
	    RequestAudioFIFOWriteBuffer(pSS->pAudioFIFO, &pBuf,
					pSS->pAudioFIFO->outChunk);
	memset(pBuf, 0, (padLen * sizeof(int)));

	count *= 2;		/* PCM size */
#if (0 == NO_SW_8x_UPSAMPLE)
	count /= 8;		/* hide 8x upsampling */
#endif
	if (pSS->bStereo)
		count *= 2;	/* PCM/sample */

	if (!(pSS->pAudioConfig->dmaOn) && (pSS->pAudioConfig->bDisableOnEmpty))
		EnableAudioDMA(pSS->pAudioConfig);

	return count;
}

/*===========================================================================
FUNCTION int TeardownAudioSS( tAudioSS *pAudioSS, int bDoItNow )

DESCRIPTION  Tear down the  audio subsystem either now or wait for it
   to finish processing.

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int TeardownAudioSS(tAudioSS *pAudioSS, int bDoItNow)
{
	/* Wait for FIFO to clear */
	if (!bDoItNow) {
		if (!(pAudioSS->pAudioConfig->dmaOn))
			EnableAudioDMA(pAudioSS->pAudioConfig);
		while (GetAudioFIFOCount(pAudioSS->pAudioFIFO) > 0)
			wait_for_completion(&pAudioSS->writeReady);
	}
	/* Teardown DMA channel */
	TeardownAudioDMA(pAudioSS->pAudioConfig);

	/* Turn off WB Codec */
	TurnoffWBCodec(pAudioSS->pWBCodec);

	/* Don't have to do anything with FIFO front end */
	return 0;
}

/*===========================================================================
FUNCTION void CleanupAudioSS(void)

DESCRIPTION  Cleanup the audio subsystem.  Tear down the DMA, turn of the
    codec, set buffer pointers back to initial values...

DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
void CleanupAudioSS(void)
{
	if (gAudioSS.pAudioConfig->dmaOn) {
		TeardownAudioDMA(gAudioSS.pAudioConfig);
		TurnoffWBCodec(gAudioSS.pWBCodec);
	}

	if (NULL != gpAudioBuf) {
		dma_free_coherent(NULL, gAudioBufLen * sizeof(int), gpAudioBuf,
				  (dma_addr_t) gAudioBufHandle);
		gpAudioBuf = NULL;
		gAudioBufHandle = NULL;
	}

	ReturnIOVirtualAddresses();
}

