#ifndef	__AUDIOSS_H__
#define	__AUDIOSS_H__	1

#include <linux/completion.h>	/* struct completion */
#include "audioFIFO.h"

#define IRQ_AUDIO_OUTn			IRQ_AUDIO_OUT1

#define NO_HW_8x_UPSAMPLE                        0
#define NO_SW_8x_UPSAMPLE                        1
#define NO_8x_UPSAMPLE		(NO_HW_8x_UPSAMPLE & NO_SW_8x_UPSAMPLE)

typedef enum AudioSSSamplingRateEnum {
	SR_48000 = 0,
	SR_44100,
	SR_32000,
	SR_24000,
	SR_22050,
	SR_16000,
	SR_12000,
	SR_11025,
	SR_8000,
	SR_COUNT
} AudioSSSamplingRateType;

typedef struct AudioSSRateConfigStruct {
	unsigned int rate;
	unsigned int m;
	unsigned int not_2d;
	unsigned int not_nm;
} AudioSSRateConfigType;

typedef struct AudioSSConfigStruct {
	unsigned int clock_source;
	AudioSSRateConfigType config[SR_COUNT];
} AudioSSConfigType;

extern AudioSSConfigType AudioSSConfig;

/* Here is the default clock scheme on Scorpion (RUMI 2).
* When changing clock source, make sure ALL FOUR settings below are correct.
* The rest of the definitions fall out from these:
*
* When we go to other possible sources, make ADJ_CLOCK_SOURCE 1.
* At that point, clock settings are not predefined and involve some calculation
* at run-time.*/
/*
#define ADJ_CLOCK_SOURCE		0

#define	TCXO_AUDIO_CLOCK_SOURCE	0
#define TCXO_AUDIO_CLOCK_FREQ	19200000

#define	PLAND1_AUDIO_CLOCK_SOURCE	4
#define PLAND1_AUDIO_CLOCK_FREQ		196608000

#define	PLANA1_AUDIO_CLOCK_SOURCE	1
#define PLANA1_AUDIO_CLOCK_FREQ		768000000

#define	PLANL96_AUDIO_CLOCK_SOURCE	2
#define PLANL96_AUDIO_CLOCK_FREQ	96000000

#define	PLANL192_AUDIO_CLOCK_SOURCE
#define PLANL192_AUDIO_CLOCK_FREQ	192000000

#define	PLANL288_AUDIO_CLOCK_SOURCE	2
#define PLANL288_AUDIO_CLOCK_FREQ	288000000
*/
/*           fclock * M
* faudio = ------------
*          512 * P * N
*
*     512 * P * N *faudio
* M = -------------------
*      fclock
*
* M must be <= N/2
* N-M must be < 65536
* P may be 1,2, or 4
*/
typedef enum {
	CODEC_SSBI = 0,
	TX_SSBI,
	PMIC_SSBI
} eSSBIChain;

typedef enum {
	AUDIOSS_IF_WB = 0,
	AUDIOSS_IF_PCM,
	AUDIOSS_IF_I2S
} eAudioSSInterface;

typedef struct tWBCodec {
	int bConfig;
	int count;
	unsigned char reg[32];
	unsigned char val[32];
} tWBCodec;

typedef struct tDMAChanConfig {
	int chan;
	int ctl;
	int *base;
	int len;
	int periodLen;
	int defaultSample;
} tDMAChanConfig;

typedef struct tAudioConfig {
	int dmaOn;
	eAudioSSInterface interface;
	int ctl;
	int irqEnable0;
	int irqEnable1;
	int bDisableOnEmpty;
	tDMAChanConfig dma[2];
} tAudioConfig;

typedef struct tAudioSS {
	/* configuration */
	int samplesPerSec;
	int bStereo;
	int framesPerKSec;
	int bNonBlock;
	/* control */
	struct completion writeReady;
	tAudioFIFO *pAudioFIFO;
	tAudioConfig *pAudioConfig;
	tWBCodec *pWBCodec;
	/* status */
	unsigned int mSecPlayed;
} tAudioSS;

struct audio_buf_info;


/* AudioSS */
tAudioSS *GetAudioSS(void);
tAudioSS *InitAudioSS(int prefBufSize);
int ConfigAudioSS(tAudioSS *pAudioSS);
int GetAudioSSFrameSize(tAudioSS *pSS);
void GetAudioSSBufInfo(tAudioSS *pSS, struct audio_buf_info *pInfo);
int WriteAudioSSMono(tAudioSS *pSS, short *pData, int len);
int WriteAudioSSStereo(tAudioSS *pSS, int *pData, int len);
int FlushAudioSS(tAudioSS *pSS);
int TeardownAudioSS(tAudioSS *pAudioSS, int bDoItNow);
void CleanupAudioSS(void);	/* counterpart of InitAudioSS */

#if (0 == NO_SW_8x_UPSAMPLE)
/* Until becomes part of scLib:*/
extern void vInitUpsampleBuf(void);
#endif

#if (0 == NO_SW_8x_UPSAMPLE)
extern int vUpsample8Mono(int *pOut, short *pIn, int inLen);
#endif

#if (0 == NO_SW_8x_UPSAMPLE)
/* Until becomes part of scLib:*/
extern int vUpsample8Stereo(int *pOut, int *pIn, int inLen);
#endif

#endif / *__AUDIOSS_H__ * /

