/*===========================================================================
				<Raptor.h>
DESCRIPTION: Define offsets and specifics for the Raptor(8600) chip

EXTERNALIZED FUNCTIONS

INITIALIZATION AND SEQUENCING REQUIREMENTS
  Detail how to initialize and use this service.  The sequencing aspect
  is only needed if the order of operations is important.

  Copyright (c) 2005,2008 QUALCOMM USA, INC.
  
  All source code in this file is licensed under the following license
  except where indicated.
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, you can find it at http://www.fsf.org
===========================================================================*/

#ifndef __raptor_2_h__
#define __raptor_2_h__

/*===========================================================================

			INCLUDE FILES FOR MODULE

===========================================================================*/
#define __MSMHW_PROC_DEF__  1
#define __MSMHW_MODEM_PROC__ 1

#include "audiohw.h"

/*===========================================================================

		DEFINITIONS AND DECLARATIONS FOR MODULE

This section contains definitions for constants, macros, types, variables
and other items needed by this module.

===========================================================================*/

/*===========================================================================
  Constant / Define Declarations
===========================================================================*/

/*
 * LEDs are not in the msmhwioreg.h file
 */
#define LED_P_ADDRESS 0x70000282

/*
 * Conversion from virtual to physical and back, needs to be defined
 * for all platforms.  Nice if the memory map supports a simple arithmatic
 * calculation so a function does not have to be called.
 *
 * For raptor all of the apps processor IO is in the 0xA0000000 range.  Since
 * the kernel likes to hang out at C0000000, we will just add 0x50000000 to
 * the physical address to create the virtual
 */

#define VA_OFFSET 0		/*0x50000000 */
#define PA_TO_VA(pa)  ((pa) + VA_OFFSET)
#define PA_TO_VA_2(pa)  ((pa) + 0x60000000)
#define VA_TO_PA(va)  ((va) - VA_OFFSET)

#define VA_1M_PAGE_MASK(a) (a)&0xFFFFF
#define VA_4K_PAGE_MASK(a) (a)&0xFFF

#define VIC_VIRT_ADDR(a) (a)+VA_OFFSET

/*
 * Add Raptor changes
 */

#define RAPTOR_A(a) HWIO_ ## a ## _ADDR

/*
 * Interrupt Controller
 */
#define A11S_VIC_BASE SPSS_VIC_BASE
#define VICIRQ_STATUS_0 RAPTOR_A(VICIRQ_STATUS_0)
#define VICINTENCLEAR_0 RAPTOR_A(VICINTENCLEAR_0)
#define VICINTENCLEAR_1 RAPTOR_A(VICINTENCLEAR_1)
#define VICINTCLEAR_0   RAPTOR_A(VICINTCLEAR_0)
#define VICINTCLEAR_1   RAPTOR_A(VICINTCLEAR_1)
#define VICINTEN_0      RAPTOR_A(VICINTEN_0)
#define VICINTEN_1      RAPTOR_A(VICINTEN_1)
#define VICINTSELECT_0  RAPTOR_A(VICINTSELECT_0)
#define VICINTSELECT_1  RAPTOR_A(VICINTSELECT_1)
#define VICINTEN_0      RAPTOR_A(VICINTEN_0)
#define VICINTEN_1      RAPTOR_A(VICINTEN_1)
#define VICINTENCLEAR_0 RAPTOR_A(VICINTENCLEAR_0)
#define VICINTENCLEAR_1 RAPTOR_A(VICINTENCLEAR_1)
#define VICINTCLEAR_0   RAPTOR_A(VICINTCLEAR_0)
#define VICINTCLEAR_1   RAPTOR_A(VICINTCLEAR_1)
#define VICCONFIG       RAPTOR_A(VICCONFIG)
#define VICINTTYPE_0    RAPTOR_A(VICINTTYPE_0)
#define VICINTTYPE_1    RAPTOR_A(VICINTTYPE_1)
/* #define VICVECTADDRn(n)    RAPTOR_A(VICVECTADDRn)(n) */
#define VICVECTADDRn(n)    0xAC000400
/* #define VICVECTPRIORITYn(n) RAPTOR_A(VICVECTPRIORITYn )(n) */
#define VICVECTPRIORITYn(n) 0xAC000200
#define VIC_INT_POLARITY_0 RAPTOR_A(VIC_INT_POLARITY_0)
#define VIC_INT_POLARITY_1 RAPTOR_A(VIC_INT_POLARITY_1)

#define VICINTMASTEREN   RAPTOR_A(VICINTMASTEREN)
#define VICINTMASTEREN__IRQ_ENABLE___M HWIO_VICINTMASTEREN_IRQ_ENABLE_BMSK
#define VICFIQ_STATUS_0__GP_TIMER_EXP___S HWIO_VICFIQ_STATUS_0_GP_TIMER_EXP_SHFT
#define VICFIQ_STATUS_0__ADM_AARM___S  HWIO_VICFIQ_STATUS_0_ADM_AARM_SHFT
#define VICFIQ_STATUS_0__MDP___S HWIO_VICFIQ_STATUS_0_MDP_SHFT
#define VICFIQ_STATUS_0__UART1___S HWIO_VICFIQ_STATUS_0_UART_GROUP_IRQ_SHFT
/*
 * Daisy chain interrupts, define here so code is common.
 */
#define VIC_SIRC_DAISY_IRQ1 HWIO_VICFIQ_STATUS_0_SPSS_SIRC_IRQ0_SHFT
#define VIC_SIRC_DAISY_IRQ2 (32 + HWIO_VICFIQ_STATUS_1_SPSS_SIRC_IRQ1_SHFT)
#define VIC_GPIO_DAISY_SHADOW_IRQ2 (SIRC_BASE_IRQ + \
		HWIO_SPSS_SIRC_RAW_STATUS_GPIO_GROUP2_SC_SHDW_IRQ_SHFT)
#define VIC_GPIO_DAISY_SECURE_IRQ2 (SIRC_BASE_IRQ + \
		HWIO_SPSS_SIRC_RAW_STATUS_GPIO_GROUP2_SC_SECURE_IRQ_SHFT)
#define VIC_GPIO_DAISY_SHADOW_IRQ1 (SIRC_BASE_IRQ + \
		HWIO_SPSS_SIRC_RAW_STATUS_GPIO_GROUP_SC_SHDW_IRQ_SHFT)
#define VIC_GPIO_DAISY_SECURE_IRQ1 (SIRC_BASE_IRQ + \
		HWIO_SPSS_SIRC_RAW_STATUS_GPIO_GROUP_SC_SECURE_IRQ_SHFT)
#define IRQ_VIC_START 0
#define IRQ_VIC_SIZE  63
#define IRQ_VIC_REG_END 32
#define IRQ_VIC_END   63
#define IRQ_SIRC_SIZE   32
#define IRQ_SIRC_START (IRQ_VIC_END+1)
#define IRQ_SIRC_END   (IRQ_SIRC_START+IRQ_SIRC_SIZE)
#define IRQ_GPIO_SIZE  170
#define IRQ_GPIO_START (IRQ_SIRC_END+1)
#define IRQ_GPIO_END   (IRQ_GPIO_START+IRQ_GPIO_SIZE)
/*
 * General Purpose Timer
 */
#define AGPT_COUNT_VAL RAPTOR_A(AGPT_COUNT_VAL)
#define AGPT_MATCH_VAL RAPTOR_A(AGPT_MATCH_VAL)
#define AGPT_CLEAR     RAPTOR_A(AGPT_CLEAR)
#define AGPT_ENABLE    RAPTOR_A(AGPT_ENABLE)
#define AGPT_ENABLE__EN___M  HWIO_AGPT_ENABLE_EN_BMSK
#define AGPT_ENABLE__CLR_ON_MATCH_EN___M  HWIO_AGPT_ENABLE_CLR_ON_MATCH_EN_BMSK
/*
 * UART
#define UART1_BASE  HWIO_UART1_UART_MR1_ADDR
#define UART2_BASE  HWIO_UART2_UART_MR1_ADDR
 */
#define UART_CR     RAPTOR_A(UART1_UART_CR)
#define UART_MREG   RAPTOR_A(UART1_UART_MREG)
#define UART_NREG   RAPTOR_A(UART1_UART_NREG)
#define UART_DREG   RAPTOR_A(UART1_UART_DREG)
#define UART_MNDREG RAPTOR_A(UART1_UART_MNDREG)
#define UART_IPR    RAPTOR_A(UART1_UART_IPR)
#define UART_IMR    RAPTOR_A(UART1_UART_IMR)
#define UART_RFWR   RAPTOR_A(UART1_UART_RFWR)
#define UART_TFWR   RAPTOR_A(UART1_UART_TFWR)
#define UART_CSR    RAPTOR_A(UART1_UART_CSR)
#define UART_IRDA   RAPTOR_A(UART1_UART_IRDA)
#define UART_HCR    RAPTOR_A(UART1_UART_HCR)
#define UART_MR1    RAPTOR_A(UART1_UART_MR1)
#define UART_MR2    RAPTOR_A(UART1_UART_MR2)
#define UART_RF     RAPTOR_A(UART1_UART_RF)
#define UART_SR     RAPTOR_A(UART1_UART_SR)
#define UART_TF     RAPTOR_A(UART1_UART_TF)
#define UART_ISR    RAPTOR_A(UART1_UART_ISR)
#define ROW_RESET__UART1___M 0x10
#define ROW_RESET__UART2___M 0x4
#define ROW_RESET__UART3___M 0x2
#define GPIO1_PAGE   RAPTOR_A(GPIO1_PAGE)
#define GPIO1_CONFIG RAPTOR_A(GPIO1_CONFIG)
#define GPIO1_CFG    RAPTOR_A(GPIO1_CFG)
#define GPIO_OE_2   RAPTOR_A(GPIO_OE_2)
#define MCI_CLK__INVERT_IN___M 0 /*KAS*/
/*
 * GPIO Config
 */
#define GPIO_IN_1                      RAPTOR_A(GPIO_IN_1)
#define GPIO_OUT_1                     RAPTOR_A(GPIO_OUT_1)
#define GPIO_OE_1                      RAPTOR_A(GPIO_OE_1)
#define ROW_RESET 0xA8600218
/*
 * Ethernet
 */
/* RAPTOR2 SURF Configuration */
#define ETHER_START_PHY_ADDR 0x70000300
#define ETHER_END_PHY_ADDR   0x700003FF
#define ETHER_START_IRQ      252
#define ETHER_END_IRQ        252
#else
#define ETHER_START_PHY_ADDR 0x70000300
#define ETHER_END_PHY_ADDR   0x700003FF
#define ETHER_START_IRQ      252
#define ETHER_END_IRQ        252
/* #define ETHER_START_IRQ      (64+23) */
/* #define ETHER_END_IRQ        (64+23) */

/*===========================================================================
  Type Declarations
===========================================================================*/

/*===========================================================================
  Local Object Definitions
===========================================================================*/

/*===========================================================================
  Forward Declarations
===========================================================================*/

/*===========================================================================
				Function Definitions
===========================================================================*/

#endif

/* 	END */


