/* linux/include/asm-arm/arch-msm/board.h
 *
 * Copyright (C) 2007 Google, Inc.
 * Author: Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef __ASM_ARCH_MSM_BOARD_H
#define __ASM_ARCH_MSM_BOARD_H

#include <linux/types.h>
#include <linux/input.h>

/* platform device data structures */

struct mddi_panel_info;
struct msm_mddi_platform_data
{
	void (*mddi_client_power)(int on);
	void (*mddi_enable)(struct mddi_panel_info *panel, int on);
	void (*panel_power)(struct mddi_panel_info *panel, int on);
	unsigned has_vsync_irq:1;
	unsigned long fb_base;
	unsigned long fb_size;
};

#define PHY_TYPE_MASK		0xFF
#define PHY_MODEL_MASK		0xFF00
#define PHY_TYPE(x)		((x) & PHY_TYPE_MASK)
#define PHY_MODEL(x)		((x) & PHY_MODEL_MASK)

#define USB_PHY_MODEL_65NM	0x100
#define USB_PHY_MODEL_180NM	0x200
#define USB_PHY_UNDEFINED	0x00
#define USB_PHY_INTEGRATED	0x01
#define USB_PHY_EXTERNAL	0x02

struct msm_panel_common_pdata {
	int (*backlight_level)(int level);
};

struct lcdc_platform_data {
	int (*lcdc_gpio_config)(int on);
};

struct tvenc_platform_data {
	int (*pm_vid_en)(int on);
};

struct mddi_platform_data {
	void (*mddi_power_save)(int on);
};

struct function {
	char	*func_name;
	int	interface_num;
	int	num_interfaces;
};

struct usb_composition {
    /* USB device descriptor fields */
	__u16   vendor_id;
	__u16   product_id;
	char   	*product_name;
	char   	*serial_number;
	char   	*manufacturer_name;

	int     num_functions;
	struct  function funcs[10];
};

struct msm_hsusb_platform_data
{
	__u16   version;
	unsigned phy_info;
	int num_compositions;
	struct usb_composition *compositions;
};

struct android_pmem_platform_data
{
	const char* name;
	/* starting physical address of memory region */
	unsigned long start;
	/* size of memory region */
	unsigned long size;
	/* set to indicate the region should not be managed with an allocator */
	unsigned no_allocator;
	/* set to indicate maps of this region should be cached, if a mix of
	 * cached and uncached is desired, set this and open the device with
	 * O_SYNC to get an uncached region */
	unsigned cached;
	/* The MSM7k has bits to enable a write buffer in the bus controller*/
	unsigned buffered;
};

struct msm_camera_device_platform_data{
	int sensor_reset;
	int sensor_pwd;
	int vcm_pwd;
	const char *sensor_name;
};

/* common init routines for use by arch/arm/mach-msm/board-*.c */

void __init msm_add_devices(void);
void __init msm_map_common_io(void);
void __init msm_map_halibut_io(void);
void __init msm_map_qsd8x50_io(void);
void __init msm_init_irq(void);
void __init msm_init_gpio(void);

struct mmc_platform_data;
int __init msm_add_sdcc(unsigned int controller, struct mmc_platform_data *plat);

#if defined(CONFIG_USB_FUNCTION_MSM_HSUSB)
void msm_hsusb_set_vbus_state(int online);
#else
static inline void msm_hsusb_set_vbus_state(int online) {}
#endif

struct input_dev *msm_keypad_get_input_dev(void);
void config_camera_on_gpios(void);
void config_camera_off_gpios(void);

void __init msm_fb_register_device(char *name, void *pdata);
#endif
