/* Hw_netlink.c
 *
 * Author: <Mobile@huawei.com> *
 * Copyright (c) 2009 Huawei Technologies Co., Ltd
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
/*===========================================================================*/
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/list.h>
#include <linux/skbuff.h>
#include <linux/netlink.h>
#include <linux/moduleparam.h>
#include <linux/mutex.h>
#include <linux/device.h>
#include <linux/wait.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <net/sock.h>
#include <linux/delay.h>

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Netlink module for kernel<->userspace messge.");

#ifdef FEATURE_E5832
int syswatch_usb_state = 0;
EXPORT_SYMBOL(syswatch_usb_state);

#define SET_ONE 		'0'
#define SET_MANY 		'1'
#define REMOVE_ONE_PROCESS 	'2'
#define RECORD_THE_SEND 	'3'

static char curr_mode = 0;

static char * for_test_read_buff   = NULL;
static int for_test_read_buff_size = MONITORED_PROCESS_BUFF_LEN;
static int for_test_read_buff_curr_used = 0;

static char * event_record_buff   = NULL;
static int event_record_buff_size = MONITORED_PROCESS_BUFF_LEN;
static int event_record_buff_curr_used = 0;

static u32 us_pid = -1;

#define SYSWATCH_OK 			0
#define SYSWATCH_EORROR 		-1
#define SYSWATCH_NAME_TOO_LONG 		-2
#define SYSWATCH_MPQ_END 		-1

static void syswatch_show_monitored(void);

#define DELAY_FOR_NL_SEND msecs_to_jiffies(1)
#define NL_LEN_MAX    256
static struct delayed_work syswatch_nl_send_work;
static char nl_cmd = 0;
static char nl_name[NL_LEN_MAX] = {0};

static void syswatch_send_work(struct work_struct *w)
{
    struct sk_buff *skb;
    struct generic_nl_msg *body_msg;
    struct nlmsghdr *nlh;
    static int seq = 0;
    int size;

    if (!nl_name || (nl_name[0] == '\0'))
    {
        size = NLMSG_SPACE(sizeof(*body_msg));
    }
    else
    {
        size = NLMSG_SPACE(sizeof(*body_msg) + strlen(nl_name) + 1);
    }

    skb = alloc_skb(size, GFP_ATOMIC);
    if (!skb)
    {
        printk(KERN_ALERT "OOM when syswatch_nl_send");
        return;
    }

    nlh = NLMSG_PUT(skb, KERNEL_PID, seq++, NLMSG_DONE, size - sizeof(*nlh));
    body_msg = (struct generic_nl_msg *)NLMSG_DATA(nlh);

    memset(body_msg, 0, size - sizeof(*nlh));
    body_msg->comm = nl_cmd;

    if (nl_name && (nl_name[0] != '\0'))
    {
        body_msg->len = strlen(nl_name) + 1;
        strcpy(body_msg->data, nl_name);

        if (curr_mode == RECORD_THE_SEND)
        {
            if (event_record_buff_curr_used + body_msg->len < event_record_buff_size)
            {
                if (event_record_buff_curr_used > 0)
                {
                    event_record_buff[event_record_buff_curr_used - 1] = ' ';
                }

                strcpy(event_record_buff + event_record_buff_curr_used, nl_name);
                event_record_buff_curr_used += body_msg->len;
            }
        }
    }
    else
    {
        body_msg->len = 0;
    }

    NETLINK_CB(skb).dst_group = SYSWATCH_USERSPACE_GROUP;

nlmsg_failure:
    netlink_unicast(syswatch_nl_sock, skb, us_pid, 0);

}

int syswatch_nl_send(char *name, char comm)
{
    if((NULL == name ) || (strlen(name) >= NL_LEN_MAX))
    {
        nl_name[0] = '\0'; 
    }
    else
    {
        strcpy(nl_name, name);
    }
    nl_cmd = comm;
    
    schedule_delayed_work(&syswatch_nl_send_work, DELAY_FOR_NL_SEND);
    return 0;
}

int syswatch_nl_send_nodelay(char *name, char comm)
{
    struct sk_buff *skb = NULL;
    struct generic_nl_msg *body_msg = NULL;
    struct nlmsghdr *nlh = NULL;
    static int seq = 0;
    int size = 0;

    if (NULL == name || (name[0] == '\0'))
    {
        size = NLMSG_SPACE(sizeof(*body_msg));
    }
    else
    {
        size = NLMSG_SPACE(sizeof(*body_msg) + strlen(name) + 1);
    }

    skb = alloc_skb(size, GFP_ATOMIC);
    if (NULL == skb)
    {
        printk(KERN_ALERT "OOM when syswatch_nl_send");
        return -ENOMEM;
    }

    nlh = NLMSG_PUT(skb, KERNEL_PID, seq++, NLMSG_DONE, size - sizeof(*nlh));
    body_msg = (struct generic_nl_msg *)NLMSG_DATA(nlh);

    memset(body_msg, 0, size - sizeof(*nlh));
    body_msg->comm = comm;

    if (name && (name[0] != '\0'))
    {
        body_msg->len = strlen(name) + 1;
        strcpy(body_msg->data, name);

        if (curr_mode == RECORD_THE_SEND)
        {
            if (event_record_buff_curr_used + body_msg->len < event_record_buff_size)
            {
                if (event_record_buff_curr_used > 0)
                {
                    event_record_buff[event_record_buff_curr_used - 1] = ' ';
                }

                strcpy(event_record_buff + event_record_buff_curr_used, name);
                event_record_buff_curr_used += body_msg->len;
            }
        }
    }
    else
    {
        body_msg->len = 0;
    }

    NETLINK_CB(skb).dst_group = SYSWATCH_USERSPACE_GROUP;
nlmsg_failure:
    return netlink_unicast(syswatch_nl_sock, skb, us_pid, 0);
}
EXPORT_SYMBOL_GPL(syswatch_nl_send_nodelay);

EXPORT_SYMBOL_GPL(syswatch_nl_send);
static int  syswatch_init_mpq(void)
{
    int idx = 0;
    struct monitor_cell *monitored_processes = syswatch_mpq.pool;

    if (!syswatch_mpq.pool)
    {
        monitored_processes = (struct monitor_cell *)kmalloc(MONITORED_PROCESS_BUFF_LEN, GFP_ATOMIC);
        if (!monitored_processes)
        {
            printk(KERN_ALERT "OOM in syswatch_nl_input.\n");
            return -1;
        }

        syswatch_mpq.pool = monitored_processes;
    }

    memset(syswatch_mpq.pool, 0, MONITORED_PROCESS_BUFF_LEN);

    for (; idx < MAX_MONITORED_PROCESSES_NUM - 1; ++idx)
    {
        monitored_processes[idx].next_idx = idx + 1;
    }

    monitored_processes[idx].next_idx = SYSWATCH_MPQ_END;

    syswatch_mpq.first_used_idx = SYSWATCH_MPQ_END;
    syswatch_mpq.first_free_idx = 0;
    syswatch_mpq.curr_num = 0;

    return SYSWATCH_OK;
}

int syswatch_find_monitored_process(char *name, int len)
{
    int idx = SYSWATCH_MPQ_END;

    if (len >= ONE_PROCESS_NAME_LEN)
    {
        printk(KERN_ALERT "The length of monitored process cannot be longer than 16 bytes.\n"
               "Set monitored processes failed.\n");
        return SYSWATCH_NAME_TOO_LONG;
    }

    idx = syswatch_mpq.first_used_idx;

    while (idx != SYSWATCH_MPQ_END)
    {
        if (!strncmp(syswatch_mpq.pool[idx].comm, name, len))
        {
            char *str = syswatch_mpq.pool[idx].comm;

            if ((str[len] == ' ') || (str[len] == '\0'))
            {
                break;
            }
        }

        idx = syswatch_mpq.pool[idx].next_idx;
    }

    return idx;
}

EXPORT_SYMBOL_GPL(syswatch_find_monitored_process);

int syswatch_rm_monitored_process(char *name)
{
    int idx = -1;

    idx = syswatch_find_monitored_process(name, strlen(name));

    // find the target process in the moniroed processes
    if ((idx != SYSWATCH_MPQ_END) && (idx != SYSWATCH_NAME_TOO_LONG))
    {
        int i = syswatch_mpq.first_used_idx;

        if (idx == syswatch_mpq.first_used_idx)
        {
            syswatch_mpq.first_used_idx = syswatch_mpq.pool[idx].next_idx;
        }
        else
        {
            while (i != SYSWATCH_MPQ_END)
            {
                if (syswatch_mpq.pool[i].next_idx == idx)
                {
                    break;
                }

                i = syswatch_mpq.pool[i].next_idx;
            }

            syswatch_mpq.pool[i].next_idx = syswatch_mpq.pool[idx].next_idx;
        }

        memset(syswatch_mpq.pool[idx].comm, 0, ONE_PROCESS_NAME_LEN);

        // add this back to free index list
        syswatch_mpq.pool[idx].next_idx = syswatch_mpq.first_free_idx;
        syswatch_mpq.first_free_idx = idx;

        -- (syswatch_mpq.curr_num);

        syswatch_show_monitored();

        return SYSWATCH_OK;
    }

    return SYSWATCH_EORROR;
}

static int syswatch_set_monitored_processes(char *names)
{
    char *next = NULL, *pos = NULL;
    int ret   = SYSWATCH_EORROR;
    int index = SYSWATCH_MPQ_END;

    pos = names;

    if (!names || (names[0] == '\0'))
    {
        return ret;
    }

    if (!syswatch_mpq.pool)
    {
        ret = syswatch_init_mpq();
        if (ret)
        {
            return SYSWATCH_EORROR;
        }
    }

    // move forward all the ' '
    while (*pos == ' ')
    {
        ++pos;
    }

    while ((next = strchr(pos, ' ')))
    {
        if (syswatch_mpq.curr_num >= MAX_MONITORED_PROCESSES_NUM)
        {
            printk(KERN_ALERT "Cannot add any more monitored processes.\n");
            ret = -1;
            goto error;
        }

        // actually this check is also done in following syswatch_find_monitored_process
        // but here add some more debug information
        // so just keep it here
        if (next - pos >= ONE_PROCESS_NAME_LEN)
        {
            char *temp = kzalloc(next - pos + 1, GFP_KERNEL);
            if (!temp)
            {
                printk(KERN_ALERT "OOM in syswatch_set_monitored_processes.\n");
            }

            if (temp)
            {
                strncpy(temp, pos, next - pos);
            }

            printk(KERN_ALERT "NOTE:  \"%s\"  is longer than 16 bytes.\n"
                   "Set this monitored process failed.\n", temp ? temp : "");

            if (temp)
            {
                kfree(temp);
            }

            goto move_forward;
        }

        // skip the duplicated element
        ret = syswatch_find_monitored_process(pos, next - pos);
        if ((ret != SYSWATCH_MPQ_END) && (ret != SYSWATCH_NAME_TOO_LONG))
        {
            printk(KERN_ALERT "There is duplicated monitor process.\n");
            goto move_forward;
        }
        else
        {}

        // get the free cell index
        index = syswatch_mpq.first_free_idx;

        // adjust the next free index to the first one
        syswatch_mpq.first_free_idx = syswatch_mpq.pool[index].next_idx;

        // copy one process's name to the free cell
        strncpy(syswatch_mpq.pool[index].comm, pos, next - pos);
        // remember to add the ending NULL
        syswatch_mpq.pool[index].comm[next - pos] = '\0';

        // add this cell to the used list
        syswatch_mpq.pool[index].next_idx = syswatch_mpq.first_used_idx;
        syswatch_mpq.first_used_idx = index;
        ++ (syswatch_mpq.curr_num);
move_forward:

        // move forward all the ' '
        while (*next == ' ')
        {
            ++next;
        }

        pos = next;
    }

    // check whether there is trailing one
    if ((*pos != '\0') && (*pos != '\n'))
    {
        char last_one[ONE_PROCESS_NAME_LEN] = {0};
        int last_one_len = strlen(pos);

        if (last_one_len >= ONE_PROCESS_NAME_LEN)
        {
            printk(KERN_ALERT "The length of monitored process cannot be longer than 16 bytes.\n"
                   "Set monitored processes failed.\n");
            ret = SYSWATCH_EORROR;
            goto error;
        }

        if (syswatch_mpq.curr_num >= MAX_MONITORED_PROCESSES_NUM)
        {
            printk(KERN_ALERT "Cannot add any more monitored processes.\n");
            ret = SYSWATCH_EORROR;
            goto error;
        }

        strncpy(last_one, pos, last_one_len);

        // skip the duplicated element
        ret = syswatch_find_monitored_process(pos, strlen(last_one));
        if ((ret != SYSWATCH_MPQ_END) && (ret != SYSWATCH_NAME_TOO_LONG))
        {
            printk(KERN_ALERT "There is duplicated monitor process.\n");
            goto error;
        }

        index = syswatch_mpq.first_free_idx;
        syswatch_mpq.first_free_idx = syswatch_mpq.pool[index].next_idx;
        strcpy(syswatch_mpq.pool[index].comm, last_one);
        syswatch_mpq.pool[index].next_idx = syswatch_mpq.first_used_idx;
        syswatch_mpq.first_used_idx = index;
        ++ (syswatch_mpq.curr_num);
    }

    ret = SYSWATCH_OK;
error:
    return ret;
}

/*-------------------------------------------------
ԭͣ static int syswatch_add_monitored_process(char *name);
     		ضһҪؽ
    룺 		ҪصĽ
     		
  ֵ 	ӳɹ0
                򷵻-1
--------------------------------------------------*/
static int syswatch_add_monitored_process(char *name)
{
    int result = 0;
    int idx = SYSWATCH_MPQ_END;
    int len = 0;

    if (!name || (name[0] == '\0'))
    {
        return SYSWATCH_EORROR;
    }

    if (!syswatch_mpq.pool)
    {
        result = syswatch_init_mpq();
        if (result)
        {
            return SYSWATCH_EORROR;
        }
    }

    len = strlen(name);

    // skip the duplicated element
    result = syswatch_find_monitored_process(name, len);
    if ((result != SYSWATCH_MPQ_END) && (result != SYSWATCH_NAME_TOO_LONG))
    {
        printk(KERN_ALERT "This process is already set to monitored.\n");
        return SYSWATCH_EORROR;
    }
    else if (result == SYSWATCH_NAME_TOO_LONG)
    {
        return SYSWATCH_EORROR;
    }
    else
    {
        if (syswatch_mpq.curr_num >= MAX_MONITORED_PROCESSES_NUM)
        {
            printk(KERN_ALERT "Cannot add any more monitored processes.\n");
            return SYSWATCH_EORROR;
        }

        idx = syswatch_mpq.first_free_idx;
        syswatch_mpq.first_free_idx = syswatch_mpq.pool[idx].next_idx;

        strcpy(syswatch_mpq.pool[idx].comm, name);

        syswatch_mpq.pool[idx].next_idx = syswatch_mpq.first_used_idx;
        syswatch_mpq.first_used_idx = idx;
        ++ (syswatch_mpq.curr_num);
    }

    return SYSWATCH_OK;
}

static int syswatch_rm_all_monitored_processes(void)
{
    int ret = 0;

    if (syswatch_mpq.pool)
    {
        kfree(syswatch_mpq.pool);
    }

    syswatch_mpq.pool = NULL;
    syswatch_mpq.curr_num = 0;
    syswatch_mpq.first_used_idx = SYSWATCH_MPQ_END;
    syswatch_mpq.first_free_idx = SYSWATCH_MPQ_END;

    return ret;
}

static void syswatch_show_monitored(void)
{
    int idx = -1;
    char *buff = for_test_read_buff;


    memset(buff, 0, MONITORED_PROCESS_BUFF_LEN);
    for_test_read_buff_curr_used = 0;

    for (idx = syswatch_mpq.first_used_idx;
         idx != SYSWATCH_MPQ_END
       && (for_test_read_buff_curr_used + ONE_PROCESS_NAME_LEN) <= for_test_read_buff_size;
         idx = syswatch_mpq.pool[idx].next_idx)
    {
        int len = strlen(syswatch_mpq.pool[idx].comm);
        strcpy(buff, syswatch_mpq.pool[idx].comm);
        buff[len] = ' ';
        buff += len + 1;
        for_test_read_buff_curr_used += len + 1;
    }

    // because we always overwrite '\0' by ' ' and move forward; for the last one, we should move
    // backward and recover to '\0'
    if (syswatch_mpq.first_used_idx != SYSWATCH_MPQ_END)
    {
        *(buff - 1) = '\0';
    }

    return;
}

static void syswatch_nl_input(struct sk_buff *__skb)
{
    struct sk_buff *skb;
    struct nlmsghdr *nlh;
    struct generic_nl_msg *body_msg;

    skb = skb_get(__skb);
    if (skb->len >= NLMSG_SPACE(0))
    {
        nlh = nlmsg_hdr(skb);
        body_msg = NLMSG_DATA(nlh);

        // store the user space syswatch process id information
        us_pid = nlh->nlmsg_pid;

        switch (body_msg->comm)
        {
        case SET_MONITORED_PROCESSES:
            syswatch_set_monitored_processes(body_msg->data);

            syswatch_nl_inited = 1;

            if ((syswatch_usb_state & SYSWATCH_USB_CONNECTED) == SYSWATCH_USB_CONNECTED )
            {
                syswatch_nl_send(NULL, USB_UP);
            }
            
            if ((syswatch_usb_state & SYSWATCH_USB_DISCONNECTED) == SYSWATCH_USB_DISCONNECTED )
            {
                syswatch_nl_send(NULL, USB_DOWN);
            }
            
            if ((syswatch_usb_state & SYSWATCH_USB_PWR_RESET) == SYSWATCH_USB_PWR_RESET)
            {
                syswatch_nl_send(NULL, USB_PWR_RESET);
            }

            if ((syswatch_usb_state & SYSWATCH_SD_CARD_ATTACH) == SYSWATCH_SD_CARD_ATTACH)
            {
                syswatch_nl_send(NULL, SDCARD_ATTACH);
            }
            
            if ((syswatch_usb_state & SYSWATCH_SD_CARD_REMOVE) == SYSWATCH_SD_CARD_REMOVE)
            {
                syswatch_nl_send(NULL, SDCARD_REMOVE);
            }
            
            if ((syswatch_usb_state & SYSWATCH_EXTERNAL_CHARGER_ATTACH) == SYSWATCH_EXTERNAL_CHARGER_ATTACH)
            {
                syswatch_nl_send(NULL, EXTERNAL_CHARGER_ATTACH);
            }
            
            if ((syswatch_usb_state & SYSWATCH_EXTERNAL_CHARGER_REMOVE) == SYSWATCH_EXTERNAL_CHARGER_REMOVE)
            {
                syswatch_nl_send(NULL, EXTERNAL_CHARGER_REMOVE);
            }
            if ((syswatch_usb_state & SYSWATCH_NDIS_UP) == SYSWATCH_NDIS_UP)
            {
                syswatch_nl_send(NULL, NDIS_UP);
            }

            break;
        case ADD_MONITORED_PROCESS:
            syswatch_add_monitored_process(body_msg->data);
            break;
        case RM_MONITORED_PROCESS:
            syswatch_rm_monitored_process(body_msg->data);
            break;
        case RM_ALL_MONITORED_PROCESSES:
            syswatch_rm_all_monitored_processes();
            break;
        default:
            printk(KERN_ALERT "NOT implemented in syswatch_nl_input.\n");
            break;
        }
    }

    syswatch_show_monitored();

    return;
}

static int hw_netlink_open(struct inode *inode, struct file *file)
{
    return 0;
}

static ssize_t hw_netlink_write(struct file *file, const char __user *data, size_t len,
                                loff_t *ppos)
{
    int ret = -1;
    char *buff = NULL;
    char *kbuff = NULL;

    kbuff = buff = kzalloc(len, GFP_KERNEL);
    if (!buff)
    {
        printk(KERN_ALERT "OOM in hw_netlink_write.\n");
        return -1;
    }

    ret = copy_from_user(buff, data, len);
    if (ret)
    {
        printk(KERN_ALERT "copy_from_user error in hw_netlink_write.\n");
        kfree(kbuff);
        return ret;
    }

    while (*buff == ' ')
    {
        ++buff;
    }

    // replace the '\n' with '\0'
    // the '\n' is for 'echo' in script test
    // if not test by bash's echo
    // should change here
    buff[strlen(buff) - 1] = '\0';

    if (!strcmp(buff, "clear"))
    {
        // delete this
        printk(KERN_ALERT "we received 'clear'.\n");
        syswatch_rm_all_monitored_processes();
        syswatch_show_monitored();
        kfree(kbuff);
        return len;
    }

    if ((buff[0] == SET_MANY) && (buff[1] == ' '))
    {
        buff += 2;
        syswatch_set_monitored_processes(buff);
    }
    else if ((buff[0] == SET_ONE) && (buff[1] == ' '))
    {
        buff += 2;
        syswatch_add_monitored_process(buff);
    }
    else if ((buff[0] == RECORD_THE_SEND) && ((buff[1] == ' ') || (buff[1] == '\0')))
    {
        curr_mode = RECORD_THE_SEND;
        goto no_change;
    }
    else if ((buff[0] == REMOVE_ONE_PROCESS) && (buff[1] == ' '))
    {
        buff += 2;
        syswatch_rm_monitored_process(buff);
    }
    else
    {
        printk(KERN_ALERT "Wrong message format, if you want to set monitored processes.\n"
               "'0 set_one_monitored_process'\n"
               "'1 set_several_monitored_process'\n"
               "'2 remove_one_monitored_process'\n"
               "'3 start_record_the_syswatch_kernel_msg'\n"
        );

        goto no_change;
    }

    syswatch_show_monitored();
no_change:
    kfree(kbuff);
    return len;
}

// forced to read all in one time
// have checked that busybox defines macro "CONFIG_FEATURE_COPYBUF_KB"
// to be "4 KB", so it can read all the result in one time, if not , should do more here
static ssize_t hw_netlink_read(struct file *file, char __user *buf, size_t count,
                               loff_t *ppos)
{
    int ret = -1;
    char *buff = NULL;
    int size = 0;
    static int last_time_get_all = 0;

    if (count < MONITORED_PROCESS_BUFF_LEN)
    {
        printk(KERN_ALERT "Cannot read in one time.\n");
        return ret;
    }

    if (last_time_get_all)
    {
        last_time_get_all = 0;
        return 0;
    }

    buff = (curr_mode == RECORD_THE_SEND) ? event_record_buff : for_test_read_buff;
    size = (curr_mode == RECORD_THE_SEND) ? event_record_buff_curr_used :
           for_test_read_buff_curr_used;

    if (size)
    {
        if (size < MONITORED_PROCESS_BUFF_LEN)
        {
            buff[size - 1] = '\n';
            buff[size] = '\0';
        }

        ret = copy_to_user(buf, buff, size);
        if (ret)
        {
            printk(KERN_ALERT "hw_netlink_read copy_to_user failed.\n");
            return ret;
        }

        if (curr_mode == RECORD_THE_SEND)
        {
            curr_mode = 0;
            memset(event_record_buff, 0, event_record_buff_size);
            event_record_buff_curr_used = 0;
        }

        last_time_get_all = 1;
    }
    else
    {}

    return size;
}

static const struct file_operations hw_netlink_fops =
{
    .owner = THIS_MODULE,
    .open  = hw_netlink_open,
    .write = hw_netlink_write,
    .read  = hw_netlink_read,
};

static struct miscdevice hw_netlink_miscdev =
{
    .minor = MISC_DYNAMIC_MINOR,
    .name  = "nl_syswatch",
    .fops  = &hw_netlink_fops,
};

static int __devinit syswatch_nl_init(void)
{
    int ret;

    ret = misc_register(&hw_netlink_miscdev);
    if (ret)
    {
        printk(KERN_ALERT "syswatch misc_register failed.\n");
        return -1;
    }
    INIT_DELAYED_WORK(&syswatch_nl_send_work, syswatch_send_work);
	
    for_test_read_buff = kzalloc(for_test_read_buff_size, GFP_KERNEL);
    if (!for_test_read_buff)
    {
        printk(KERN_ALERT "syswatch_nl_init kmalloc failed.\n");
        misc_deregister(&hw_netlink_miscdev);
        return -1;
    }

    event_record_buff = kzalloc(event_record_buff_size, GFP_KERNEL);
    if (!event_record_buff)
    {
        printk(KERN_ALERT "syswatch_nl_init kmalloc failed.\n");
        misc_deregister(&hw_netlink_miscdev);
        kfree(for_test_read_buff);
        return -1;
    }

    syswatch_nl_sock = netlink_kernel_create(&init_net, NETLINK_SYSWATCH,
                                             32, syswatch_nl_input, NULL, THIS_MODULE);

    if (!syswatch_nl_sock)
    {
        printk(KERN_ALERT "kernel create netlink failed.\n");
        misc_deregister(&hw_netlink_miscdev);
        kfree(for_test_read_buff);
        kfree(event_record_buff);
        return -EIO;
    }

    syswatch_mpq.pool  = NULL;
    syswatch_nl_inited = 0;

    return 0;
}

static void __devexit syswatch_nl_fini(void)
{
    syswatch_nl_inited = 0;
    cancel_delayed_work_sync(&syswatch_nl_send_work);
    netlink_kernel_release(syswatch_nl_sock);

    misc_deregister(&hw_netlink_miscdev);
    kfree(event_record_buff);
    kfree(for_test_read_buff);
}

subsys_initcall(syswatch_nl_init);
module_exit(syswatch_nl_fini);
#endif
