/* linux/sound/soc/msm/msm7201.c
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 *
 * All source code in this file is licensed under the following license except
 * where indicated.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org.
 */

#include <linux/init.h>
#include <linux/err.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/time.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <sound/core.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>
#include <sound/pcm.h>
#include <sound/tlv.h>
#include <sound/initval.h>
#include <sound/control.h>
#include <asm/dma.h>
#include <linux/dma-mapping.h>

#include "msm-pcm.h"
#include <asm/mach-types.h>
#include <asm/arch/msm_rpcrouter.h>

static struct msm_rpc_endpoint *snd_ep;

struct msm_snd_rpc_ids {
	unsigned long   prog;
	unsigned long   vers;
	unsigned long   rpc_set_snd_device;
	int device;
};

static struct msm_snd_rpc_ids snd_rpc_ids;

static struct platform_device *msm_audio_snd_device;

struct snd_soc_codec_dai msm7201_dai = {
	.name = "ASOC",
	.playback = {
		.stream_name = "Playback",
		.channels_min = USE_CHANNELS_MIN,
		.channels_max = USE_CHANNELS_MAX,
		.rates = USE_RATE,
		.rate_min = USE_RATE_MIN,
		.rate_max = USE_RATE_MAX,
		.formats = USE_FORMATS,
	},
	.capture = {
		.stream_name = "Capture",
		.channels_min = USE_CHANNELS_MIN,
		.channels_max = USE_CHANNELS_MAX,
		.rate_min = USE_RATE_MIN,
		.rates = USE_RATE,
		.formats = USE_FORMATS,
	},
};

struct snd_soc_cpu_dai msm_cpudai = {
	.name = "MSM_7200A",
	.id = 0,
	.type = SND_SOC_DAI_PCM,
	.playback = {
		.channels_min = USE_CHANNELS_MIN,
		.channels_max = USE_CHANNELS_MAX,
		.rates = USE_RATE,
		.rate_min = USE_RATE_MIN,
		.rate_max = USE_RATE_MAX,
		.formats = USE_FORMATS,
	},
	.capture = {
		.channels_min = USE_CHANNELS_MIN,
		.channels_max = USE_CHANNELS_MAX,
		.rate_min = USE_RATE_MIN,
		.rates = USE_RATE,
		.formats = USE_FORMATS,
	},
};

static struct snd_soc_dai_link msm_dai = {
	.name = "ASOC",
	.stream_name = "ASOC",
	.codec_dai = &msm7201_dai,
	.cpu_dai = &msm_cpudai,
};

struct snd_soc_machine snd_soc_machine_msm = {
	.name 		= "msm-audio",
	.dai_link	= &msm_dai,
	.num_links = 1,
};

static int snd_msm_volume_info(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1; /* Volume Param, in dB */
	uinfo->value.integer.min = MIN_DB;
	uinfo->value.integer.max = MAX_DB;
	return 0;
}

static int snd_msm_volume_get(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_value *ucontrol)
{
	spin_lock_irq(&the_locks.mixer_lock);
	ucontrol->value.integer.value[0] = msm_vol_ctl.volume;
	spin_unlock_irq(&the_locks.mixer_lock);
	return 0;
}

static int snd_msm_volume_put(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_value *ucontrol)
{
	int change;
	int volume;

	volume = ucontrol->value.integer.value[0];
	spin_lock_irq(&the_locks.mixer_lock);
	change = (msm_vol_ctl.volume != volume);
	if (change) {
		msm_vol_ctl.update = 1;
		msm_vol_ctl.volume = volume;
	}
	spin_unlock_irq(&the_locks.mixer_lock);
	return change;
}

static int snd_msm_device_info(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1; /* Device */

	/*
	 * The number of devices supported is 26 (0 to 25)
	 */
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 25;
	return 0;
}

static int snd_msm_device_get(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_value *ucontrol)
{
	ucontrol->value.integer.value[0] = (uint32_t)snd_rpc_ids.device;
	return 0;
}

int msm_snd_init_rpc_ids(void)
{
	snd_rpc_ids.prog	= 0x30000002;
	snd_rpc_ids.vers	= 0x00010001;
	/*
	 * The magic number 2 corresponds to the rpc call
	 * index for snd_set_device
	 */
	snd_rpc_ids.rpc_set_snd_device = 2;
	return 0;
}

int msm_snd_rpc_connect(void)
{
	if (snd_ep) {
		printk(KERN_INFO "%s: snd_ep already connected\n", __func__);
		return 0;
	}

	/* Initialize rpc ids */
	if (msm_snd_init_rpc_ids()) {
		printk(KERN_ERR "%s: snd rpc ids initialization failed\n"
			, __func__);
		return -ENODATA;
	}

	snd_ep = msm_rpc_connect_compatible(snd_rpc_ids.prog,
				snd_rpc_ids.vers, 0);
	if (IS_ERR(snd_ep)) {
		printk(KERN_ERR "%s: failed (compatible VERS = %ld)\n",
				__func__, snd_rpc_ids.vers);
		snd_ep = NULL;
		return -EAGAIN;
	}
	return 0;
}

int msm_snd_rpc_close(void)
{
	int rc = 0;

	if (IS_ERR(snd_ep)) {
		printk(KERN_ERR "%s: snd handle unavailable, rc = %ld\n",
				__func__, PTR_ERR(snd_ep));
		return -EAGAIN;
	}

	rc = msm_rpc_close(snd_ep);
	snd_ep = NULL;

	if (rc < 0) {
		printk(KERN_ERR "%s: close rpc failed! rc = %d\n",
				__func__, rc);
		return -EAGAIN;
	} else
		printk(KERN_INFO "rpc close success\n");

	return rc;
}

static int snd_msm_device_put(struct snd_kcontrol *kcontrol,
			struct snd_ctl_elem_value *ucontrol)
{
	int rc = 0;
	struct snd_start_req {
		struct rpc_request_hdr hdr;
		uint32_t rpc_snd_device;
		uint32_t snd_mute_ear_mute;
		uint32_t snd_mute_mic_mute;
		uint32_t callback_ptr;
		uint32_t client_data;
	} req;

	struct snd_start_rep {
		struct rpc_reply_hdr hdr;
		uint32_t err_flag;
	} rep;

	snd_rpc_ids.device = (int)ucontrol->value.integer.value[0];
	req.hdr.type = 0;
	req.hdr.rpc_vers = 2;

	req.rpc_snd_device = be32_to_cpu(snd_rpc_ids.device);
	req.snd_mute_ear_mute = be32_to_cpu(1);
	req.snd_mute_mic_mute = be32_to_cpu(0);
	req.client_data = be32_to_cpu(0);

	rc = msm_snd_rpc_connect();
	if (rc < 0)
		return rc;

	req.hdr.prog = snd_rpc_ids.prog;
	req.hdr.vers = snd_rpc_ids.vers;

	rc = msm_rpc_call_reply(snd_ep, snd_rpc_ids.rpc_set_snd_device ,
			&req, sizeof(req), &rep, sizeof(rep), 100 * HZ);

	if (rc < 0) {
		printk(KERN_ERR "%s: snd rpc call failed! rc = %d\n",
			__func__, rc);
	} else
		printk(KERN_INFO "snd device connected \n");

	rc = msm_snd_rpc_close();

	return rc;
}

/* Supported range -50dB to 18dB */
static const DECLARE_TLV_DB_LINEAR(db_scale_linear, -5000, 1800);

#define MSM_EXT(xname, xindex, fp_info, fp_get, fp_put, addr) \
{ .iface = SNDRV_CTL_ELEM_IFACE_MIXER, \
  .access = SNDRV_CTL_ELEM_ACCESS_READWRITE, \
  .name = xname, .index = xindex, \
  .info = fp_info,\
  .get = fp_get, .put = fp_put, \
  .private_value = addr, \
}

#define MSM_EXT_TLV(xname, xindex, fp_info, fp_get, fp_put, addr, tlv_array) \
{ .iface = SNDRV_CTL_ELEM_IFACE_MIXER, \
  .access = (SNDRV_CTL_ELEM_ACCESS_TLV_READ | \
		SNDRV_CTL_ELEM_ACCESS_READWRITE), \
  .name = xname, .index = xindex, \
  .info = fp_info,\
  .get = fp_get, .put = fp_put, .tlv.p = tlv_array, \
  .private_value = addr, \
}

static struct snd_kcontrol_new snd_msm_controls[] = {
	MSM_EXT_TLV("PCM Playback Volume", 0, snd_msm_volume_info, \
	snd_msm_volume_get, snd_msm_volume_put, 0, db_scale_linear),
	MSM_EXT("device", 1, snd_msm_device_info, snd_msm_device_get, \
						 snd_msm_device_put, 0),
};

static int msm_new_mixer(struct snd_card *card)
{
	unsigned int idx;
	int err;

	strcpy(card->mixername, "MSM Mixer");
	for (idx = 0; idx < ARRAY_SIZE(snd_msm_controls); idx++) {
		err = snd_ctl_add(card,
				snd_ctl_new1(&snd_msm_controls[idx], NULL));
		if (err < 0)
			return err;
	}
	return 0;
}


static int __devinit msm_pcm_probe(struct platform_device *devptr)
{
	struct snd_card *card;
	struct snd_soc_codec *codec;
	int ret;

	struct snd_soc_device *socdev = platform_get_drvdata(devptr);

	codec = kzalloc(sizeof(struct snd_soc_codec), GFP_KERNEL);
	if (codec == NULL)
		return -ENOMEM;

	codec->name = "MSM-CARD";
	codec->owner = THIS_MODULE;
	socdev->codec = codec;
	mutex_init(&codec->mutex);

	INIT_LIST_HEAD(&codec->dapm_widgets);
	INIT_LIST_HEAD(&codec->dapm_paths);

	/* register pcms */
	ret = snd_soc_new_pcms(socdev, SNDRV_DEFAULT_IDX1, SNDRV_DEFAULT_STR1);
	if (ret < 0) {
		printk(KERN_ERR "msm_soc: failed to create pcms\n");
		goto __nopcm;
	}

	card = socdev->codec->card;
	ret = msm_new_mixer(card);
	if (ret < 0) {
		printk(KERN_ERR "msm_soc:ALSA MSM Mixer Fail");
		goto __nodev;
	}

	ret = snd_soc_register_card(socdev);
	if (ret < 0) {
		printk(KERN_ERR "msm_soc: failed to register card\n");
		goto __nodev;
	}

	return 0;

__nodev:
	snd_soc_free_pcms(socdev);
__nopcm:
	kfree(codec);
	return ret;
}


static int __devexit msm_pcm_remove(struct platform_device *devptr)
{
	struct snd_soc_device *socdev = platform_get_drvdata(devptr);
	snd_soc_free_pcms(socdev);
	kfree(socdev->codec);
	platform_set_drvdata(devptr, NULL);
	return 0;
}


static int pcm_preallocate_dma_buffer(struct snd_pcm *pcm,
	int stream)
{
	struct snd_pcm_substream *substream = pcm->streams[stream].substream;
	struct snd_dma_buffer *buf = &substream->dma_buffer;
	size_t size;
	if (!stream)
		size = PLAYBACK_DMASZ;
	else
		size = CAPTURE_DMASZ;

	buf->dev.type = SNDRV_DMA_TYPE_DEV;
	buf->dev.dev = pcm->card->dev;
	buf->private_data = NULL;
	buf->area = dma_alloc_coherent(pcm->card->dev, size,
					   &buf->addr, GFP_KERNEL);
	if (!buf->area)
		return -ENOMEM;

	buf->bytes = size;
	return 0;
}

static void msm_pcm_free_dma_buffers(struct snd_pcm *pcm)
{
	struct snd_pcm_substream *substream;
	struct snd_dma_buffer *buf;
	int stream;

	for (stream = 0; stream < 2; stream++) {
		substream = pcm->streams[stream].substream;
		if (!substream)
			continue;

		buf = &substream->dma_buffer;
		if (!buf->area)
			continue;

		dma_free_coherent(pcm->card->dev, buf->bytes,
				      buf->area, buf->addr);
		buf->area = NULL;
	}
}
static int msm_pcm_new(struct snd_card *card,
			struct snd_soc_codec_dai *codec_dai,
			struct snd_pcm *pcm)
{
	int ret;
	if (!card->dev->coherent_dma_mask)
		card->dev->coherent_dma_mask = DMA_32BIT_MASK;

	if (codec_dai->playback.channels_min) {
		ret = pcm_preallocate_dma_buffer(pcm,
			SNDRV_PCM_STREAM_PLAYBACK);
		if (ret)
			return ret;
	}

	if (codec_dai->capture.channels_min) {
		ret = pcm_preallocate_dma_buffer(pcm,
			SNDRV_PCM_STREAM_CAPTURE);
		if (ret)
			msm_pcm_free_dma_buffers(pcm);
	}
	return ret;
}

struct snd_soc_platform msm_soc_platform = {
	.name		= "msm-audio",
	.probe          = msm_pcm_probe,
	.remove         = __devexit_p(msm_pcm_remove),
	.pcm_ops 	= &msm_pcm_ops,
	.pcm_new	= msm_pcm_new,
	.pcm_free	= msm_pcm_free_dma_buffers,
};

/* msm_audio audio subsystem */
static struct snd_soc_device msm_audio_snd_devdata = {
	.machine = &snd_soc_machine_msm,
	.platform = &msm_soc_platform,
};


static int __init msm_audio_init(void)
{
	int ret;
	struct snd_soc_codec_device *msm_soc_codec_device;

	msm_audio_snd_device = platform_device_alloc("soc-audio", -1);
	if (!msm_audio_snd_device)
		return -ENOMEM;
	msm_soc_codec_device = kzalloc(sizeof(struct snd_soc_codec_device),
								GFP_KERNEL);
	if (!msm_soc_codec_device) {
		platform_device_put(msm_audio_snd_device);
		return -ENOMEM;
	}
	msm_audio_snd_devdata.codec_dev = msm_soc_codec_device;

	platform_set_drvdata(msm_audio_snd_device, &msm_audio_snd_devdata);
	msm_audio_snd_devdata.dev = &msm_audio_snd_device->dev;
	ret = platform_device_add(msm_audio_snd_device);
	if (ret) {
		platform_device_put(msm_audio_snd_device);
		return ret;
	}
	mutex_init(&the_locks.lock);
	mutex_init(&the_locks.write_lock);
	mutex_init(&the_locks.read_lock);
	spin_lock_init(&the_locks.read_dsp_lock);
	spin_lock_init(&the_locks.write_dsp_lock);
	spin_lock_init(&the_locks.mixer_lock);
	init_waitqueue_head(&the_locks.write_wait);
	init_waitqueue_head(&the_locks.read_wait);
	msm_vol_ctl.volume = MSM_PLAYBACK_DEFAULT_VOLUME;
	msm_vol_ctl.pan = MSM_PLAYBACK_DEFAULT_PAN;

	return ret;
}

static void __exit msm_audio_exit(void)
{
	kfree(msm_audio_snd_devdata.codec_dev);
	platform_device_unregister(msm_audio_snd_device);
}

module_init(msm_audio_init);
module_exit(msm_audio_exit);

MODULE_AUTHOR("QUALCOMM-QCT");
MODULE_DESCRIPTION("PCM module");
MODULE_LICENSE("GPL v2");
