/* linux/sound/soc/msm/qsd8k.c
 *
 * Copyright (c) 2008 QUALCOMM USA, INC.
 *
 * All source code in this file is licensed under the following license except
 * where indicated.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can find it at http://www.fsf.org.
 */

#include <linux/init.h>
#include <linux/err.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/time.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <sound/core.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>
#include <sound/pcm.h>
#include <sound/initval.h>
#include <sound/control.h>
#include <asm/dma.h>
#include <linux/dma-mapping.h>

#include "qsd-pcm.h"

static struct platform_device *qsd_audio_snd_device;

struct snd_soc_codec_dai qsd8k_dai = {
	.name = "ASOC",
	.playback = {
		.stream_name 	= "Playback",
		.channels_min 	= USE_CHANNELS_MIN,
		.channels_max 	= USE_CHANNELS_MAX,
		.rates 		= USE_RATE,
		.rate_min 	= USE_RATE_MIN,
		.rate_max 	= USE_RATE_MAX,
		.formats 	= USE_FORMATS,
	},
	.capture = {
		.stream_name 	= "Capture",
		.channels_min 	= USE_CHANNELS_MIN,
		.channels_max 	= USE_CHANNELS_MAX,
		.rate_min 	= USE_RATE_MIN,
		.rates 		= USE_RATE,
		.formats 	= USE_FORMATS,
	},
};

struct snd_soc_cpu_dai qsd_cpudai = {
	.name = "QSD_8650",
	.id = 0,
	.type = SND_SOC_DAI_PCM,
	.playback = {
		.channels_min 	= USE_CHANNELS_MIN,
		.channels_max 	= USE_CHANNELS_MAX,
		.rates 		= USE_RATE,
		.rate_min 	= USE_RATE_MIN,
		.rate_max 	= USE_RATE_MAX,
		.formats 	= USE_FORMATS,
	},
	.capture = {
		.channels_min 	= USE_CHANNELS_MIN,
		.channels_max 	= USE_CHANNELS_MAX,
		.rate_min	= USE_RATE_MIN,
		.rates 		= USE_RATE,
		.formats 	= USE_FORMATS,
	},
};


static struct snd_soc_dai_link qsd_dai = {
	.name 		= "ASOC",
	.stream_name 	= "ASOC",
	.codec_dai 	= &qsd8k_dai,
	.cpu_dai 	= &qsd_cpudai,
};


struct snd_soc_machine snd_soc_machine_qsd = {
	.name 		= "qsd-audio",
	.dai_link	= &qsd_dai,
	.num_links 	= 1,
};

static int __devinit qsd_pcm_probe(struct platform_device *devptr)
{
	struct snd_card *card;
	struct snd_soc_codec *codec;
	int ret;

	struct snd_soc_device *socdev = platform_get_drvdata(devptr);

	codec = kzalloc(sizeof(struct snd_soc_codec), GFP_KERNEL);
	if (codec == NULL)
		return -ENOMEM;

	codec->name = "MSM-CARD";
	codec->owner = THIS_MODULE;
	socdev->codec = codec;
	mutex_init(&codec->mutex);

	INIT_LIST_HEAD(&codec->dapm_widgets);
	INIT_LIST_HEAD(&codec->dapm_paths);

	/* register pcms */
	ret = snd_soc_new_pcms(socdev, SNDRV_DEFAULT_IDX1, SNDRV_DEFAULT_STR1);
	if (ret < 0) {
		printk(KERN_ERR "qsd_soc: failed to create pcms\n");
		goto __nopcm;
	}

	card = socdev->codec->card;

	ret = snd_soc_register_card(socdev);
	if (ret < 0) {
		printk(KERN_ERR "qsd_soc: failed to register card\n");
		goto __nodev;
	}

	return 0;

__nodev:
	snd_soc_free_pcms(socdev);
__nopcm:
	kfree(codec);
	return ret;
}

static int __devexit qsd_pcm_remove(struct platform_device *devptr)
{
	struct snd_soc_device *socdev = platform_get_drvdata(devptr);
	snd_soc_free_pcms(socdev);
	kfree(socdev->codec);
	platform_set_drvdata(devptr, NULL);
	return 0;
}

static int qsd_pcm_new(struct snd_card *card,
			struct snd_soc_codec_dai *codec_dai,
			struct snd_pcm *pcm)
{
	if (!card->dev->coherent_dma_mask)
		card->dev->coherent_dma_mask = DMA_32BIT_MASK;

	return 0;
}

struct snd_soc_platform qsd_soc_platform = {
	.name		= "qsd-audio",
	.probe          = qsd_pcm_probe,
	.remove         = __devexit_p(qsd_pcm_remove),
	.pcm_ops 	= &qsd_pcm_ops,
	.pcm_new	= qsd_pcm_new,
};

/* qsd_audio audio subsystem */
static struct snd_soc_device qsd_audio_snd_devdata = {
	.machine = &snd_soc_machine_qsd,
	.platform = &qsd_soc_platform,
};

static int __init qsd_audio_init(void)
{
	int ret;
	struct snd_soc_codec_device *qsd_soc_codec_device;

	qsd_audio_snd_device = platform_device_alloc("soc-audio", -1);
	if (!qsd_audio_snd_device)
		return -ENOMEM;
	qsd_soc_codec_device = kzalloc(sizeof(struct snd_soc_codec_device),
								GFP_KERNEL);
	if (!qsd_soc_codec_device) {
		platform_device_put(qsd_audio_snd_device);
		return -ENOMEM;
	}
	qsd_audio_snd_devdata.codec_dev = qsd_soc_codec_device;

	platform_set_drvdata(qsd_audio_snd_device, &qsd_audio_snd_devdata);
	qsd_audio_snd_devdata.dev = &qsd_audio_snd_device->dev;

	ret = platform_device_add(qsd_audio_snd_device);
	if (ret) {
		platform_device_put(qsd_audio_snd_device);
		return ret;
	}
	mutex_init(&the_locks.lock);

	return ret;
}

static void __exit qsd_audio_exit(void)
{
	kfree(qsd_audio_snd_devdata.codec_dev);
	platform_device_unregister(qsd_audio_snd_device);
}

module_init(qsd_audio_init);
module_exit(qsd_audio_exit);

MODULE_AUTHOR("QUALCOMM-QCT");
MODULE_DESCRIPTION("PCM module");
MODULE_LICENSE("GPL v2");
