/*******************************************************************************

 @file tests/unit/libs/stringl/src/test_stringl.c
 @brief stringl test code.

 ------------------------------------------------------------------------------
 Copyright (c) 2007 QUALCOMM Incorporated.
 All Rights Reserved. QUALCOMM Proprietary and Confidential.
 ------------------------------------------------------------------------------
******************************************************************************/


#include "test_stringl.h"

/* Test the Nominal Unit test
   a. Copy Data in Src to Dst variables.
   b. Concatinate Data  in Src to Dst variable.
*/
int test_nominal_local() {
    char dst[DST_SIZE]="";
    
    size_t OP_SIZE = 0;
    size_t OLD_SIZE = 0;

    TPassed = 0; TFailed = 0;
    OP_SIZE = strlcpy(dst, SRC, DST_SIZE);
    if (OP_SIZE == strlen(SRC))
        TPassed++;
    else 
        TFailed++;
    
    OLD_SIZE = strlen(dst);
    OP_SIZE = strlcat(dst, SRC, DST_SIZE);
    if (OP_SIZE == (OLD_SIZE + strlen(SRC)))
        TPassed++;
    else
        TFailed++;
    
    return (TPassed - TFailed);
}

/* Test the Nominal Unit test 
   a. Copy Data in SRC to pSrc.
   b. Concatinate data in pSrc with pDst.
   c. Copy Data in SRC to pSrc with SIZ set to greater than the length of pDst.
   d. Concatinate Data in pSrc to pDst with SIZ set to greater than lenght of
      pDst.
*/
int test_nominal_dynamic() {
    char *pSrc = (char *) malloc(SRC_SIZE);
    char *pDst = (char *) malloc(DST_SIZE);
    
    size_t OP_SIZE = 0;

    TPassed = 0; TFailed = 0;
    
    memset(pSrc, 0, SRC_SIZE);
    memset(pDst, 0, DST_SIZE);

    OP_SIZE = strlcpy(pSrc, SRC, SRC_SIZE);
    if(OP_SIZE == strlen(SRC))
        TPassed++;
    else
        TFailed++;        

    OP_SIZE = strlcat(pDst, pSrc, DST_SIZE);
    if (OP_SIZE == strlen(pSrc))
        TPassed++;
    else
        TFailed++;        

    OP_SIZE = strlcpy(pSrc, SRC, SRC_SIZE+1);
    if(OP_SIZE == strlen(SRC))
        TPassed++;
    else
        TFailed++;        

    OP_SIZE = strlcat(pDst, pSrc, SRC_SIZE+1);
    if (OP_SIZE == strlen(pDst))
        TPassed++;
    else
        TFailed++;        

    return (TPassed - TFailed);
}

/* Adverse unit test
   a. Copy 0 Data in src to dst variables.
   b. Concatinate 0 char data in src to dst variable.
   c. Copy Data in SRC to src variable with SIZ set to greater than
   length of SRC.
   d. Concatinate data in src to dst variable with SIZ set to greater
   than length of src.
   e. Copy data to dst with SIZ=0
   f. Concatinate data to dst with SIZ=0
*/
int test_adverse(){
    char src[SRC_SIZE]="";
    char dst[DST_SIZE]="";

    TPassed = 0; TFailed = 0;

    size_t OP_SIZE = 0;
    size_t OLD_SIZE = 0;

    OP_SIZE = strlcpy(dst, src, DST_SIZE);
    if(OP_SIZE == 0)
        TPassed++;
    else { 
        printf("Failed 1\n");
        TFailed++;
    }
    
    OP_SIZE = strlcat(dst, src, DST_SIZE);
    if(OP_SIZE == 0)
        TPassed++;
    else { 
        printf("Failed 2\n");
        TFailed++;
    }

    OP_SIZE = strlcpy(src, SRC, SRC_SIZE+1);
    if(OP_SIZE == strlen(SRC))
        TPassed++;
    else { 
        printf("Failed 3\n");
        TFailed++;        
    }

    OLD_SIZE = strlen(dst);
    OP_SIZE = strlcat(dst, src, DST_SIZE+1);
    if(OP_SIZE == (OLD_SIZE + strlen(src)))
        TPassed++;
    else { 
        printf("Failed 4\n");
        TFailed++;
    }
   
    OLD_SIZE = strlen(src);
    OP_SIZE = strlcpy(src, SRC, 0);
    if(OP_SIZE == OLD_SIZE)
        TPassed++;
    else { 
        printf("Failed 5\n");
        TFailed++;
    }

    OLD_SIZE = strlen(dst);
    OP_SIZE = strlcat(dst, src, 0);
    if(OP_SIZE == OLD_SIZE)
        TPassed++;
    else { 
        printf("Failed 6\n");
        TFailed++;
    }

    while((OP_SIZE = strlcat(dst, src, DST_SIZE)) <= DST_SIZE);
    if(OP_SIZE != DST_SIZE-1)
        TPassed++;
    else { 
        printf("Failed 7\n");
        TFailed++;
    }

    return (TPassed - TFailed);
}

int main(void) {
    int RetVal = 0;
    test_nominal_local();
    printf("Nominal tests set 1: %d passed\n", TPassed);
    printf("Nominal tests set 1: %d falied\n", TFailed);
    RetVal = TFailed;
    
    test_nominal_dynamic(); 
    printf("Nominal test set 2:  %d passed\n", TPassed);
    printf("Nominal test set 2:  %d failed\n", TFailed);
    RetVal += TFailed;
        
    test_adverse(); 
    printf("Adverse tests: %d passed\n", TPassed);
    printf("Adverse tests: %d failed\n", TFailed);
    RetVal += TFailed;
    
    return (RetVal == 0) ? 0 : -(RetVal);
}
