#!/bin/sh
################################################################################
# @file  copy_counters.sh
# @brief Script to copy the gcov data to given directory
#
# ------------------------------------------------------------------------------
# Copyright (c) 2007 QUALCOMM Incorporated.
# All Rights Reserved. QUALCOMM Proprietary and Confidential.
# ------------------------------------------------------------------------------
################################################################################

#default gcov data directory
gcov_data_path=""
gcov_dest_path=""
olddir=`pwd`

print_usage() {
	echo "Usage: " $0 " params..."
	echo "Params:"
	echo -e "\t--data-path=<dir> Optional. Source path to GCOV data. Defaults to /proc/gcov"
	echo -e "\t--dest-path=<dir> Destination path where GCOV data to be copied"
	echo -e "\t-h\t\tPrint this help and exit\n"
#	echo "Note: Destination path must have similiar directory structore as gcov data source path.  Typically it should be the directory where the source code is compiled with GCOV compiler options"
}

n=$#

if [ $n -gt 0 ] && [ $n -lt 3 ]
then
	for i in $*
	do
		case $i in
			--data-path=*)
				#echo $i
				gcov_data_path=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
				;;
			--dest-path=*)
				#echo $i
				gcov_dest_path=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
				;;
			-h)
				print_usage
				exit 0
				;;
			*)
				#unknown 
				echo "Unknown parameter(s) passed"
				print_usage
				exit 1
				;;
		esac 
	done
fi	
if [ $n -eq 0 ]
then
   echo "Please input destination path where GCOV data to be copied"
   print_usage
   exit 1
fi

if [ -z $gcov_dest_path ]
then
	echo "Please input destination path where GCOV data to be copied"
	print_usage
	exit 1
fi

if [ ! -d $gcov_dest_path ]
then
	echo $gcov_dest_path ": No such file or directory exists"
	exit 1
fi

if [ ! -d $gcov_data_path ]
then
	echo $gcov_data_path ": No such file or directory exists"
	exit 1
fi

if [ -z $gcov_data_path ]
then
	gcov_data_path=/proc/gcov/
	myTarFile=$gcov_dest_path/`echo $gcov_data_path|sed 's|/$||'|sed 's|.*/||'`.tar
	echo "Taking default GCOV data path" $gcov_data_path
	echo "Tarring all GCOV data files from $gcov_data_path to $myTarFile"
	cd $gcov_data_path
	myCmd="tar -cf $myTarFile *"
	echo $myCmd
	eval $myCmd
elif [ -d $gcov_data_path ]
then
	myTarFile=$gcov_dest_path/`echo $gcov_data_path|sed 's|/$||'|sed 's|.*/||'`.tar
	echo "Tarring GCOV data files from $gcov_data_path to $myTarFile"
	cd $gcov_data_path
	myCmd="tar -cf $myTarFile *"
	echo $myCmd
	eval $myCmd
fi

cd $olddir
echo "done"
exit 0

