#!/bin/sh
################################################################################
# @file  zero_counters.sh
# @brief Script to reset the gcov data
#
# ------------------------------------------------------------------------------
# Copyright (c) 2007 QUALCOMM Incorporated.
# All Rights Reserved. QUALCOMM Proprietary and Confidential.
# ------------------------------------------------------------------------------
################################################################################

#default gcov data directory
gcov_data_path=""

print_usage() {
	echo "Usage: " $0 " [options]"
	echo "Options:"
	echo -e "\t-path=<dir/file> Optional. Path to GCOV data. Defaults to /proc/gcov"
	echo -e "\t-h\t\t Print this help and exit"
	echo -e "\n"
}

n=$#

if [ $n -gt 0 ] && [ $n -lt 3 ]
then
	#echo $n;
	for i in $*
	do
		case $i in
			-path=*)
				#echo $i
				gcov_data_path=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
				;;
			-h)
				print_usage
				exit 0
				;;
			*)
				#unknown 
				echo "Unknown parameter(s) passed"
				print_usage
				exit 1
				;;
		esac 
	done
else 
if [ $n -eq 0 ]
then	
	gcov_data_path=/proc/gcov
	echo "Taking default GCOV data path" $gcov_data_path
else
	echo "Unknown parameter(s) passed"
	print_usage
	exit 1
fi
fi

if [ -z $gcov_data_path ]
then
   echo "Please input path to GCOV data"
   print_usage
   exit 1
fi

if [ -f $gcov_data_path ]
then
  echo "Resetting the GCOV data file" $gcov_data_path"..."
  echo 0 > $gcov_data_path
  echo "done"
  exit 0
fi


if [ ! -d $gcov_data_path ]
then
   echo $gcov_data_path ": No such file or directory exists"
   exit 1
fi

olddir=`pwd`
cd $gcov_data_path

#echo $olddir

echo "Resetting GCOV data files under" $gcov_data_path "..."

for F in `find . -name "*.gcda"`;
do 
	#echo "echo 0 >" $gcov_data_path/$F
	echo 0 > $F;
done

cd $olddir

echo "done"
exit 0

