# Script to test the gpio driver.

failed=0
cd /debug/gpio/ 2>/dev/null
if [ $? -ne 0 ]; then
       echo "Unable to find /debug/gpio/"
       echo "Debug fs support required to proceed with tests."
       exit 1;
fi


if [ $# -eq 3 ]; then
	valid_gpio=$1
	invalid_gpio=$2
	nonremoted_gpio=$3
elif [ $1 -eq 0 ]; then
	valid_gpio=11
	invalid_gpio=211
	nonremoted_gpio=16
else
	echo "GPIO_TLMM Test App"
	echo "===================="
	echo "To execute:"
	echo " ./gpio_tlmm.sh [valid GPIO] [invalid GPIO] [non-Remoted GPIO]"
	echo " The GPIOs are optional. Pass 0 to use default values [11,211,16]"
	echo " instead."
	echo "  Example:"
	echo "  ./gpio_tlmm.sh           - Will display this message"
	echo "  ./gpio_tlmm.sh 0         - Will run with 11,211 and 16 as"
	echo "                             valid, invalid and non-remoted GPIOs"
	echo "  ./gpio_tlmm.sh 10 210 16 - Will run with 10,210 and 16 as"
	echo "                             valid, invalid and non-remoted GPIOs"
	exit 1;
fi

func=0
output=1
pull=0
drvstr=0

pack_gpio(){
	raw_config=$(( ((($1 & 1023)<<4)|\
						 ($2 & 15)|\
						 (($3 & 1) << 14)|\
						 (($4 & 3) << 15)|\
						 (($5 & 15) << 17)) ))
	echo $raw_config
}
echo "Testing GPIO TLMM"
echo "================="

#############
# valid gpio
#############
raw_config=`pack_gpio $valid_gpio $func $output $pull $drvstr`
status=`cat enable`
echo "$raw_config" > enable
status=`cat enable`
if [ $status -ne 0 ]; then
   echo "FAILED: Enabling Valid GPIO"
   failed=$(($failed + 1))
fi
status=`cat disable`
echo "$raw_config" > disable
status=`cat disable`
if [ $status -ne 0 ]; then
	echo "FAILED: Disabling Valid GPIO"
	failed=$(($failed + 1))
fi

###############
# invalid gpio
###############
raw_config=`pack_gpio $invalid_gpio $func $output $pull $drvstr`
status=`cat enable`
echo "$raw_config" > enable
status=`cat enable`
if [ $status -eq 0 ]; then
	echo "FAILED: Enabling invalid GPIO did not fail as expected"
	failed=$(($failed + 1))
fi

##################
# non-remoted gpio
##################
raw_config=`pack_gpio $nonremoted_gpio $func $output $pull $drvstr`
status=`cat enable`
echo "$raw_config" > enable
status=`cat enable`
if [ $status -eq 0 ]; then
	echo "FAILED: Enabling non-remoted GPIO did not fail as expected"
	failed=$(($failed + 1))
fi

if [ $failed -ne 0 ]; then
	exit 1;
else
	echo "ALL TESTS PASSED"
	exit 0;
fi

