#!/bin/sh

# Script to insert the usb modules dynamically and remove the modules
#
# BB5D01352.USB: yangjianping 2009-6-18 ADD USB FAKE DRIVER PID16=0x1E34, change code can be indexed by key word: 0x1E34, PID16
# BB5D02119 wenping changed begin

PID1=0x9017
PID2=0x9016
PID3=0x9014
PID4=0x9013
PID5=0x9012
PID6=0xF000
PID7=0xF009
PID8=0xD00D
PID9=0x0
PID10=0x1001
PID11=0x1002
PID12=0x1003
PID13=0x1401
PID14=0x141D
PID15=0x1E03
PID16=0x1E34
PID17=0x1446
PID18=0x142D	#for linux & MAC system
PID20=0xFFFF


usage()
{	
	echo "USAGE: ./usb_composition  <product id> <dev_mac>"
	echo "Valid USB product ids are  0x9017,0x9016,0x9014,0x9013,0x9012,0xF000,0xF009,0xD00D,0x1001,0x1401,0x141D,0x1E03,0x1E34,0x1446,0x142D"   # BB5D01352.USB: yangjianping
	echo " SN0	 PID			COMPOSITION "
	echo "  1.	0x9017	      DIAG + MODEM + NMEA + MASS_STORAGE " 
	echo "  2.	0x9016	      DIAG + MODEM + NMEA " 
	echo "  3. 	0x9014	      ETHERNET + MASS_STORAGE " 
	echo "  4.	0x9013	      DIAG + MODEM + MASS_STORAGE " 
	echo "  5.	0x9012	      DIAG + MODEM " 
	echo "  6.  0xF000	      MASS_STORAGE " 
	echo "  7.  0xF009	      ETHERNET " 
	echo "  8. 	0xD00D	      DIAG + MODEM + NMEA + MASS_STORAGE + ADB " 
	echo "  9. 	0x0           TO REMOVE MODULES THAT ARE ALREADY LOADED "
	echo "  10. 0x1001        E5832:MODEM + DIAG  "
	echo "  11. 0x1002        E5832:MODEM  "
	echo "  12. 0x1003        E5832:DIAG  "
	echo "  13. 0x1401        E5832:DIAG + CDROM + PCUI +NDIS + SD "
	echo "  14. 0x141D        E5832:DIAG + PCUI + NDIS"
	echo "  15. 0x1E03        E5832:NDIS + DIAG + PCUI + MASSSTORE"
	echo "  16. 0x1E34        E5832:FAKE|MASSSTORE"                        # BB5D01352.USB: yangjianping
	echo "  17. 0x1446        E5832:FAKE|MASSSTORE"                        # QMI: jiazhida
	echo "  18. 0x142D        E5832 for Linux:DIAG + CDROM + PCUI +NDIS + SD "                        
	exit 1
}
#Check correct number of parameters
if  [[ $# -ne 1 ] && [ $# -ne 2 ]]
then
	usage
fi


if !([[ "$1" == "$PID1" ]] || [[ "$1" == "$PID2" ]] || [[ "$1" == "$PID3" ]]	\
	|| [[ "$1" == "$PID4" ]] || [[ "$1" == "$PID5" ]] || [[ "$1" == "$PID6" ]] \
	|| [[ "$1" == "$PID7" ]] || [[ "$1" == "$PID8" ]] || [[ "$1" == "$PID9" ]] \
	|| [[ "$1" == "$PID10" ]]|| [[ "$1" == "$PID11" ]]|| [[ "$1" == "$PID12" ]]\
	|| [[ "$1" == "$PID13" ]]|| [[ "$1" == "$PID14" ]]|| [[ "$1" == "$PID15" ]]\
	|| [[ "$1" == "$PID16" ]] || [[ "$1" == "$PID17" ]] || [[ "$1" == "$PID18" ]])
	
then
	echo "Not a vaild product id"
        echo ""
	usage
	exit 1 	
fi


#remove the USB modules if loaded
remove_modules()
{
	lsmod | grep msm_hsusb >> /tmp/log
	i=`echo $?` 
	if [[ $i -eq 0 ]]
	then
		#remove g_ether_cdc_ecm
		lsmod | grep msm_hsusb | grep g_ether_cdc_ecm >> /tmp/log
		i=`echo $?` 
		if [[ $i -eq 0 ]]
		then
			rmmod g_ether_cdc_ecm
		fi
		#remove serial
		lsmod | grep msm_hsusb | grep serial >> /tmp/log
		i=`echo $?` 
			if [[ $i -eq 0 ]]
		then
			rmmod serial
		fi
		#remove adb
		lsmod | grep msm_hsusb | grep adb >> /tmp/log
		i=`echo $?` 
		if [[ $i -eq 0 ]]
		then
			rmmod adb
		fi
		#remove  mass_storage
		lsmod | grep msm_hsusb | grep mass_storage >> /tmp/log
		i=`echo $?` 
		if [[ $i -eq 0 ]]
		then
			rmmod  mass_storage
		fi
		#remove diagchar
		lsmod | grep diag | grep diagchar >> /tmp/log
		i=`echo $?`
		if [[ $i -eq 0 ]]
		then
			rmmod diagchar		
		fi          
		#remove diag
		lsmod | grep msm_hsusb | grep diag >> /tmp/log
		i=`echo $?` 
		if [[ $i -eq 0 ]]
		then
			rmmod diag
		fi	
	
		rmmod msm_hsusb
	fi
	rm -f /tmp/log
}

# remove the modules if pid = 0x0 
if [[ "$1" == "$PID9" ]]
then
	remove_modules
	exit 0
fi

#remove the USB modules if loaded
remove_modules

#Insert the modules
if [[ "$1" == "$PID1" ]]
then
	insmod msm_hsusb pid=0x9017
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		echo "Failed to insert msm_hsusb for PID=0x9017"
		exit 1
	fi
	modprobe diagchar
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then 
		modprobe diag
		i=`echo $?`
		if [[ $i -ne 0 ]]
		then 
			echo "Failed to insert diag for PID=0x9017"
			rmmod diagchar
			rmmod msm_hsusb
			exit 1
		fi
	fi
	insmod serial instances=2
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert serial for PID=0x9017"
		rmmod diag
		rmmod diagchar
		rmmod msm_hsusb
                exit 1
        fi
	insmod  mass_storage
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert mass_storage for PID=0x9017"
		rmmod diag
		rmmod diagchar
		rmmod serial
		rmmod msm_hsusb
                exit 1
        fi
	exit 0
fi

if [[ "$1" == "$PID2" ]]
then
        insmod msm_hsusb pid=0x9016
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0x9016"
                exit 1
        fi
	modprobe diagchar
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then 
		modprobe diag
		i=`echo $?`
		if [[ $i -ne 0 ]]
		then
			echo "Failed to insert diag for PID=0x9016"
			rmmod diagchar
			rmmod msm_hsusb
			exit 1
		fi
	fi
        insmod serial instances=2
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert serial for PID=0x9016"
                rmmod diag
		rmmod diagchar
                rmmod msm_hsusb
                exit 1
        fi
        exit 0
fi

if [[ "$1" == "$PID3" ]]
then
        insmod msm_hsusb pid=0x9014
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0x9014"
                exit 1
        fi
        insmod  mass_storage
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert  mass_storage for PID=0x9014"
                rmmod msm_hsusb
                exit 1
        fi
        insmod ether_cdc_ecm
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert ether_cdc_ecm for PID=0x9014"
                rmmod  mass_storage
                rmmod msm_hsusb
                exit 1
        fi
        exit 0
fi

if [[ "$1" == "$PID4" ]]
then
        insmod msm_hsusb pid=0x9013
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0x9013"
                exit 1
        fi
	modprobe diagchar
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		modprobe diag
		i=`echo $?`
		if [[ $i -ne 0 ]]
		then
			echo "Failed to insert diag for PID=0x9013"
			rmmod diagchar
			rmmod msm_hsusb
			exit 1
		fi
	fi							 
        insmod serial instances=1
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert serial for PID=0x9013"
		rmmod diag
		rmmod diagchar
                rmmod msm_hsusb
                exit 1
        fi
	insmod  mass_storage
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert  mass_storage for PID=0x9013"
                rmmod serial
                rmmod diag
		rmmod diagchar
		rmmod msm_hsusb
                exit 1
        fi
        exit 0
fi

if [[ "$1" == "$PID5" ]]
then
        insmod msm_hsusb pid=0x9012
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0x9012"
                exit 1
        fi
	modprobe diagchar
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		modprobe diag
		i=`echo $?`
		if [[ $i -ne 0 ]]
		then
			echo "Failed to insert diag for PID=0x9012"
			rmmod diagchar
			rmmod msm_hsusb
			exit 1
		fi											 
	fi
        insmod serial instances=1
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert serial with 1 instance for PID=0x9012"
                rmmod diag
		rmmod diagchar
                rmmod msm_hsusb
                exit 1
        fi
        exit 0
fi

if [[ "$1" == "$PID6" ]]
then
        insmod msm_hsusb pid=0xF000
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0xF000"
                exit 1
        fi
        insmod  mass_storage
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert diag for PID=0xF000"
                rmmod msm_hsusb
                exit 1
	fi
        exit 0
fi

if [[ "$1" == "$PID7" ]]
then
        insmod msm_hsusb pid=0xF009
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0xF009"
                exit 1
        fi
        insmod ether_cdc_ecm
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert diag for PID=0xF009"
                rmmod msm_hsusb
                exit 1
        fi
        exit 0
fi

if [[ "$1" == "$PID8" ]]
then
        insmod msm_hsusb pid=0xD00D
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert msm_hsusb for PID=0xD00D"
                exit 1
        fi
	modprobe diagchar
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		modprobe diag
		i=`echo $?`
		if [[ $i -ne 0 ]]
		then
			echo "Failed to insert diag for PID=0xD00D"
			rmmod diagchar
			rmmod msm_hsusb
			exit 1
		fi
	fi							 
        insmod serial instances=2
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert serial for PID=0xD00D"
                rmmod diag
		rmmod diagchar
                rmmod msm_hsusb
                exit 1
        fi
        insmod  mass_storage
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert  mass_storage for PID=0xD00D"
                rmmod diag
		rmmod diagchar
                rmmod serial
                rmmod msm_hsusb
                exit 1
        fi
        insmod adb 
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert  adb for PID=0xD00D"
                rmmod diag
		rmmod diagchar
                rmmod serial
		rmmod  mass_storage
                rmmod msm_hsusb
                exit 1
        fi
	exit 0
fi

#add modem+diag
if [[ "$1" == "$PID10" ]]
then
	insmod msm_hsusb pid=0x1001
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		echo "Failed to insert msm_hsusb for PID=0x1001"
		exit 1
	fi
	insmod serial instances=1
	i=`echo $?`
	if [[ $i -ne 0 ]]
    	then
                echo "Failed to insert serial for PID=0x1001"
			rmmod msm_hsusb
        	exit 1
    	fi
	insmod diag
	i=`echo $?`
	if [[ $i -ne 0 ]]
    	then
        	echo "Failed to insert diag for PID=0x1001"
			rmmod serial
			rmmod msm_hsusb
        	exit 1
    	fi
	exit 0
fi

#add modem
if [[ "$1" == "$PID11" ]]
then
	insmod msm_hsusb pid=0x1002
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		echo "Failed to insert msm_hsusb for PID=0x1002"
		exit 1
	fi
	insmod serial instances=1
	i=`echo $?`
	if [[ $i -ne 0 ]]
    	then
                echo "Failed to insert serial for PID=0x1002"
			rmmod msm_hsusb
        	exit 1
    	fi
	exit 0
fi

#add diag
if [[ "$1" == "$PID12" ]]
then
	insmod msm_hsusb pid=0x1003 cms_pid=$3 imei_sn=$5
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		echo "Failed to insert msm_hsusb for PID=0x1003"
		exit 1
	fi
        insmod  mass_storage cms_pid=$3
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert diag for PID=0x1003"
                rmmod msm_hsusb
                exit 1
	fi
	exit 0
fi

#add FAKE OR MASSSTORAGE 
#  BB5D02345 wenping for diag control mode begin
if ([[ "$1" == "$PID17" ]] || [[ "$1" == "$PID13" ]])
then
	insmod g_msm_hsusb start_pid=$1 imei_sn=$5 diag_val=$4
	i=`echo $?`
	if [[ $i -ne 0 ]]
	then
		echo "Failed to insert msm_hsusb for PID=$1"
		exit 1
	fi
#if have  diag ,then  inmode diag &diag char.  else don't inmode diag .

  if ([[ "$4" == 0 ]] || [[ "$4" == 2 ]])
  then	
  #insert DIAG
       insmod diag
       i=`echo $?`
       if [[ $i -ne 0 ]]
       then
		echo "Failed to insert diag for PID=$1"
		rmmod g_msm_hsusb
                exit 1
       fi
  #insert DIAGCHAR
       insmod diagchar
       i=`echo $?`
       if [[ $i -ne 0 ]]
       then
		echo "Failed to insert diagchar"
		        rmmod diag
	        	rmmod g_msm_hsusb
                exit 1
       fi
    fi      
#insert mass_storage 
        insmod  mass_storage pwr_off_chg=$3 
	i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert diag for PID=$1"
                rmmod g_msm_hsusb
                exit 1
	fi       
#insert nmea
  if ([[ "$4" == 0 ]] || [[ "$4" == 1 ]])
  then	
      insmod serial instances=1
      i=`echo $?`
       if [[ $i -ne 0 ]]
       then
		echo "Failed to insert nmea"		
		rmmod mass_storage
		rmmod diagchar		
              rmmod diag		
              rmmod g_msm_hsusb		
                exit 1
	fi
  fi
#insert qmi_tty
      insmod qmi_tty
      i=`echo $?`
	if [[ $i -ne 0 ]]
        then
                echo "Failed to insert qmi_tty for PID=$1"
#                rmmod g_ether_cdc_ecm                
                rmmod serial
                rmmod mass_storage
		rmmod diagchar                
                rmmod diag                
                rmmod g_msm_hsusb
                exit 1
        fi        

       
       	
	exit 0
fi
# BB5D02119 wenping changed end
#  BB5D02345 wenping for diag control mode begin

