################################################################################
# Copyright (C) 2008 by Qualcomm
################################################################################

# Make version check.  3.81 or later is needed for proper SRCLIST expansion.
ifeq ($(shell if [ $(subst .,,$(MAKE_VERSION)) -lt 381 ]; then echo bad; fi;),bad)
    $(error GNU Make $(MAKE_VERSION), need 3.81.  Add /pkg/gnu/make/3.81/bin to your path)
endif

################################################################################
# Touch <package-name>.<step>-start file to calculate processing times
################################################################################

ifndef PTXDIST_QUIET
targetinfo =                                    \
    target="$(strip $(@))";                         \
    touch "$${target}-start";                         \
    target="target: $${target\#\#*/}";                  \
    echo -e "\n$${target//?/-}\n$${target}\n$${target//?/-}\n";     \
    echo $@ : $^ | sed                          \
        -e "s@$(SRCDIR)@@g"                     \
        -e "s@$(STATEDIR)@@g"                       \
        -e "s@$(RULESDIR)@@g"                       \
        -e "s@$(PROJECTRULESDIR)@@g"                    \
        -e "s@$(PTXDIST_PLATFORMCONFIGDIR)@@g"              \
        -e "s@$(PTXDIST_WORKSPACE)@@g"                  \
        -e "s@$(PTXDIST_TOPDIR)@@g"                     \
        -e "s@/@@g" >> $(DEP_OUTPUT);
endif

################################################################################
# Defines
################################################################################

QCT_APPS_MBN_NANE  := apps.mbn
QCT_AMSS_APPS_MBN  := $(IMAGEDIR)/$(QCT_APPS_MBN_NANE)

# calculate size of newly created initrd.gz
BOOTWEDGE_RAMDISK_IMAGE := $(IMAGEDIR)/initrd.gz
BOOTWEDGE_RD_SIZE  = $(if $(wildcard $(BOOTWEDGE_RAMDISK_IMAGE)),$(shell printf 0x%x `stat -c%s $(BOOTWEDGE_RAMDISK_IMAGE)`),0x0)

NAND_ROOTFS_IMAGE = $(IMAGEDIR)/nandimage

BLD_PRD_TXT := $(PTXDIST_WORKSPACE)/../../BuildProducts.txt

################################################################################
# QCT image generation performed after package processing is complete
################################################################################
.PHONY: qcom-post-image-step

# Add dependency to 'images' target. qcom-post-image-step is invoked after 
# other rules/other/images.make targets
$(STATEDIR)/images: qcom-post-image-step

# create apps.mbn, install elf and mbn into amss tree
qcom-post-image-step: $(BOOTWEDGE_RAMDISK_IMAGE)
qcom-post-image-step: $(QCT_AMSS_APPS_MBN)

ifeq "$(PTXCONF_IMAGE_YAFFS2)" "y"
qcom-post-image-step: $(NAND_ROOTFS_IMAGE)
endif # image_yaffs2=y

################################################################################
# Generate apps mbn & elf
################################################################################
.PHONY: $(QCT_AMSS_APPS_MBN)

$(QCT_AMSS_APPS_MBN):
# Build the bootwedge now that the initrd.gz size is known
	@echo "Building $(BOOTWEDGE)"
	$(BOOTWEDGE_ENV) $(BOOTWEDGE_PATH) $(MAKE) $(BOOTWEDGE_MAKEVARS) INITRD_SIZE=$(BOOTWEDGE_RD_SIZE)
# generate an elf formatted image containing bootwedge, kernel, initramfs
	@echo "Creating $(notdir $@)"
	cd $(IMAGEDIR) && $(PTXCONF_SYSROOT_HOST)/bin/elfed $(PTXCONF_BOOTWEDGE_PADDR) $(BOOTWEDGE_DIR)/wedge $(KERNEL_OBJDIR)/vmlinux.stripped $(BOOTWEDGE_RAMDISK_IMAGE)
# copy kernel to images dir to ease path selection from .cmm scripts
	@cp -f $(KERNEL_OBJDIR)/vmlinux $(IMAGEDIR)
	@cp -f $(KERNEL_OBJDIR)/vmlinux.stripped $(IMAGEDIR)
# CRM build script requires that BuildProducts.txt contain an image name
	-rm -f $(BLD_PRD_TXT)
	@echo `cd $(PTXDIST_WORKSPACE)/../.. && find LINUX/project/platform-$(PTXCONF_PLATFORM)/images -name apps.mbn` >  $(BLD_PRD_TXT)

################################################################################
# create the YAFFS2 image
################################################################################
$(IMAGEDIR)/root.yaffs2: $(STATEDIR)/image_working_dir
	@echo -n "Creating root.yaffs2 from working dir..."
	@cd $(WORKDIR);                                                 \
	(awk -F: $(DOPERMISSIONS) $(IMAGEDIR)/permissions &&            \
	(                                                               \
		echo -n "$(PTXCONF_SYSROOT_HOST)/bin/mkyaffs2image ";   \
		echo -n "$(WORKDIR) ";                          \
		echo -n "$@" )                  \
	) | $(FAKEROOT) -- > /dev/null 2>&1
	@echo "done."

$(NAND_ROOTFS_IMAGE): $(IMAGEDIR)/root.yaffs2
	@echo "Run mknandimage on the image file output from mkyaffs2image"
	@$(PTXCONF_SYSROOT_HOST)/bin/mknandimage $(PTXCONF_IMAGE_YAFFS2_BBM_OFFSET) $(PTXCONF_IMAGE_YAFFS2_BIT_WIDTH) < $(IMAGEDIR)/root.yaffs2 > $(IMAGEDIR)/nandimage

